﻿﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.TBI;

public partial class TBI_Instruments_FunctionalMobilityAssessment : BaseTBIInstrumentPage {

    #region Variable Declarations

    PatientManager patientManager = new PatientManager();
    InstrumentType instrumentType = new InstrumentType();

    protected int _surveyTypeId = 15; // Database Id for Functional Mobility Assessment

    protected List<int> q1ChoiceIds = new List<int>(); // just for this instrument.

    SURVEY _survey;   // the root object in the Survey set
    SURVEY_RESULT _surveyResult; // the object representing the actual data in the Survey

    List<STD_QUESTION> QuestionList = new List<STD_QUESTION>();

    #endregion

    /// <summary>
    /// Default Constructor
    /// </summary>
    public TBI_Instruments_FunctionalMobilityAssessment() {
        //Manager Save Notification Event Handler
        patientManager.BOSaveSuccess += manager_BOSaveSuccess;
    }

    #region Protected Methods

    /// <summary>
    /// Page Load method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e) {
        TBIServiceTO _tbiServiceTo;

        // confirm instrument is active - if not, a redirect will occur.
        this.CheckActiveStatus(_surveyTypeId);

        try {
            _tbiServiceTo = (TBIServiceTO)Session["TBIServiceTO"];
            _tbiInstrumentManager.TbiServiceTo = _tbiServiceTo;

            string instTitle = _tbiInstrumentManager.GetInstrumentByID(_surveyTypeId).Name;

            LabelBreadCrumb.Text = "TBI Instruments > Instrument Associations > " + instTitle;
            LabelInstrumentTitle.Text = instTitle;
            Page.Title = "TBI Instruments - " + instTitle;

            if (_tbiInstrumentManager.TbiServiceTo != null) {
                Label labelUser = (Label)Master.FindControl("LabelCurrentUser");
                labelUser.Text = _tbiInstrumentManager.TbiServiceTo.UserName;

                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }
        } catch (Exception ex) {
            //Redirect to message that session is ended and need to log back in from CPRS.
            Exception _tbilogerror = HandleTBIException(ex, "SatisfactionWithLifeScale.aspx", "100027", "SESSIOENDED");
            Response.Redirect("Warning.aspx?err=2", false);
        }

        if (!IsPostBack) {
            if (instrumentType.Status != null) {
                _tbiInstrumentManager.TbiServiceTo.SelectedInstrumentType = instrumentType;
            }

            string _ssn = "";
            if (_tbiInstrumentManager.TbiServiceTo != null) {
                _ssn = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }

            if (!string.IsNullOrEmpty(_ssn)) {
                string patICN = patientManager.GetPatientICNBySSN(_ssn);

                if (!string.IsNullOrEmpty(patICN)) {
                    ViewState["PatientICN"] = patICN;
                    RefreshSurvey(patICN);
                } else {
                    throw new ArgumentException("Patient with this SSN does not exist.");
                }

                if (_survey != null && !_survey.isNew) {
                    SetUpSurvey(_survey);
                }
                string referrer = "";
                if (Request.UrlReferrer != null) {
                    referrer = Request.UrlReferrer.PathAndQuery;
                }
                ViewState["referrer"] = referrer;
            }

            //Check if the request is to start a new item 
            //a new item page request is signified when the querystring parameter id equals zero or is not present
            if (_tbiInstrumentManager.TbiServiceTo == null)
            {
                Logging.WriteLogToFile("FunctionalMobilityAssessment.Page_Load(): WarningRedirect: _tbiInstrumentManager.TbiServiceTo == null");
                //Redirect to message that session is ended and need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            } else {
                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN.Trim().Length == 9 ? String.Format("{0:000-00-0000}", int.Parse(_tbiInstrumentManager.TbiServiceTo.PatientSSN)) : _tbiInstrumentManager.TbiServiceTo.PatientSSN;
                //LabelFacility.Text = _tbiInstrumentManager.TbiServiceTo.InstitutionName;
            }
        }
        QuestionList = GetAllQuestionsForSurvey();

        #region Set Label Text and max dates

        LabelDate1.Text = GetQuestionText("D");

        // Set control values for Diagnosis Question
        ctrlDiagnosis.QuestionText = GetQuestionText("DX");
        ctrlDiagnosis.SurveyTypeCode = HiddenSurveyType.Value;

        // Set control values for Tech Dev Question
        ctrlTechDev.QuestionText = GetQuestionText("1");
        ctrlTechDev.SurveyTypeCode = HiddenSurveyType.Value;

        LabelForQ2.Text = GetQuestionText("2");
        LabelForQ2Comment.Text = GetQuestionText("2Comment");

        LabelForQ3.Text = GetQuestionText("3");
        LabelForQ3Comment.Text = GetQuestionText("3Comment");

        LabelForQ4.Text = GetQuestionText("4");
        LabelForQ4Comment.Text = GetQuestionText("4Comment");

        LabelForQ5.Text = GetQuestionText("5");
        LabelForQ5Comment.Text = GetQuestionText("5Comment");

        LabelForQ6.Text = GetQuestionText("6");
        LabelForQ6Comment.Text = GetQuestionText("6Comment");

        LabelForQ7.Text = GetQuestionText("7");
        LabelForQ7Comment.Text = GetQuestionText("7Comment");

        LabelForQ8.Text = GetQuestionText("8");
        LabelForQ8Comment.Text = GetQuestionText("8Comment");

        LabelForQ9.Text = GetQuestionText("9");
        LabelForQ9Comment.Text = GetQuestionText("9Comment");

        LabelForQ10.Text = GetQuestionText("10");
        LabelForQ10Comment.Text = GetQuestionText("10Comment");

        LabelForQ11.Text = GetQuestionText("11");
        LabelForQ11Comment.Text = GetQuestionText("11Comment");

        dpDate1.MaxDate = DateTime.Now;

        #endregion

    }

    protected override void LoadViewState(object savedState) {
        base.LoadViewState(savedState);
        string surveyXml = (string)ViewState["surveyXml"];
        _survey = SerializerUtil.Deserialize<SURVEY>(surveyXml);
    }

    protected override object SaveViewState() {
        ViewState["surveyXml"] = _survey.Serialize();
        return base.SaveViewState();
    }

    /// <summary>
    /// Returns the Question_Text of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected string GetQuestionText(string questionId) {
        foreach (STD_QUESTION ques in QuestionList) {
            if (ques.QUESTION_NUMBER.ToLower() == questionId.ToLower()) {
                return ques.QUESTION_TEXT;
            }
        }

        return "";
    }

    /// <summary>
    /// Returns the Question_Id of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected int GetQuestionId(string questionNumber) {
        foreach (STD_QUESTION ques in QuestionList) {
            if (ques.QUESTION_NUMBER.ToLower() == questionNumber.ToLower()) {
                return ques.ID;
            }
        }

        return 0;
    }

    /// <summary>
    /// Save Draft button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public void btnSaveSurveyDraft_Click(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);
        SetUpSurvey(_survey);
    }

    /// <summary>
    /// Save and Prepare Note button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnSubmit_Click(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSubmitSurvey.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);

        if (status > 0) {
            Session["survey"] = _survey;
            Response.Redirect("InstrumentSummaryNote.aspx", false);
        } else {
            SetUpSurvey(_survey); //Follow Save Draft logic 
        }
    }

    /// <summary>
    /// Inactive timer firing.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void InActiveTimer_Tick(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";

        Page.Validate();

        if (Page.IsValid) {
            SaveCompleteSurvey(ref status);
            //SetUpSurvey(_survey);
            Response.Redirect("FunctionalMobilityAssessment.aspx", false);
        } else {
            SaveFailMessage = "Auto-Save Validation Failed. Please complete required fields in the survey and press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        }
    }

    /// <summary>
    /// Cancel button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCancel_Click(object sender, EventArgs e) {
        ViewState["surveyXml"] = null;
        ViewState["PatientICN"] = null;
        ViewState["referrer"] = null;
        Session.Remove("survey");
        Response.Redirect("Instrument.aspx", false);
        //this.ClientScript.RegisterStartupScript(typeof(Page), "closePage", "<script type='text/javascript'> window.open('','_self','');window.close(); </script>");
    }

    /// <summary>
    /// fired on binding of control to data.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    //protected void cblQ1_DataBound(object sender, EventArgs e) {
    //    // bind question 1 choices
    //    foreach (int i in q1ChoiceIds) {
    //        foreach (ListItem li in cblQ1.Items) {
    //            if (li.Value == i.ToString()) {
    //                li.Selected = true;
    //                break;
    //            }
    //        }
    //    }
    //}

    #endregion

    #region Private Methods

    /// <summary>
    /// Get all questions for this survey
    /// </summary>
    /// <returns></returns>
    private List<STD_QUESTION> GetAllQuestionsForSurvey() {
        return _tbiInstrumentManager.SelectAllQuestions(_surveyTypeId).ToList();
    }

    /// <summary>
    /// Refresh Survey
    /// </summary>
    /// <param name="patientICN"></param>
    private void RefreshSurvey(string patientICN) {
        var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

        if (surveys.Count > 0) {
            _survey = surveys.First();
        } else {
            _survey = new SURVEY();
        }

        _survey.SetAsChangeTrackingRoot();
    }

    /// <summary>
    /// Save Complete Survey
    /// </summary>
    /// <param name="status"></param>
    private void SaveCompleteSurvey(ref Int32 status) {
        Page.Validate();
        if (Page.IsValid) {
            string patientICN = String.Empty;

            status = 0;

            //Retrieve patient's ICN from the session variable
            if (ViewState["PatientICN"] != null) {
                patientICN = ViewState["PatientICN"].ToString();
                if (String.IsNullOrEmpty(patientICN.Trim()))
                {
                    Logging.WriteLogToFile("FunctionalMobilityAssessment.SaveCompletedSurvey(): WarningRedirect: IsNullOrEmpty(patientICN.Trim()");
                    //Lost state. Redirect to message that need to log back in from CPRS.
                    Response.Redirect("Warning.aspx?err=2", false);
                }
            }
            else
            {
                Logging.WriteLogToFile("FunctionalMobilityAssessment.SaveCompletedSurvey(): WarningRedirect: ViewState[PatientICN] != null");
                //Lost state. Redirect to message that need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }

            if (_survey != null) {
                if (_survey.isNew) {
                    //Ensure that a concurrent session hasn't already inserted a record for this survey
                    var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

                    if (surveys.Count > 0)
                    {
                        Logging.WriteLogToFile("FunctionalMobilityAssessment.SaveCompletedSurvey(): WarningRedirect: surveys.Count > 0");
                        //A concurrent session has already inserted a record for this survey and it is not new anymore.
                        Response.Redirect("Warning.aspx?err=9", false);
                    } else {
                        _survey.SURVEY_STATUS = "inprocess";

                        _survey.SetAsInsertOnSubmit();

                        _survey.patient_ICN = patientICN;

                        _survey.STD_SURVEY_TYPE_ID = _surveyTypeId;

                        // TODO: change to datepicker
                        _survey.SURVEY_DATE = DateTime.Today;

                        //Save survey if it's new before adding any responses
                        SuppressSaveMessage = true;
                        _tbiInstrumentManager.Update(_survey);
                        SuppressSaveMessage = false;

                        RefreshSurvey(patientICN);
                    }
                }

                SaveSurvey(ref _survey);

                _survey.SetAsUpdateOnSubmit();
                status = _tbiInstrumentManager.Update(_survey);

                (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);

                RefreshSurvey(patientICN);
                SuppressSaveMessage = false;
            }
        }
    }

    /// <summary>
    /// Save survey.
    /// </summary>
    /// <param name="survey"></param>
    private void SaveSurvey(ref SURVEY survey) {
        List<STD_QUESTION> questions = GetAllQuestionsForSurvey();

        //Delete all the survey results from database if there are any.
        SURVEY surveyWithResults = _tbiInstrumentManager.GetSurveyWithAllAnswers(survey.SURVEYS_ID);
        _tbiInstrumentManager.DeleteAllSurveyResults(surveyWithResults);

        // Question DX
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question DX choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(ctrlDiagnosis.GetSelectedId());
            _surveyResult.RESULT_TEXT = ctrlDiagnosis.GetSelectedResultText();
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question DX");
        }

        // Question D
        _surveyResult = new SURVEY_RESULT();

        if (dpDate1.SelectedDate != null) {
            STD_QUESTION questionD = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "D");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question D choice
                _surveyResult.STD_QUESTION_CHOICE_ID = questionD.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ((DateTime)dpDate1.SelectedDate).ToShortDateString();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question D");
            }
        }

        // Question 1
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question TechDev choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(ctrlTechDev.GetSelectedId());
            _surveyResult.RESULT_TEXT = ctrlTechDev.GetSelectedResultText();
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 1");
        }

        // OLD CODE - Multiple select before control.
        //if (getNumberOfSelectionsQ1() > 0) {
        //    foreach (ListItem li in cblQ1.Items) {
        //        if (li.Selected) {
        //            _surveyResult = new SURVEY_RESULT();

        //            try {
        //                // Add Question 1 choice
        //                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(li.Value);
        //                _surveyResult.RESULT_TEXT = li.Text;
        //                survey.SURVEY_RESULTs.Add(_surveyResult);
        //            } catch (Exception surveyEx) {
        //                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 1");
        //            }
        //        }
        //    }
        //}

        // Question 2
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 2 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ2.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ2.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 2");
        }

        // Question 2Comment
        if (!String.IsNullOrEmpty(txtCommentsQ2.Text)) {
            STD_QUESTION question2Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "2Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 2Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question2Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ2.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 2 Comment");
            }
        }

        // Question 2 Rating
        if (ddlPriority1.SelectedIndex > 0) {
            STD_QUESTION question2Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "2R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question2Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority1.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 2 Rating");
            }
        }

        // Question 3
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 3 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ3.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ3.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 3");
        }

        // Question 3Comment
        if (!String.IsNullOrEmpty(txtCommentsQ3.Text)) {
            STD_QUESTION question3Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "3Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 3Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question3Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ3.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 3 Comment");
            }
        }

        // Question 3 Rating
        if (ddlPriority2.SelectedIndex > 0) {
            STD_QUESTION question3Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "3R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 3 Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question3Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority2.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 3 Rating");
            }
        }

        // Question 4
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 4 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ4.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ4.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 4");
        }

        // Question 4Comment
        if (!String.IsNullOrEmpty(txtCommentsQ4.Text)) {
            STD_QUESTION question4Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "4Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 4Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question4Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ4.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 4 Comment");
            }
        }

        // Question 4 Rating
        if (ddlPriority3.SelectedIndex > 0) {
            STD_QUESTION question4Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "4R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 4 Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question4Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority3.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 4 Rating");
            }
        }

        // Question 5
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 5 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ5.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ5.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 5");
        }

        // Question 5Comment
        if (!String.IsNullOrEmpty(txtCommentsQ5.Text)) {
            STD_QUESTION question5Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "5Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 2Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question5Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ5.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 5 Comment");
            }
        }

        // Question 5 Rating
        if (ddlPriority4.SelectedIndex > 0) {
            STD_QUESTION question5Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "5R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 5 Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question5Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority4.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 5 Rating");
            }
        }

        // Question 6
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 6 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ6.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ6.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 6");
        }

        // Question 6Comment
        if (!String.IsNullOrEmpty(txtCommentsQ6.Text)) {
            STD_QUESTION question6Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "6Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 6Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question6Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ6.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 6 Comment");
            }
        }

        // Question 6 Rating
        if (ddlPriority5.SelectedIndex > 0) {
            STD_QUESTION question6Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "6R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 6 Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question6Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority5.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 6 Rating");
            }
        }

        // Question 7
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 7 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ7.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ7.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 7");
        }

        // Question 7Comment
        if (!String.IsNullOrEmpty(txtCommentsQ7.Text)) {
            STD_QUESTION question7Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "7Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 7Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question7Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ7.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 7 Comment");
            }
        }

        // Question 7 Rating
        if (ddlPriority6.SelectedIndex > 0) {
            STD_QUESTION question7Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "7R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 7 Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question7Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority6.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 7 Rating");
            }
        }

        // Question 8
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 8 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ8.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ8.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 8");
        }

        // Question 8Comment
        if (!String.IsNullOrEmpty(txtCommentsQ8.Text)) {
            STD_QUESTION question8Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "8Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 8Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question8Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ8.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 8 Comment");
            }
        }

        // Question 8 Rating
        if (ddlPriority7.SelectedIndex > 0) {
            STD_QUESTION question8Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "8R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 8 Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question8Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority7.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 8 Rating");
            }
        }

        // Question 9
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 9 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ9.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ9.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 9");
        }

        // Question 9Comment
        if (!String.IsNullOrEmpty(txtCommentsQ9.Text)) {
            STD_QUESTION question9Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "9Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 9Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question9Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ9.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 9 Comment");
            }
        }

        // Question 9 Rating
        if (ddlPriority8.SelectedIndex > 0) {
            STD_QUESTION question9Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "9R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 9 Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question9Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority8.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 9 Rating");
            }
        }

        // Question 10
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 10 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ10.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ10.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 10");
        }

        // Question 10Comment
        if (!String.IsNullOrEmpty(txtCommentsQ10.Text)) {
            STD_QUESTION question10Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "10Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 10Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question10Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ10.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 10 Comment");
            }
        }

        // Question 10 Rating
        if (ddlPriority9.SelectedIndex > 0) {
            STD_QUESTION question10Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "10R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 10 Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question10Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority9.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 10 Rating");
            }
        }

        // Question 11
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 11 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ11.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ11.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 11");
        }

        // Question 11Comment
        if (!String.IsNullOrEmpty(txtCommentsQ11.Text)) {
            STD_QUESTION question11Comment = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "11Comment");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 11Comment choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question11Comment.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ11.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 11 Comment");
            }
        }

        // Question 10 Rating
        if (ddlPriority10.SelectedIndex > 0) {
            STD_QUESTION question11Rating = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "11R");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 11 Rank choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question11Rating.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ddlPriority10.SelectedValue;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "FunctionalMobilityAssessment.aspx", "100027", "SaveSurvey() - Question 11 Rating");
            }
        }
    }// END SaveSurvey

    /// <summary>
    /// Initialize/setup controls
    /// </summary>
    /// <param name="survey"></param>
    private void SetUpSurvey(SURVEY survey) {
        try {
            foreach (var sResult in survey.SURVEY_RESULTs) {
                switch (sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER) {
                    case "DX":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            ctrlDiagnosis.SelectedChoiceId = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();

                            if (sResult.STD_QUESTION_CHOICE.CHOICE_TEXT.ToLower().Contains("other")) {
                                ctrlDiagnosis.SetOtherDescriptionText(sResult.RESULT_TEXT);
                            }
                        } else {
                            ctrlDiagnosis.SelectedChoiceId = null;
                        }
                        break;
                    case "D":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            DateTime temp;
                            if (DateTime.TryParse(sResult.RESULT_TEXT, out temp)) {
                                dpDate1.SelectedDate = temp;
                            }
                        }

                        break;
                    //case "1":
                    //    // add to list and bind after foreach statement.
                    //    if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                    //        q1ChoiceIds.Add(sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID);
                    //    }

                    //    break;
                    case "1":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            ctrlTechDev.SelectedChoiceId = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();

                            if (sResult.STD_QUESTION_CHOICE.CHOICE_TEXT.ToLower().Contains("other")) {
                                ctrlTechDev.SetOtherDescriptionText(sResult.RESULT_TEXT);
                            }
                        } else {
                            ctrlTechDev.SelectedChoiceId = null;
                        }
                        break;
                    case "2":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ2.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ2.SelectedIndex = -1;
                        }

                        break;
                    case "2Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ2.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "2R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority1.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    case "3":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ3.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ3.SelectedIndex = -1;
                        }

                        break;
                    case "3Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ3.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "3R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority2.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    case "4":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ4.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ4.SelectedIndex = -1;
                        }

                        break;
                    case "4Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ4.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "4R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority3.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    case "5":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ5.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ5.SelectedIndex = -1;
                        }

                        break;
                    case "5Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ5.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "5R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority4.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    case "6":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ6.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ6.SelectedIndex = -1;
                        }

                        break;
                    case "6Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ6.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "6R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority5.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    case "7":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ7.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ7.SelectedIndex = -1;
                        }

                        break;
                    case "7Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ7.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "7R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority6.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    case "8":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ8.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ8.SelectedIndex = -1;
                        }

                        break;
                    case "8Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ8.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "8R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority7.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    case "9":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ9.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ9.SelectedIndex = -1;
                        }

                        break;
                    case "9Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ9.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "9R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority8.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    case "10":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ10.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ10.SelectedIndex = -1;
                        }

                        break;
                    case "10Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ10.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "10R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority9.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    case "11":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ11.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ11.SelectedIndex = -1;
                        }

                        break;
                    case "11Comment":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtCommentsQ11.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "11R":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            ddlPriority10.SelectedValue = sResult.RESULT_TEXT;
                        }

                        break;
                    default:
                        break;
                }
            }

            // bind question 1 choices
            //foreach (int i in q1ChoiceIds) {
            //    foreach (ListItem li in cblQ1.Items) {
            //        if (li.Value == i.ToString()) {
            //            li.Selected = true;
            //            break;
            //        }
            //    }
            //}

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "FunctionalMobilityAssessment.aspx", "100027", "SetUpSurvey()");
        }
    }// END SetUpSurvey

    /// <summary>
    /// Retrieve instrument title
    /// </summary>
    /// <param name="surveyID"></param>
    /// <returns></returns>
    private string GetInstrumentTitle(int surveyID) {
        string ret = String.Empty;

        try {
            List<string> instrumentTitle = _tbiInstrumentManager.GetInstrumentByID(surveyID).Title;
            if (instrumentTitle.Count > 0) {
                foreach (string titleLine in instrumentTitle) {
                    if (titleLine.Contains("PhD")) {
                        ret = ret + "<small>" + titleLine + "</small>" + "<br \\>";
                    } else {
                        ret = ret + titleLine + "<br \\>";
                    }
                }
            }

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "FunctionalMobilityAssessment.aspx", "100027", "GetInstrumentTitle()");
            // Ignore error
        }

        return ret;
    }

    /// <summary>
    /// Returns the number of selected entries for Question 1
    /// </summary>
    /// <returns></returns>
    //private int getNumberOfSelectionsQ1() {
    //    int numSelections = 0;

    //    foreach (ListItem li in cblQ1.Items) {
    //        if (li.Selected) {
    //            numSelections++;
    //        }
    //    }

    //    return numSelections;
    //}

    #endregion
}