﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.TBI;
using VeteransAffairs.Registries.BusinessManager.MDWS; // Add 29-Oct-2010 per JH
using System.Text;
using System.Runtime.InteropServices;
using Telerik.Web.UI;


public partial class TBI_InstrumentMpaiEntry : BaseTBIInstrumentPage //System.Web.UI.Page 
{

    #region Variable Declarations

    private const string FailedFormValuesName = "MPAIFailedFormValues";
    private const string DeferredMessageKeyName = "MPAIDeferredMessageKey";
    private const int _surveyTypeId = 4; // Database Id for MPAI 

    PatientManager patientManager = new PatientManager();
    InstrumentType instrumentType = new InstrumentType();

    SURVEY _survey;   // the root object in the Survey set

    #endregion

    #region Public Methods

    public TBI_InstrumentMpaiEntry()
    {
        //Manager Save Notification Event Handler
        patientManager.BOSaveSuccess += manager_BOSaveSuccess;
    }

    #endregion

    #region Protected Methods

    private InstrumentType selectedInstrumentType;

    protected void Page_Init(object sender, EventArgs e)
    {
        selectedInstrumentType = _tbiInstrumentManager.GetInstrumentByID(_surveyTypeId);
        VeteransAffairs.Registries.QuestionChoiceRetriever.Reset(_tbiInstrumentManager, _surveyTypeId);
    }

    protected void Page_Unload(object sender, EventArgs e)
    {
        VeteransAffairs.Registries.QuestionChoiceRetriever.Reset(null, -1);
    }

    bool preventNotification = false;
    protected void Page_PreRenderComplete(object sender, EventArgs e)
    {
        Dictionary<string, string> cache;
        if (!Page.IsPostBack && Helpers.SetupPageFromCache(this, FailedFormValuesName, out cache))
        {
            SuppressSaveMessage = false;
            SaveFailMessage = "Unable to save the survey.\r\n\r\nThere may be network issues preventing you from saving.\r\n\r\nPlease try to submit your changes again.";
            RenderStartUpMessage(false);
        }
        else if (Page.IsPostBack && !preventNotification)
        {
            DeferredMessage = "Save was Successful!";
        }
        else if (DeferredMessage != null)
        {
            SuppressSaveMessage = false;
            SaveSuccessMessage = DeferredMessage;
            RenderStartUpMessage(true);
            DeferredMessage = null;
        }
    }

    private static string DeferredMessage
    {
        get { return HttpContext.Current.Session[DeferredMessageKeyName] as string; }
        set { HttpContext.Current.Session[DeferredMessageKeyName] = value; }
    }

    /// <summary>
    /// In addition to the normal actions taken in Page_Load,
    /// instantiate the MDWS Web Service (T)ransfer (O)bject.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            this.CheckActiveStatus(_surveyTypeId);

            LabelInstrumentTitle.Text = GetInstrumentTitle(_surveyTypeId);
            SetQuestionLabels();

            //Confirm instrument is active - if not, a redirect will occur.
            _tbiInstrumentManager.TbiServiceTo = base.GetTBIServiceTo();
            if (_tbiInstrumentManager.TbiServiceTo == null)
            {
                Logging.WriteLogToFile("InstrumentMpaiEntry.Page_Load(): _tbiInstrumentManager.TbiServiceTo == null");
                //Redirect to message that session is ended and need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }

            LabelBreadCrumb.Text = "TBI Instruments > Instrument Associations > " + selectedInstrumentType.Name;
            if (!IsPostBack)
            {
                string _ssn = string.Empty;

                base.AddtoViewStateTBIServiceTo(_tbiInstrumentManager.TbiServiceTo);

                if (instrumentType.Status != null)
                {
                    _tbiInstrumentManager.TbiServiceTo.SelectedInstrumentType = instrumentType;
                }

                _ssn = _tbiInstrumentManager.TbiServiceTo.PatientSSN;

                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN.Trim().Length == 9 ? String.Format("{0:000-00-0000}", int.Parse(_tbiInstrumentManager.TbiServiceTo.PatientSSN)) : _tbiInstrumentManager.TbiServiceTo.PatientSSN;

                Label labelUser = (Label)Master.FindControl("LabelCurrentUser");
                if (labelUser != null)
                {
                    labelUser.Text = _tbiInstrumentManager.TbiServiceTo.UserName;
                }

                if (!string.IsNullOrEmpty(_ssn))
                {
                    string patICN = patientManager.GetPatientICNBySSN(_ssn);

                    if (!string.IsNullOrEmpty(patICN))
                    {
                        ViewState["PatientICN"] = patICN;
                        RefreshSurvey(patICN);
                    }
                    else
                    {
                        throw new ArgumentException("Patient with this SSN does not exist.");
                    }

                    if (_survey != null && !_survey.isNew)
                    {
                        SetUpSurvey(_survey);
                    }
                    string referrer = string.Empty;
                    if (Request.UrlReferrer != null)
                    {
                        referrer = Request.UrlReferrer.PathAndQuery;
                    }
                    ViewState["referrer"] = referrer;
                }

                if (_survey != null)
                {
                    if (LabelNoteTypeID.Text == "")
                    {
                        List<STD_QUESTION_CHOICE> choices = VeteransAffairs.Registries.QuestionChoiceRetriever.GetQuestionChoices("-1");
                        if (_tbiInstrumentManager.IsInitialNoteType(_tbiInstrumentManager.TbiServiceTo.PatientICN, _surveyTypeId))
                        {
                            LabelNoteType.Text = choices[0].CHOICE_TEXT;
                            LabelNoteTypeID.Text = choices[0].STD_QUESTION_CHOICE_ID.ToString();
                        }
                        else
                        {
                            LabelNoteType.Text = choices[1].CHOICE_TEXT;
                            LabelNoteTypeID.Text = choices[1].STD_QUESTION_CHOICE_ID.ToString();
                        }
                    }
                }
            }
        }
        catch (Exception ex2)
        {
            //Redirect to message that session is ended and need to log back in from CPRS.
            Exception _tbilogerror = HandleTBIException(ex2, "InstrumentMpaiEntry.aspx", "100027", "Page_Load()");
            Response.Redirect("Warning.aspx?err=2");
        }
    }

    protected override void LoadViewState(object savedState)
    {
        base.LoadViewState(savedState);
        string surveyXml = (string)ViewState["surveyXml"];
        _survey = SerializerUtil.Deserialize<SURVEY>(surveyXml);
    }

    protected override object SaveViewState()
    {
        ViewState["surveyXml"] = _survey.Serialize();
        return base.SaveViewState();
    }

    public void btnSaveSurveyDraft_Click(object sender, EventArgs e)
    {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);
        Response.Redirect(Request.Url.PathAndQuery, false);
    }

    protected void btnSubmit_Click(object sender, EventArgs e)
    {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSubmitSurvey.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);

        if (status > 0)
        {
            preventNotification = true;
            Session["survey"] = _survey;
            Session["rawScore"] = hiddenRawScore.Value.ToString();
            Session["tScore"] = hiddenTScore.Value.ToString();
            Response.Redirect("InstrumentSummaryNote.aspx", false);
        }
        else
        {
            Response.Redirect(Request.Url.PathAndQuery, false);
        }
    }

    /// <summary>
    /// Cancel button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCancel_Click(object sender, EventArgs e)
    {
        ViewState["surveyXml"] = null;
        ViewState["PatientICN"] = null;
        ViewState["referrer"] = null;
        Session.Remove("survey");
        Response.Redirect("Instrument.aspx", false);
        //this.ClientScript.RegisterStartupScript(typeof(Page), "closePage", "<script type='text/javascript'> window.open('','_self','');window.close(); </script>");
    }

    /// <summary>
    /// Inactive timer firing.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void InActiveTimer_Tick(object sender, EventArgs e)
    {
        preventNotification = true;
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);
        Response.Redirect(Request.Url.PathAndQuery, false);
    }

    #endregion Protected Methods

    #region Private Methods

    private void RefreshSurvey(string patientICN)
    {
        var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();
        if (surveys.Count > 0)
        {
            _survey = surveys.First();
        }
        else
        {
            _survey = new SURVEY();
        }
        _survey.SetAsChangeTrackingRoot();
    }

    private void SaveCompleteSurvey(ref Int32 status)
    {
        string patientICN = String.Empty;
        status = 0;

        try
        {
            //Retrieve patient's ICN from the session variable
            if (ViewState["PatientICN"] != null)
            {
                patientICN = ViewState["PatientICN"].ToString();
                if (String.IsNullOrEmpty(patientICN.Trim()))
                {
                    Logging.WriteLogToFile("InstrumentMpaiEntry.SaveCompletedSurvey(): WarningRedirect: String.IsNullOrEmpty(patientICN.Trim()");
                    //Lost state. Redirect to message that need to log back in from CPRS.
                    Response.Redirect("Warning.aspx?err=2", false);
                }
            }
            else
            {
                Logging.WriteLogToFile("InstrumentMpaiEntry.SaveCompletedSurvey(): WarningRedirect: ViewState[PatientICN] != null");
                //Lost state. Redirect to message that need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }

            if (_survey != null)
            {
                if (_survey.isNew)
                {
                    //Ensure that a concurrent session hasn't already inserted a record for this survey
                    var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

                    if (surveys.Count > 0)
                    {
                        Logging.WriteLogToFile("InstrumentMpaiEntry.SaveCompletedSurvey(): WarningRedirect: surveys.Count > 0");
                        //A concurrent session has already inserted a record for this survey and it is not new anymore.
                        Response.Redirect("Warning.aspx?err=9", false);
                    }
                    else
                    {
                        _survey.SURVEY_STATUS = "inprocess";
                        _survey.SetAsInsertOnSubmit();
                        _survey.patient_ICN = patientICN;
                        _survey.STD_SURVEY_TYPE_ID = _surveyTypeId;
                        _survey.SURVEY_DATE = DateTime.Today;

                        //Save survey if it's new before adding any responses
                        SuppressSaveMessage = true;
                        _tbiInstrumentManager.Update(_survey);
                        SuppressSaveMessage = false;

                        RefreshSurvey(patientICN);
                    }
                }

                SaveSurvey(ref _survey);

                _survey.SetAsUpdateOnSubmit();
                status = _tbiInstrumentManager.Update(_survey);
                if (status != 0)
                {
                    (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);
                    RefreshSurvey(patientICN);
                    SuppressSaveMessage = false;
                }
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentMpaiEntry.aspx", "100027", "SaveCompleteSurvey()");
        }
        finally
        {
            if (status == 0)
                Helpers.SavePageToCache(this, FailedFormValuesName);
            else
                Session[FailedFormValuesName] = null;//clear out the cached version on success
        }
    }

    private void SaveText(SURVEY survey, TextBox textBox, string questionNumber)
    {
        List<STD_QUESTION_CHOICE> availableChoices = VeteransAffairs.Registries.QuestionChoiceRetriever.GetQuestionChoices(questionNumber);
        string inputText = textBox.Text.Trim();
        int id = availableChoices.FirstOrDefault(item => item.CHOICE_SORT_ORDER == 0).STD_QUESTION_CHOICE_ID;
        SaveGeneric(survey, id, inputText);
    }

    //Expects to caller to understand the impact of saving a simple item - may cause questions to have more than 1 response
    private void SaveGeneric(SURVEY survey, int id, string text)
    {
        SURVEY_RESULT currentResult = survey.SURVEY_RESULTs.FirstOrDefault(item => item.STD_QUESTION_CHOICE_ID == id);
        if (null != currentResult)
        {
            if (currentResult.RESULT_TEXT != text)
            {
                currentResult.SetAsUpdateOnSubmit();
                currentResult.RESULT_TEXT = text;
            }
            else
            {
                currentResult.SetAsNoChangeOnSubmit();
            }
        }
        else if (text != string.Empty)
        {
            SURVEY_RESULT result = new SURVEY_RESULT();
            result.RESULT_TEXT = text;
            result.STD_QUESTION_CHOICE_ID = id;
            survey.SURVEY_RESULTs.Add(result);
        }
    }

    /// <summary>
    /// Save survey.
    /// </summary>
    /// <param name="survey"></param>
    private void SaveSurvey(ref SURVEY survey)
    {
        //first, mark all current answers for deletion
        survey.SetAsChangeTrackingRoot();
        foreach (SURVEY_RESULT tempResult in survey.SURVEY_RESULTs)
            tempResult.SetAsDeleteOnSubmit();
        //now determine whether to add or update each item

        //Survey header:
        SaveGeneric(survey, Convert.ToInt32(LabelNoteTypeID.Text), LabelNoteType.Text);
        Helpers.SaveSingleSelect(survey, RadioButtonListPersonReporting, "0");
        if (!String.IsNullOrEmpty(TextBoxPersonReportingDetail.Text.Trim()) && (RadioButtonListPersonReporting.SelectedIndex == RadioButtonListPersonReporting.Items.Count - 1))
        {
            SaveText(survey, TextBoxPersonReportingDetail, "0-1");
        }

        //Survey questions
        Helpers.SaveSingleSelect(survey, RadioButtonListQ1, "1");
        SaveText(survey, TextBoxC1, "C1");
        Helpers.SaveSingleSelect(survey, RadioButtonListQ2, "2");
        SaveText(survey, TextBoxC2, "C2");
        Helpers.SaveSingleSelect(survey, RadioButtonListQ3, "3");
        SaveText(survey, TextBoxC3, "C3");
        Helpers.SaveSingleSelect(survey, RadioButtonListQ4, "4");
        SaveText(survey, TextBoxC4, "C4");
        Helpers.SaveSingleSelect(survey, RadioButtonListQ5, "5");
        SaveText(survey, TextBoxC5, "C5");
        Helpers.SaveSingleSelect(survey, RadioButtonListQ6, "6");
        SaveText(survey, TextBoxC6, "C6");
        if (RadioButtonListQ7A.SelectedIndex > -1)
        {
            Helpers.SaveSingleSelect(survey, RadioButtonListQ7A, "7A");
            SaveText(survey, TextBoxC7A, "C7A");
        }
        if (RadioButtonListQ7B.SelectedIndex > -1)
        {
            if (RadioButtonListQ7BR.SelectedIndex > -1)
            {
                Helpers.SaveSingleSelect(survey, RadioButtonListQ7BR, "7B-R");
            }
            Helpers.SaveSingleSelect(survey, RadioButtonListQ7B, "7B");
            SaveText(survey, TextBoxC7B, "C7B");
        }
        Helpers.SaveSingleSelect(survey, RadioButtonListQ8, "8");
        SaveText(survey, TextBoxC8, "C8");
    }// END SaveSurvey

    private void SetNoteInformation(SURVEY_RESULT result)
    {
        LabelNoteTypeID.Text = result.STD_QUESTION_CHOICE_ID.ToString();
        LabelNoteType.Text = result.STD_QUESTION_CHOICE.CHOICE_TEXT;
    }

    private void SetRadioElement(SURVEY_RESULT result, RadioButtonList radioButton)
    {
        if (result.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.HasValue)
        {
            radioButton.SelectedIndex = result.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.Value;
        }
        else
        {
            radioButton.SelectedIndex = -1;
        }
    }

    private void SetTextBoxElement(SURVEY_RESULT survey, TextBox textBox)
    {
        textBox.Text = survey.RESULT_TEXT;
    }

    /// <summary>
    /// Initialize/setup controls
    /// </summary>
    /// <param name="survey"></param>
    private void SetUpSurvey(SURVEY survey)
    {
        try
        {
            foreach (var sResult in survey.SURVEY_RESULTs)
            {
                switch (sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER)
                {
                    case "-1": SetNoteInformation(sResult); break;
                    case "0": SetRadioElement(sResult, RadioButtonListPersonReporting); break;
                    case "0-1": SetTextBoxElement(sResult, TextBoxPersonReportingDetail); break;
                    case "1": SetRadioElement(sResult, RadioButtonListQ1); break;
                    case "2": SetRadioElement(sResult, RadioButtonListQ2); break;
                    case "3": SetRadioElement(sResult, RadioButtonListQ3); break;
                    case "4": SetRadioElement(sResult, RadioButtonListQ4); break;
                    case "5": SetRadioElement(sResult, RadioButtonListQ5); break;
                    case "6": SetRadioElement(sResult, RadioButtonListQ6); break;
                    case "7A": SetRadioElement(sResult, RadioButtonListQ7A); break;
                    case "7B-R": SetRadioElement(sResult, RadioButtonListQ7BR); break;
                    case "7B": SetRadioElement(sResult, RadioButtonListQ7B); break;
                    case "8": SetRadioElement(sResult, RadioButtonListQ8); break;
                    case "C1": SetTextBoxElement(sResult, TextBoxC1); break;
                    case "C2": SetTextBoxElement(sResult, TextBoxC2); break;
                    case "C3": SetTextBoxElement(sResult, TextBoxC3); break;
                    case "C4": SetTextBoxElement(sResult, TextBoxC4); break;
                    case "C5": SetTextBoxElement(sResult, TextBoxC5); break;
                    case "C6": SetTextBoxElement(sResult, TextBoxC6); break;
                    case "C7A": SetTextBoxElement(sResult, TextBoxC7A); break;
                    case "C7B": SetTextBoxElement(sResult, TextBoxC7B); break;
                    case "C8": SetTextBoxElement(sResult, TextBoxC8); break;
                }
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentMpaiEntry.aspx", "100027", "SetUpSurvey()");
        }
    }

    private void SetQuestionLabels()
    {
        List<STD_QUESTION> questions = VeteransAffairs.Registries.QuestionChoiceRetriever.GetQuestions();
        NoteTypeLabel.Text = questions.First(q => q.QUESTION_NUMBER == "-1").QUESTION_TEXT + ":";
        PersonReportingLabel.Text = questions.First(q => q.QUESTION_NUMBER == "0").QUESTION_TEXT + ":";
        Q1Label.Text = "1. " + questions.First(q => q.QUESTION_NUMBER == "1").QUESTION_TEXT;
        C1Label.Text = questions.First(q => q.QUESTION_NUMBER == "C1").QUESTION_TEXT + ":";
        Q2Label.Text = "2. " + questions.First(q => q.QUESTION_NUMBER == "2").QUESTION_TEXT;
        C2Label.Text = questions.First(q => q.QUESTION_NUMBER == "C2").QUESTION_TEXT + ":";
        Q3Label.Text = "3. " + questions.First(q => q.QUESTION_NUMBER == "3").QUESTION_TEXT;
        C3Label.Text = questions.First(q => q.QUESTION_NUMBER == "C3").QUESTION_TEXT + ":";
        Q4Label.Text = "4. " + questions.First(q => q.QUESTION_NUMBER == "4").QUESTION_TEXT;
        C4Label.Text = questions.First(q => q.QUESTION_NUMBER == "C4").QUESTION_TEXT + ":";
        Q5Label.Text = "5. " + questions.First(q => q.QUESTION_NUMBER == "5").QUESTION_TEXT;
        C5Label.Text = questions.First(q => q.QUESTION_NUMBER == "C5").QUESTION_TEXT + ":";
        Q6Label.Text = "6. " + questions.First(q => q.QUESTION_NUMBER == "6").QUESTION_TEXT;
        C6Label.Text = questions.First(q => q.QUESTION_NUMBER == "C6").QUESTION_TEXT + ":";
        Q7ALabel.Text = "7A. " + questions.First(q => q.QUESTION_NUMBER == "7A").QUESTION_TEXT;
        C7ALabel.Text = questions.First(q => q.QUESTION_NUMBER == "C7A").QUESTION_TEXT + ":";
        Q7BLabel.Text = "7B. " + questions.First(q => q.QUESTION_NUMBER == "7B").QUESTION_TEXT;
        Q7BRLabel.Text = questions.First(q => q.QUESTION_NUMBER == "7B-R").QUESTION_TEXT + ":";
        C7BLabel.Text = questions.First(q => q.QUESTION_NUMBER == "C7B").QUESTION_TEXT + ":";
        Q8Label.Text = "8. " + questions.First(q => q.QUESTION_NUMBER == "8").QUESTION_TEXT;
        C8Label.Text = questions.First(q => q.QUESTION_NUMBER == "C8").QUESTION_TEXT + ":";
    }

    private string GetInstrumentTitle(int surveyID)
    {
        string ret = String.Empty;

        try
        {
            List<string> instrumentTitle = selectedInstrumentType.Title;
            if (instrumentTitle.Count > 0)
            {
                foreach (string titleLine in instrumentTitle)
                {
                    if (titleLine.Contains("PhD"))
                    {
                        ret = ret + "<small>" + titleLine + "</small>" + "<br \\>";
                    }
                    else
                    {
                        ret = ret + titleLine + "<br \\>";
                    }
                }
            }

        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentMpaiEntry.aspx", "100027", "GetInstrumentTitle()");
            // Ignore error
        }

        return ret;
    }

    #endregion Private Methods
}