﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.TBI;
using System.Text;

public class Result {
    public DateTime date;
    public string result;
}

public class AllResults {
    public DateTime date;
    public string result1;
    public string result2;
    public string result3;
    public string result4;
    public string result5;
    public string result6;
}

public partial class TBI_Instruments_JFKComaRecoveryScale : BaseTBIInstrumentPage {
    #region Variable Declarations

    PatientManager patientManager = new PatientManager();
    InstrumentType instrumentType = new InstrumentType();

    protected int _surveyTypeId = 20; // Database Id for JFK Coma Recovery Scale

    SURVEY _survey;   // the root object in the Survey set
    SURVEY_RESULT _surveyResult; // the object representing the actual data in the Survey

    List<STD_QUESTION> QuestionList = new List<STD_QUESTION>();

    #endregion

    /// <summary>
    /// Default Constructor
    /// </summary>
    public TBI_Instruments_JFKComaRecoveryScale() {
        //Manager Save Notification Event Handler
        patientManager.BOSaveSuccess += manager_BOSaveSuccess;
    }

    #region Protected Methods

    /// <summary>
    /// Page Load method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e) {
        TBIServiceTO _tbiServiceTo;

        try {
            _tbiServiceTo = (TBIServiceTO)Session["TBIServiceTO"];
            _tbiInstrumentManager.TbiServiceTo = _tbiServiceTo;

            string instTitle = _tbiInstrumentManager.GetInstrumentByID(_surveyTypeId).Name;

            LabelBreadCrumb.Text = "TBI Instruments > Instrument Associations > " + instTitle;
            LabelInstrumentTitle.Text = instTitle;
            Page.Title = "TBI Instruments - " + instTitle;

            if (_tbiInstrumentManager.TbiServiceTo != null) {
                Label labelUser = (Label)Master.FindControl("LabelCurrentUser");
                labelUser.Text = _tbiInstrumentManager.TbiServiceTo.UserName;

                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }
        } catch (Exception ex) {
            //Redirect to message that session is ended and need to log back in from CPRS.
            Exception _tbilogerror = HandleTBIException(ex, "JFKComaRecoveryScale.aspx", "100027", "SESSIOENDED");
            Response.Redirect("Warning.aspx?err=2", false);
        }

        if (!IsPostBack) {
            if (instrumentType.Status != null) {
                _tbiInstrumentManager.TbiServiceTo.SelectedInstrumentType = instrumentType;
            }

            string _ssn = "";
            if (_tbiInstrumentManager.TbiServiceTo != null) {
                _ssn = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }

            if (!string.IsNullOrEmpty(_ssn)) {
                string patICN = patientManager.GetPatientICNBySSN(_ssn);

                if (!string.IsNullOrEmpty(patICN)) {
                    ViewState["PatientICN"] = patICN;
                    RefreshSurvey(patICN);
                } else {
                    throw new ArgumentException("Patient with this SSN does not exist.");
                }

                if (_survey != null && !_survey.isNew) {
                    SetUpSurvey(_survey);
                    //calculateTotal();
                }
                string referrer = "";
                if (Request.UrlReferrer != null) {
                    referrer = Request.UrlReferrer.PathAndQuery;
                }
                ViewState["referrer"] = referrer;
            }

            //Check if the request is to start a new item 
            //a new item page request is signified when the querystring parameter id equals zero or is not present
            if (_tbiInstrumentManager.TbiServiceTo == null)
            {
                Logging.WriteLogToFile("JFKComaRecoveryScale.Page_Load(): WarningRedirect: _tbiInstrumentManager.TbiServiceTo == null");
                //Redirect to message that session is ended and need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            } else {
                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN.Trim().Length == 9 ? String.Format("{0:000-00-0000}", int.Parse(_tbiInstrumentManager.TbiServiceTo.PatientSSN)) : _tbiInstrumentManager.TbiServiceTo.PatientSSN;
                //LabelFacility.Text = _tbiInstrumentManager.TbiServiceTo.InstitutionName;
            }


        }
        QuestionList = GetAllQuestionsForSurvey();

        // Set control values for Diagnosis Question
        ctrlDiagnosis.QuestionText = GetQuestionText("DX");
        ctrlDiagnosis.SurveyTypeCode = HiddenSurveyType.Value;

        LabelForEtiology.Text = GetQuestionText("Etiology");
        LabelForDateOnset.Text = GetQuestionText("DateOnset");
        LabelForDateAdmission.Text = GetQuestionText("DateAdmission");
        LabelDate1.Text = GetQuestionText("D");
        LabelForWeekNo.Text = GetQuestionText("Week");
        LabelForQ1.Text = GetQuestionText("1");
        LabelForQ2.Text = GetQuestionText("2");
        LabelForQ3.Text = GetQuestionText("3");
        LabelForQ4.Text = GetQuestionText("4");
        LabelForQ5.Text = GetQuestionText("5");
        LabelForQ6.Text = GetQuestionText("6");
        LabelForQ7.Text = GetQuestionText("7");
        dpDate1.MaxDate = DateTime.Now;

        ////List<SURVEY> surveys = _tbiInstrumentManager.Select10MostRecentByPatientICNAndSurveyType(_tbiInstrumentManager.TbiServiceTo.PatientICN, 20, false).ToList();
        //List<SURVEY> surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(_tbiInstrumentManager.TbiServiceTo.PatientICN, 20, false).ToList();

        //List<Result> last10ForQ1 = GetLast10SortedList(surveys, "1");
        //List<Result> last10ForQ2 = GetLast10SortedList(surveys, "2");
        //List<Result> last10ForQ3 = GetLast10SortedList(surveys, "3");
        //List<Result> last10ForQ4 = GetLast10SortedList(surveys, "4");
        //List<Result> last10ForQ5 = GetLast10SortedList(surveys, "5");
        //List<Result> last10ForQ6 = GetLast10SortedList(surveys, "6");

        //lblLast10ResultsForQ1.Text = SetupLast10Surveys(last10ForQ1);
        //lblLast10ResultsForQ2.Text = SetupLast10Surveys(last10ForQ2);
        //lblLast10ResultsForQ3.Text = SetupLast10Surveys(last10ForQ3);
        //lblLast10ResultsForQ4.Text = SetupLast10Surveys(last10ForQ4);
        //lblLast10ResultsForQ5.Text = SetupLast10Surveys(last10ForQ5);
        //lblLast10ResultsForQ6.Text = SetupLast10Surveys(last10ForQ6);
    }

    private List<AllResults> GetLast10Surveys(List<SURVEY> surveys) {
        DateTime vDate = new DateTime();
        string vResult1 = "";
        string vResult2 = "";
        string vResult3 = "";
        string vResult4 = "";
        string vResult5 = "";
        string vResult6 = "";
        var list = new List<AllResults>();
        if (surveys != null) {
            foreach (SURVEY s in surveys) {
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs) {
                    if (sr.STD_QUESTION_CHOICE.CHOICE_NAME.ToUpper().Trim() == "DATE") {
                        vDate = Convert.ToDateTime(sr.RESULT_TEXT);
                        break;
                    }
                }
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs) {
                    if (sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER.Trim() == "1") {
                        vResult1 = sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_TEXT + ":   " + sr.STD_QUESTION_CHOICE.CHOICE_NAME + " - " + sr.STD_QUESTION_CHOICE.CHOICE_TEXT;
                        break;
                    }
                }
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs) {
                    if (sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER.Trim() == "2") {
                        vResult2 = sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_TEXT + ":   " + sr.STD_QUESTION_CHOICE.CHOICE_NAME + " - " + sr.STD_QUESTION_CHOICE.CHOICE_TEXT;
                        break;
                    }
                }
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs) {
                    if (sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER.Trim() == "3") {
                        vResult3 = sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_TEXT + ":   " + sr.STD_QUESTION_CHOICE.CHOICE_NAME + " - " + sr.STD_QUESTION_CHOICE.CHOICE_TEXT;
                        break;
                    }
                }
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs) {
                    if (sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER.Trim() == "4") {
                        vResult4 = sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_TEXT + ":   " + sr.STD_QUESTION_CHOICE.CHOICE_NAME + " - " + sr.STD_QUESTION_CHOICE.CHOICE_TEXT;
                        break;
                    }
                }
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs) {
                    if (sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER.Trim() == "5") {
                        vResult5 = sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_TEXT + ":   " + sr.STD_QUESTION_CHOICE.CHOICE_NAME + " - " + sr.STD_QUESTION_CHOICE.CHOICE_TEXT;
                        break;
                    }
                }
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs) {
                    if (sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER.Trim() == "6") {
                        vResult6 = sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_TEXT + ":   " + sr.STD_QUESTION_CHOICE.CHOICE_NAME + " - " + sr.STD_QUESTION_CHOICE.CHOICE_TEXT;
                        break;
                    }
                }

                list.Add(new AllResults() { date = vDate, result1 = vResult1, result2 = vResult2, result3 = vResult3, result4 = vResult4, result5 = vResult5, result6 = vResult6, });
            }
        }
        list.Sort((x, y) => DateTime.Compare(x.date, y.date));
        list.Reverse();
        list.Take(10);
        return list;
    }

    /*private List<Result> GetLast10SortedList(List<SURVEY> surveys, string questionNo)
    {
        DateTime vDate = new DateTime();
        string vResult = "";
        var list = new List<Result>();
        if (surveys != null)
        {
            foreach (SURVEY s in surveys)
            {
                //foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs.OrderBy(R => R.SURVEY.SURVEY_RESULTs[4].RESULT_TEXT))
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs)
                {
                    if (sr.STD_QUESTION_CHOICE.CHOICE_NAME.ToUpper().Trim() == "DATE")
                    {
                        vDate = Convert.ToDateTime(sr.RESULT_TEXT);
                        break;
                    }
                }
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs)
                {
                    if (sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER.Trim() == questionNo)
                    {
                        vResult = sr.STD_QUESTION_CHOICE.CHOICE_NAME + " - " + sr.STD_QUESTION_CHOICE.CHOICE_TEXT;
                        break;
                    }
                }
                list.Add(new Result() { date = vDate, result = vResult });
            }
        }
        list.Sort((x, y) => DateTime.Compare(x.date, y.date));
        list.Reverse();
        list.Take(10);
        return list;
    }*/

    /*private string SetupLast10Surveys(List<Result> last10Results)
    {
        StringBuilder sbRecentSurveys = new StringBuilder();
        sbRecentSurveys.Append("<table style='width:100%'><tr><td style='width:10px;'></td><td colspan='3'><h4>Most recent surveys</h4></td></tr>");
        foreach (Result r in last10Results)
        {
            sbRecentSurveys.Append("<tr>");
            sbRecentSurveys.Append("<td style='width:10px;'>&nbsp;</td>");
            sbRecentSurveys.Append("<td style='width:100px; align:right;'>" + r.date.ToString("MM/dd/yyyy") + "</td>");
            sbRecentSurveys.Append("<td style='width:10px;'>&nbsp;</td>");
            sbRecentSurveys.Append("<td>" + r.result + "</td>");
            sbRecentSurveys.Append("</tr>");
        }
        sbRecentSurveys.Append("</table>");

        return sbRecentSurveys.ToString();
    }*/

    protected override void LoadViewState(object savedState) {
        base.LoadViewState(savedState);
        string surveyXml = (string)ViewState["surveyXml"];
        _survey = SerializerUtil.Deserialize<SURVEY>(surveyXml);
    }

    protected override object SaveViewState() {
        ViewState["surveyXml"] = _survey.Serialize();
        return base.SaveViewState();
    }

    /// <summary>
    /// Returns the Question_Text of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected string GetQuestionText(string questionId) {
        foreach (STD_QUESTION ques in QuestionList) {
            if (ques.QUESTION_NUMBER.ToLower() == questionId.ToLower()) {
                return ques.QUESTION_TEXT;
            }
        }

        return "";
    }

    /// <summary>
    /// Returns the Question_Id of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected int GetQuestionId(string questionNumber) {
        foreach (STD_QUESTION ques in QuestionList) {
            if (ques.QUESTION_NUMBER.ToLower() == questionNumber.ToLower()) {
                return ques.ID;
            }
        }

        return 0;
    }

    /// <summary>
    /// Save Draft button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public void btnSaveSurveyDraft_Click(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);
        SetUpSurvey(_survey);
    }

    /// <summary>
    /// Save and Prepare Note button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnSubmit_Click(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSubmitSurvey.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);

        if (status > 0) {
            Session["survey"] = _survey;
            Response.Redirect("InstrumentSummaryNote.aspx", false);
        } else {
            SetUpSurvey(_survey); //Follow Save Draft logic 
        }
    }

    /// <summary>
    /// Inactive timer firing.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void InActiveTimer_Tick(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";

        Page.Validate();

        if (Page.IsValid) {
            SaveCompleteSurvey(ref status);
            //SetUpSurvey(_survey);
            Response.Redirect("JFKComaRecoveryScale.aspx", false);
        } else {
            SaveFailMessage = "Auto-Save Validation Failed. Please complete required fields in the survey and press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        }
    }

    /// <summary>
    /// Cancel button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCancel_Click(object sender, EventArgs e) {
        ViewState["surveyXml"] = null;
        ViewState["PatientICN"] = null;
        ViewState["referrer"] = null;
        Session.Remove("survey");
        Response.Redirect("Instrument.aspx", false);
        //this.ClientScript.RegisterStartupScript(typeof(Page), "closePage", "<script type='text/javascript'> window.open('','_self','');window.close(); </script>");
    }

    /// <summary>
    /// Hide or Show instructions
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void lnkBtnToggleInstructions_OnClick(object sender, EventArgs e) {
        if (PanelInstructions.Visible) {
            PanelInstructions.Visible = false;
            lnkBtnToggleInstructions.Text = "Click for Instructions";
        } else {
            PanelInstructions.Visible = true;
            lnkBtnToggleInstructions.Text = "Hide Instructions";
        }
    }

    /// <summary>
    /// Hide or Show instructions
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void lnkBtnToggleLast10Surveys_OnClick(object sender, EventArgs e) {
        if (pnlLast10.Visible) {
            pnlLast10.Visible = false;
            lnkBtnToggleLast10Surveys.Text = "Click for most recent surveys";
        } else {
            // Get 10 most recent surveys
            List<SURVEY> surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(_tbiInstrumentManager.TbiServiceTo.PatientICN, 20, false).ToList();
            List<AllResults> last10 = GetLast10Surveys(surveys);
            StringBuilder sbLast10Table = new StringBuilder();
            sbLast10Table.Append("<table border='1px' style='width:800px;margin-left:20px;'>");
            sbLast10Table.Append("<tr><td colspan='100%'><b>Most Recent Surveys</b></td></tr>");
            foreach (AllResults result in last10) {
                sbLast10Table.Append("<tr><td style='width:150px'>" + result.date.ToShortDateString() + "</td>");
                sbLast10Table.Append("<td>");
                sbLast10Table.Append(result.result1.ToString() + "<br/>");
                sbLast10Table.Append(result.result2.ToString() + "<br/>");
                sbLast10Table.Append(result.result3.ToString() + "<br/>");
                sbLast10Table.Append(result.result4.ToString() + "<br/>");
                sbLast10Table.Append(result.result5.ToString() + "<br/>");
                sbLast10Table.Append(result.result6.ToString() + "</td></tr>");
            }
            sbLast10Table.Append("</table");
            lblLast10Table.Text = sbLast10Table.ToString();

            pnlLast10.Visible = true;
            lnkBtnToggleLast10Surveys.Text = "Hide most recent surveys";
        }
    }


    /// <summary>
    /// Calculate button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCalculate_Click(object sender, EventArgs e) {
        // perform calculation
        calculateTotal();
    }


    /// <summary>
    /// Calculate total score.
    /// </summary>
    protected void calculateTotal() {
        bool isSurveyCompletelyFilledOut = true;
        double total = 0;
        STD_QUESTION_CHOICE choice;

        foreach (STD_QUESTION question in QuestionList) {
            switch (question.QUESTION_NUMBER) {
                case "1":
                    if (!String.IsNullOrEmpty(rblQ1.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ1.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            total += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;

                case "2":
                    if (!String.IsNullOrEmpty(rblQ2.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ2.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            total += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "3":
                    if (!String.IsNullOrEmpty(rblQ3.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ3.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            total += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "4":
                    if (!String.IsNullOrEmpty(rblQ4.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ4.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            total += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "5":
                    if (!String.IsNullOrEmpty(rblQ5.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ5.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            total += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "6":
                    if (!String.IsNullOrEmpty(rblQ6.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ6.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            total += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;


                default:
                    break;
            }
        }

        lblTotalScore.Text = total.ToString();

        if (isSurveyCompletelyFilledOut) {
            lblSurveyError.Text = "";
        } else {
            // ERROR ... 
            lblSurveyError.Text = "ERROR: Survey is not completed."; // TODO - Alert error.
        }
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Get all questions for this survey
    /// </summary>
    /// <returns></returns>
    private List<STD_QUESTION> GetAllQuestionsForSurvey() {
        return _tbiInstrumentManager.SelectAllQuestions(_surveyTypeId).ToList();
    }

    /// <summary>
    /// Refresh Survey
    /// </summary>
    /// <param name="patientICN"></param>
    private void RefreshSurvey(string patientICN) {
        var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

        if (surveys.Count > 0) {
            _survey = surveys.First();
        } else {
            _survey = new SURVEY();
        }

        _survey.SetAsChangeTrackingRoot();
    }

    /// <summary>
    /// Save Complete Survey
    /// </summary>
    /// <param name="status"></param>
    private void SaveCompleteSurvey(ref Int32 status) {
        Page.Validate();
        if ((Page.IsValid)) {
            string patientICN = String.Empty;

            status = 0;

            //Retrieve patient's ICN from the session variable
            if (ViewState["PatientICN"] != null) {
                patientICN = ViewState["PatientICN"].ToString();
                if (String.IsNullOrEmpty(patientICN.Trim()))
                {
                    Logging.WriteLogToFile("JFKComaRecoveryScale.SaveCompletedSurvey(): WarningRedirect: String.IsNullOrEmpty(patientICN.Trim()");
                    //Lost state. Redirect to message that need to log back in from CPRS.
                    Response.Redirect("Warning.aspx?err=2", false);
                }
            }
            else
            {
                Logging.WriteLogToFile("JFKComaRecoveryScale.SaveCompletedSurvey(): WarningRedirect: ViewState[PatientICN] != null");
                //Lost state. Redirect to message that need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }

            if (_survey != null) {
                if (_survey.isNew) {
                    //Ensure that a concurrent session hasn't already inserted a record for this survey
                    var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

                    if (surveys.Count > 0)
                    {
                        Logging.WriteLogToFile("JFKComaRecoveryScale.SaveCompletedSurvey(): WarningRedirect: surveys.Count > 0");
                        //A concurrent session has already inserted a record for this survey and it is not new anymore.
                        Response.Redirect("Warning.aspx?err=9", false);
                    } else {
                        _survey.SURVEY_STATUS = "inprocess";

                        _survey.SetAsInsertOnSubmit();

                        _survey.patient_ICN = patientICN;

                        _survey.STD_SURVEY_TYPE_ID = _surveyTypeId;

                        // TODO: change to datepicker
                        _survey.SURVEY_DATE = DateTime.Today;

                        //Save survey if it's new before adding any responses
                        SuppressSaveMessage = true;
                        _tbiInstrumentManager.Update(_survey);
                        SuppressSaveMessage = false;

                        RefreshSurvey(patientICN);
                    }
                }

                SaveSurvey(ref _survey);

                _survey.SetAsUpdateOnSubmit();
                status = _tbiInstrumentManager.Update(_survey);

                (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);

                RefreshSurvey(patientICN);
                SuppressSaveMessage = false;
            }
        }
    }

    /// <summary>
    /// Save survey.
    /// </summary>
    /// <param name="survey"></param>
    private void SaveSurvey(ref SURVEY survey) {
        calculateTotal(); // call to re-calculate total score.

        // Ensure lblSurveyError does not contain - ERROR
        if (!lblSurveyError.Text.Contains("ERROR")) {
            List<STD_QUESTION> questions = GetAllQuestionsForSurvey();

            //Delete all the survey results from database if there are any.
            SURVEY surveyWithResults = _tbiInstrumentManager.GetSurveyWithAllAnswers(survey.SURVEYS_ID);
            _tbiInstrumentManager.DeleteAllSurveyResults(surveyWithResults);


            // Question DX
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question DX choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(ctrlDiagnosis.GetSelectedId());
                _surveyResult.RESULT_TEXT = ctrlDiagnosis.GetSelectedResultText();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question DX");
            }

            // Question Etiology
            STD_QUESTION questionEtiology = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "Etiology");
            _surveyResult = new SURVEY_RESULT();
            try {
                _surveyResult.STD_QUESTION_CHOICE_ID = questionEtiology.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtEtiology.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question Etiology");
            }

            // DateOnset
            _surveyResult = new SURVEY_RESULT();
            if (dpDateOnset.SelectedDate != null) {
                STD_QUESTION questionD = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "DateOnset");
                _surveyResult = new SURVEY_RESULT();
                try {
                    _surveyResult.STD_QUESTION_CHOICE_ID = questionD.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                    _surveyResult.RESULT_TEXT = ((DateTime)dpDateOnset.SelectedDate).ToShortDateString();
                    survey.SURVEY_RESULTs.Add(_surveyResult);
                } catch (Exception surveyEx) {
                    Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question Date Onset");
                }
            }

            // DateAdmission
            _surveyResult = new SURVEY_RESULT();
            if (dpDateAdmission.SelectedDate != null) {
                STD_QUESTION questionD = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "DateAdmission");
                _surveyResult = new SURVEY_RESULT();
                try {
                    _surveyResult.STD_QUESTION_CHOICE_ID = questionD.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                    _surveyResult.RESULT_TEXT = ((DateTime)dpDateAdmission.SelectedDate).ToShortDateString();
                    survey.SURVEY_RESULTs.Add(_surveyResult);
                } catch (Exception surveyEx) {
                    Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question Date Admission");
                }
            }

            // Date
            _surveyResult = new SURVEY_RESULT();
            if (dpDate1.SelectedDate != null) {
                STD_QUESTION questionD = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "D");
                _surveyResult = new SURVEY_RESULT();
                try {
                    // Add Question DTE choice
                    _surveyResult.STD_QUESTION_CHOICE_ID = questionD.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                    _surveyResult.RESULT_TEXT = ((DateTime)dpDate1.SelectedDate).ToShortDateString();
                    survey.SURVEY_RESULTs.Add(_surveyResult);
                } catch (Exception surveyEx) {
                    Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question D");
                }
            }

            // Question Week
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question Week choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(ddlWeekNo.SelectedValue);
                _surveyResult.RESULT_TEXT = ddlWeekNo.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question Week");
            }

            // Question 1
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 1 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ1.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ1.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question 1");
            }

            // Question 2
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 2 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ2.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ2.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question 2");
            }

            // Question 3
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 3 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ3.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ3.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question 3");
            }

            // Question 4
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 4 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ4.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ4.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question 4");
            }

            // Question 5
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 5 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ5.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ5.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question 5");
            }

            // Question 6
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 6 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ6.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ6.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question 6");
            }


            // Question 7 - Total
            STD_QUESTION question7 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "7");
            _surveyResult = new SURVEY_RESULT();
            try {
                _surveyResult.STD_QUESTION_CHOICE_ID = question7.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblTotalScore.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "JFKComaRecoveryScale.aspx", "100027", "SaveSurvey() - Question 7");
            }


        } else {
            // do not save ... 
        }

    }// END SaveSurvey

    /// <summary>
    /// Initialize/setup controls
    /// </summary>
    /// <param name="survey"></param>
    private void SetUpSurvey(SURVEY survey) {
        try {
            foreach (var sResult in survey.SURVEY_RESULTs) {
                switch (sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER) {
                    case "DX":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            ctrlDiagnosis.SelectedChoiceId = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();

                            if (sResult.STD_QUESTION_CHOICE.CHOICE_TEXT.ToLower().Contains("other")) {
                                ctrlDiagnosis.SetOtherDescriptionText(sResult.RESULT_TEXT);
                            }
                        } else {
                            ctrlDiagnosis.SelectedChoiceId = null;
                        }
                        break;

                    case "Etiology":
                        txtEtiology.Text = sResult.RESULT_TEXT;
                        break;

                    case "DateOnset":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            DateTime temp;
                            if (DateTime.TryParse(sResult.RESULT_TEXT, out temp)) {
                                dpDateOnset.SelectedDate = temp;
                            }
                        }
                        break;

                    case "DateAdmission":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            DateTime temp;
                            if (DateTime.TryParse(sResult.RESULT_TEXT, out temp)) {
                                dpDateAdmission.SelectedDate = temp;
                            }
                        }
                        break;

                    case "D":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            DateTime temp;
                            if (DateTime.TryParse(sResult.RESULT_TEXT, out temp)) {
                                dpDate1.SelectedDate = temp;
                            }
                        }
                        break;

                    case "Week":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            ddlWeekNo.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            ddlWeekNo.SelectedIndex = -1;
                        }
                        break;

                    case "1":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ1.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ1.SelectedIndex = -1;
                        }
                        break;

                    case "2":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ2.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ2.SelectedIndex = -1;
                        }
                        break;
                    case "3":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ3.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ3.SelectedIndex = -1;
                        }
                        break;
                    case "4":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ4.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ4.SelectedIndex = -1;
                        }
                        break;
                    case "5":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ5.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ5.SelectedIndex = -1;
                        }
                        break;
                    case "6":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ6.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ6.SelectedIndex = -1;
                        }
                        break;
                    case "7":
                        lblTotalScore.Text = sResult.RESULT_TEXT;
                        break;

                    default:
                        break;
                }
            }
        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "JFKComaRecoveryScale.aspx", "100027", "SetUpSurvey()");
        }
    }
    // END SetUpSurvey


    /// <summary>
    /// Retrieve instrument title
    /// </summary>
    /// <param name="surveyID"></param>
    /// <returns></returns>
    private string GetInstrumentTitle(int surveyID) {
        string ret = String.Empty;

        try {
            List<string> instrumentTitle = _tbiInstrumentManager.GetInstrumentByID(surveyID).Title;
            if (instrumentTitle.Count > 0) {
                foreach (string titleLine in instrumentTitle) {
                    if (titleLine.Contains("PhD")) {
                        ret = ret + "<small>" + titleLine + "</small>" + "<br \\>";
                    } else {
                        ret = ret + titleLine + "<br \\>";
                    }
                }
            }

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "JFKComaRecoveryScale.aspx", "100027", "GetInstrumentTitle()");
            // Ignore error
        }

        return ret;
    }

    #endregion
}