﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.TBI;

public partial class TBI_Instruments_VALowVisionSurvey : BaseTBIInstrumentPage {
    #region Variable Declarations

    PatientManager patientManager = new PatientManager();
    InstrumentType instrumentType = new InstrumentType();

    protected int _surveyTypeId = 17; // Database Id for VA Low Vision Visual Functioning (VA LV VFQ 20) Survey

    SURVEY _survey;   // the root object in the Survey set
    SURVEY_RESULT _surveyResult; // the object representing the actual data in the Survey

    List<STD_QUESTION> QuestionList = new List<STD_QUESTION>();

    #endregion

    /// <summary>
    /// Default Constructor
    /// </summary>
    public TBI_Instruments_VALowVisionSurvey() {
        //Manager Save Notification Event Handler
        patientManager.BOSaveSuccess += manager_BOSaveSuccess;
    }

    #region Protected Methods

    /// <summary>
    /// Page Load method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e) {
        TBIServiceTO _tbiServiceTo;

        try {
            // confirm instrument is active - if not, a redirect will occur.
            this.CheckActiveStatus(_surveyTypeId);

            _tbiServiceTo = (TBIServiceTO)Session["TBIServiceTO"];
            _tbiInstrumentManager.TbiServiceTo = _tbiServiceTo;

            string instTitle = _tbiInstrumentManager.GetInstrumentByID(_surveyTypeId).Name;

            LabelBreadCrumb.Text = "TBI Instruments > Instrument Associations > " + instTitle;
            LabelInstrumentTitle.Text = instTitle;
            Page.Title = "TBI Instruments - " + instTitle;

            if (_tbiInstrumentManager.TbiServiceTo != null) {
                Label labelUser = (Label)Master.FindControl("LabelCurrentUser");
                labelUser.Text = _tbiInstrumentManager.TbiServiceTo.UserName;

                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }
        } catch (Exception ex) {
            //Redirect to message that session is ended and need to log back in from CPRS.
            Exception _tbilogerror = HandleTBIException(ex, "VALowVisionSurvey.aspx", "100027", "SESSIOENDED");
            Response.Redirect("Warning.aspx?err=2", false);
        }

        if (!IsPostBack) {
            if (instrumentType.Status != null) {
                _tbiInstrumentManager.TbiServiceTo.SelectedInstrumentType = instrumentType;
            }

            string _ssn = "";
            if (_tbiInstrumentManager.TbiServiceTo != null) {
                _ssn = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }

            if (!string.IsNullOrEmpty(_ssn)) {
                string patICN = patientManager.GetPatientICNBySSN(_ssn);

                if (!string.IsNullOrEmpty(patICN)) {
                    ViewState["PatientICN"] = patICN;
                    RefreshSurvey(patICN);
                } else {
                    throw new ArgumentException("Patient with this SSN does not exist.");
                }

                if (_survey != null && !_survey.isNew) {
                    SetUpSurvey(_survey);
                }
                string referrer = "";
                if (Request.UrlReferrer != null) {
                    referrer = Request.UrlReferrer.PathAndQuery;
                }
                ViewState["referrer"] = referrer;
            }

            //Check if the request is to start a new item 
            //a new item page request is signified when the querystring parameter id equals zero or is not present
            if (_tbiInstrumentManager.TbiServiceTo == null)
            {
                Logging.WriteLogToFile("VALowVisionSurvey.Page_Load(): WarningRedirect: _tbiInstrumentManager.TbiServiceTo == null");
                //Redirect to message that session is ended and need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            } else {
                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN.Trim().Length == 9 ? String.Format("{0:000-00-0000}", int.Parse(_tbiInstrumentManager.TbiServiceTo.PatientSSN)) : _tbiInstrumentManager.TbiServiceTo.PatientSSN;
                //LabelFacility.Text = _tbiInstrumentManager.TbiServiceTo.InstitutionName;
            }
        }
        QuestionList = GetAllQuestionsForSurvey();

        // Set control values for Diagnosis Question
        ctrlDiagnosis.QuestionText = GetQuestionText("DX");
        ctrlDiagnosis.SurveyTypeCode = HiddenSurveyType.Value;

        LabelDate1.Text = GetQuestionText("D");
        LabelForTYP.Text = GetQuestionText("TYP");
        LabelForQ1.Text = GetQuestionText("1");
        LabelForQ2.Text = GetQuestionText("2");
        LabelForQ3.Text = GetQuestionText("3");
        LabelForQ4.Text = GetQuestionText("4");
        LabelForQ5.Text = GetQuestionText("5");
        LabelForQ6.Text = GetQuestionText("6");
        LabelForQ7.Text = GetQuestionText("7");
        LabelForQ8.Text = GetQuestionText("8");
        LabelForQ9.Text = GetQuestionText("9");
        LabelForQ10.Text = GetQuestionText("10");
        LabelForQ11.Text = GetQuestionText("11");
        LabelForQ12.Text = GetQuestionText("12");
        LabelForQ13.Text = GetQuestionText("13");
        LabelForQ14.Text = GetQuestionText("14");
        LabelForQ15.Text = GetQuestionText("15");
        LabelForQ16.Text = GetQuestionText("16");
        LabelForQ17.Text = GetQuestionText("17");
        LabelForQ18.Text = GetQuestionText("18");
        LabelForQ19.Text = GetQuestionText("19");
        LabelForQ20.Text = GetQuestionText("20");
        LabelForQ21.Text = GetQuestionText("21");
        LabelForQ22.Text = GetQuestionText("22");
        LabelForQ23.Text = GetQuestionText("23");
        LabelForQ24.Text = GetQuestionText("24");

        dpDate1.MaxDate = DateTime.Now;
    }

    protected override void LoadViewState(object savedState) {
        base.LoadViewState(savedState);
        string surveyXml = (string)ViewState["surveyXml"];
        _survey = SerializerUtil.Deserialize<SURVEY>(surveyXml);
    }

    protected override object SaveViewState() {
        ViewState["surveyXml"] = _survey.Serialize();
        return base.SaveViewState();
    }

    /// <summary>
    /// Returns the Question_Text of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected string GetQuestionText(string questionId) {
        foreach (STD_QUESTION ques in QuestionList) {
            if (ques.QUESTION_NUMBER.ToLower() == questionId.ToLower()) {
                return ques.QUESTION_TEXT;
            }
        }

        return "";
    }

    /// <summary>
    /// Returns the Question_Id of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected int GetQuestionId(string questionNumber) {
        foreach (STD_QUESTION ques in QuestionList) {
            if (ques.QUESTION_NUMBER.ToLower() == questionNumber.ToLower()) {
                return ques.ID;
            }
        }

        return 0;
    }

    /// <summary>
    /// Save Draft button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public void btnSaveSurveyDraft_Click(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);
        SetUpSurvey(_survey);
    }

    /// <summary>
    /// Save and Prepare Note button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnSubmit_Click(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSubmitSurvey.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);

        (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);

        if (status > 0) {
            Session["survey"] = _survey;
            Response.Redirect("InstrumentSummaryNote.aspx", false);
        } else {
            SetUpSurvey(_survey); //Follow Save Draft logic 
        }
    }

    /// <summary>
    /// Inactive timer firing.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void InActiveTimer_Tick(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";

        Page.Validate();

        if (Page.IsValid) {
            SaveCompleteSurvey(ref status);
            //SetUpSurvey(_survey);
            Response.Redirect("VALowVisionSurvey.aspx", false);
        } else {
            SaveFailMessage = "Auto-Save Validation Failed. Please complete required fields in the survey and press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        }
    }

    /// <summary>
    /// Cancel button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCancel_Click(object sender, EventArgs e) {
        ViewState["surveyXml"] = null;
        ViewState["PatientICN"] = null;
        ViewState["referrer"] = null;
        Session.Remove("survey");
        Response.Redirect("Instrument.aspx", false);
        //this.ClientScript.RegisterStartupScript(typeof(Page), "closePage", "<script type='text/javascript'> window.open('','_self','');window.close(); </script>");
    }

    /// <summary>
    /// Type of submission trigger
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rblQTYP_SelectedIndexChanged(object sender, EventArgs e) {
        if (rblQTYP.SelectedItem.Text.ToLower() == "baseline") {
            pnlBaselineScores.Visible = true;
            pnlFollowupScores.Visible = false;
        } else {
            pnlBaselineScores.Visible = true;
            pnlFollowupScores.Visible = true;
        }
    }

    /// <summary>
    /// On data bound to type selection
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rblQTYP_DataBound(object sender, EventArgs e) {

        bool hasBaseline = DoesPatientHaveABaseline();

        if (hasBaseline) {
            pnlBaselineScores.Visible = true;
        }
    }

    /// <summary>
    /// Determine if patient has a baseline ... if not, force Baseline and disable follow-up.
    /// </summary>
    /// <returns></returns>
    protected bool DoesPatientHaveABaseline() {
        List<SURVEY> surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(_tbiInstrumentManager.TbiServiceTo.PatientICN, 17, false).ToList();
        bool hasBaseLine = false;
        bool hasCompleteBaseline = false;
        string averageBaselineScore = "";

        if (surveys != null) {
            foreach (SURVEY s in surveys) {
                foreach (SURVEY_RESULT sr in s.SURVEY_RESULTs.OrderBy(R => R.STD_QUESTION_CHOICE.STD_QUESTION.SORT_ORDER)) {
                    if (sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER == "TYP") {
                        if (sr.RESULT_TEXT.ToLower() == "baseline") {
                            hasBaseLine = true;
                            if (sr.SURVEY.SURVEY_STATUS.ToLower() == "completed") {
                                hasCompleteBaseline = true;
                            }
                        }
                    }

                    if (hasBaseLine) {
                        // get baseline score
                        if (sr.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER == "22") {
                            averageBaselineScore = sr.RESULT_TEXT;
                            txtQ22.Text = averageBaselineScore;
                        }
                    }
                }
            }
        }

        foreach (ListItem li in rblQTYP.Items) {
            if (li.Text.ToLower() == "baseline") {
                // baseline
                if (hasBaseLine) {
                    if (hasCompleteBaseline) {
                        li.Enabled = false; 
                    } else {
                        li.Enabled = true;
                    }
                } else {
                    li.Enabled = true;
                }
            } else {
                // follow-up
                if (hasBaseLine) {
                    if (hasCompleteBaseline) {
                        li.Enabled = true;
                    } else {
                        li.Enabled = false;
                    }
                } else {
                    li.Enabled = false;
                }
            }
        }

        return hasBaseLine;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Get all questions for this survey
    /// </summary>
    /// <returns></returns>
    private List<STD_QUESTION> GetAllQuestionsForSurvey() {
        return _tbiInstrumentManager.SelectAllQuestions(_surveyTypeId).ToList();
    }

    /// <summary>
    /// Refresh Survey
    /// </summary>
    /// <param name="patientICN"></param>
    private void RefreshSurvey(string patientICN) {
        var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

        if (surveys.Count > 0) {
            _survey = surveys.First();
        } else {
            _survey = new SURVEY();
        }

        _survey.SetAsChangeTrackingRoot();
    }

    /// <summary>
    /// Save Complete Survey
    /// </summary>
    /// <param name="status"></param>
    private void SaveCompleteSurvey(ref Int32 status) {
        Page.Validate();
        if (Page.IsValid) {

            // calculate scores
            calculateScores();

            string patientICN = String.Empty;

            status = 0;

            //Retrieve patient's ICN from the session variable
            if (ViewState["PatientICN"] != null) {
                patientICN = ViewState["PatientICN"].ToString();
                if (String.IsNullOrEmpty(patientICN.Trim()))
                {
                    Logging.WriteLogToFile("VALowVisionSurvey.SaveCompletedSurvey(): WarningRedirect: String.IsNullOrEmpty(patientICN.Trim()");
                    //Lost state. Redirect to message that need to log back in from CPRS.
                    Response.Redirect("Warning.aspx?err=2", false);
                }
            }
            else
            {
                Logging.WriteLogToFile("VALowVisionSurvey.SaveCompletedSurvey(): WarningRedirect: ViewState[PatientICN] != null");
                //Lost state. Redirect to message that need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }

            if (_survey != null) {
                if (_survey.isNew) {
                    //Ensure that a concurrent session hasn't already inserted a record for this survey
                    var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

                    if (surveys.Count > 0)
                    {
                        Logging.WriteLogToFile("VALowVisionSurvey.SaveCompletedSurvey(): WarningRedirect: surveys.Count > 0");
                        //A concurrent session has already inserted a record for this survey and it is not new anymore.
                        Response.Redirect("Warning.aspx?err=9", false);
                    } else {
                        _survey.SURVEY_STATUS = "inprocess";

                        _survey.SetAsInsertOnSubmit();

                        _survey.patient_ICN = patientICN;

                        _survey.STD_SURVEY_TYPE_ID = _surveyTypeId;

                        // TODO: change to datepicker
                        _survey.SURVEY_DATE = DateTime.Today;

                        //Save survey if it's new before adding any responses
                        SuppressSaveMessage = true;
                        _tbiInstrumentManager.Update(_survey);
                        SuppressSaveMessage = false;

                        RefreshSurvey(patientICN);
                    }
                }

                SaveSurvey(ref _survey);

                _survey.SetAsUpdateOnSubmit();
                status = _tbiInstrumentManager.Update(_survey);

                (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);

                RefreshSurvey(patientICN);
                SuppressSaveMessage = false;
            }
        }
    }

    /// <summary>
    /// Calculate scores
    /// </summary>
    private void calculateScores() {

        int numOfAnsweredQuestions = 0;
        int totalAnswerValues = 0;
        decimal averageScore = 0;

        if (rblQTYP.SelectedItem.Text.ToLower() == "baseline") {
            #region Calculate Baseline Score

            if (rblQ2.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ2.SelectedItem.Text);
            } 
            
            if (rblQ3.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ3.SelectedItem.Text);
            } 
            
            if (rblQ4.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ4.SelectedItem.Text);
            } 
            
            if (rblQ5.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ5.SelectedItem.Text);
            } 
            
            if (rblQ6.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ6.SelectedItem.Text);
            } 
            
            if (rblQ7.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ7.SelectedItem.Text);
            } 
            
            if (rblQ8.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ8.SelectedItem.Text);
            } 
            
            if (rblQ9.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ9.SelectedItem.Text);
            } 
            
            if (rblQ10.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ10.SelectedItem.Text);
            } 
            
            if (rblQ11.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ11.SelectedItem.Text);
            } 
            
            if (rblQ12.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ12.SelectedItem.Text);
            } 
            
            if (rblQ13.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ13.SelectedItem.Text);
            } 
            
            if (rblQ14.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ14.SelectedItem.Text);
            } 
            
            if (rblQ15.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ15.SelectedItem.Text);
            } 
            
            if (rblQ16.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ16.SelectedItem.Text);
            } 
            
            if (rblQ17.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ17.SelectedItem.Text);
            } 
            
            if (rblQ18.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ18.SelectedItem.Text);
            } 
            
            if (rblQ19.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ19.SelectedItem.Text);
            } 
            
            if (rblQ20.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ20.SelectedItem.Text);
            } 
            
            if (rblQ21.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ21.SelectedItem.Text);
            }

            averageScore = Math.Round(Convert.ToDecimal(totalAnswerValues) / Convert.ToDecimal(numOfAnsweredQuestions), 2);

            txtQ22.Text = averageScore.ToString();

            #endregion
        } else {
            #region Calculate Follow-Up Score

            if (rblQ2.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ2.SelectedItem.Text);
            } 
            
            if (rblQ3.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ3.SelectedItem.Text);
            } 
            
            if (rblQ4.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ4.SelectedItem.Text);
            } 
            
            if (rblQ5.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ5.SelectedItem.Text);
            } 
            
            if (rblQ6.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ6.SelectedItem.Text);
            } 
            
            if (rblQ7.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ7.SelectedItem.Text);
            } 
            
            if (rblQ8.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ8.SelectedItem.Text);
            } 
            
            if (rblQ9.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ9.SelectedItem.Text);
            } 
            
            if (rblQ10.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ10.SelectedItem.Text);
            } 
            
            if (rblQ11.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ11.SelectedItem.Text);
            } 
            
            if (rblQ12.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ12.SelectedItem.Text);
            } 
            
            if (rblQ13.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ13.SelectedItem.Text);
            } 
            
            if (rblQ14.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ14.SelectedItem.Text);
            } 
            
            if (rblQ15.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ15.SelectedItem.Text);
            } 
            
            if (rblQ16.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ16.SelectedItem.Text);
            } 
            
            if (rblQ17.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ17.SelectedItem.Text);
            } 
            
            if (rblQ18.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ18.SelectedItem.Text);
            } 
            
            if (rblQ19.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ19.SelectedItem.Text);
            } 
            
            if (rblQ20.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ20.SelectedItem.Text);
            } 
            
            if (rblQ21.SelectedItem.Text != "0") {
                numOfAnsweredQuestions++;
                totalAnswerValues += Convert.ToInt32(rblQ21.SelectedItem.Text);
            }

            averageScore = Math.Round(Convert.ToDecimal(totalAnswerValues) / Convert.ToDecimal(numOfAnsweredQuestions), 2);

            txtQ23.Text = averageScore.ToString();

            // determine overall change score ...
            if (!String.IsNullOrEmpty(txtQ22.Text)) {
                decimal baselineScore = Convert.ToDecimal(txtQ22.Text);
                decimal overall = Math.Round(averageScore - baselineScore, 2);
                txtQ24.Text = overall.ToString();
            }

            #endregion
        }

    }

    /// <summary>
    /// Save survey.
    /// </summary>
    /// <param name="survey"></param>
    private void SaveSurvey(ref SURVEY survey) {
        List<STD_QUESTION> questions = GetAllQuestionsForSurvey();

        //Delete all the survey results from database if there are any.
        SURVEY surveyWithResults = _tbiInstrumentManager.GetSurveyWithAllAnswers(survey.SURVEYS_ID);
        _tbiInstrumentManager.DeleteAllSurveyResults(surveyWithResults);

        // Question DX
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question DX choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(ctrlDiagnosis.GetSelectedId());
            _surveyResult.RESULT_TEXT = ctrlDiagnosis.GetSelectedResultText();
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question DX");
        }

        // Question D
        _surveyResult = new SURVEY_RESULT();

        if (dpDate1.SelectedDate != null) {
            STD_QUESTION questionD = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "D");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question DTE choice
                _surveyResult.STD_QUESTION_CHOICE_ID = questionD.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = ((DateTime)dpDate1.SelectedDate).ToShortDateString();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question D");
            }
        }

        // Question TYP
        _surveyResult = new SURVEY_RESULT();
        STD_QUESTION questionTYP = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "TYP");
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question TYP choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQTYP.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQTYP.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question TYP");
        }

        // Question 1
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 1 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ1.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ1.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 1");
        }

        // Question 2
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 2 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ2.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ2.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 2");
        }

        // Question 3
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 3 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ3.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ3.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 3");
        }

        // Question 4
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 4 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ4.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ4.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 4");
        }

        // Question 5
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 5 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ5.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ5.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 5");
        }

        // Question 6
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 6 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ6.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ6.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 6");
        }

        // Question 7
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 7 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ7.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ7.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 7");
        }

        // Question 8
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 8 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ8.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ8.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 8");
        }

        // Question 9
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 9 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ9.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ9.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 9");
        }

        // Question 10
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 10 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ10.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ10.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 10");
        }

        // Question 11
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 11 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ11.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ11.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 11");
        }

        // Question 12
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 12 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ12.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ12.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 12");
        }

        // Question 13
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 13 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ13.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ13.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 13");
        }

        // Question 14
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 14 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ14.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ14.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 14");
        }

        // Question 15
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 15 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ15.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ15.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 15");
        }

        // Question 16
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 16 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ16.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ16.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 16");
        }

        // Question 17
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 17 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ17.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ17.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 17");
        }

        // Question 18
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 18 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ18.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ18.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 18");
        }

        // Question 19
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 19 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ19.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ19.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 19");
        }

        // Question 20
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 20 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ20.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ20.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 20");
        }

        // Question 21
        _surveyResult = new SURVEY_RESULT();

        try {
            // Add Question 21 choice
            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ21.SelectedValue);
            _surveyResult.RESULT_TEXT = rblQ21.SelectedItem.Text;
            survey.SURVEY_RESULTs.Add(_surveyResult);
        } catch (Exception surveyEx) {
            Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 21");
        }


        // Question 22
        _surveyResult = new SURVEY_RESULT();

        if (!String.IsNullOrEmpty(txtQ22.Text)) {
            STD_QUESTION question22 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "22");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 22 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question22.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtQ22.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 22");
            }
        }

        // Question 23
        _surveyResult = new SURVEY_RESULT();

        if (!String.IsNullOrEmpty(txtQ23.Text)) {
            STD_QUESTION question23 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "23");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 23 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question23.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtQ23.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 23");
            }
        }

        // Question 24
        _surveyResult = new SURVEY_RESULT();

        if (!String.IsNullOrEmpty(txtQ24.Text)) {
            STD_QUESTION question24 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "24");
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 24 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question24.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtQ24.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "VALowVisionSurvey.aspx", "100027", "SaveSurvey() - Question 24");
            }
        }
    }// END SaveSurvey

    /// <summary>
    /// Initialize/setup controls
    /// </summary>
    /// <param name="survey"></param>
    private void SetUpSurvey(SURVEY survey) {
        try {
            foreach (var sResult in survey.SURVEY_RESULTs) {
                switch (sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER) {
                    case "DX":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            ctrlDiagnosis.SelectedChoiceId = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();

                            if (sResult.STD_QUESTION_CHOICE.CHOICE_TEXT.ToLower().Contains("other")) {
                                ctrlDiagnosis.SetOtherDescriptionText(sResult.RESULT_TEXT);
                            }
                        } else {
                            ctrlDiagnosis.SelectedChoiceId = null;
                        }
                        break;
                    case "D":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            DateTime temp;
                            if (DateTime.TryParse(sResult.RESULT_TEXT, out temp)) {
                                dpDate1.SelectedDate = temp;
                            }
                        }

                        break;
                    case "TYP":
                        if (sResult.RESULT_TEXT.ToLower() == "baseline") {
                            pnlBaselineScores.Visible = true;
                            pnlFollowupScores.Visible = false;
                        } else {
                            pnlBaselineScores.Visible = true;
                            pnlFollowupScores.Visible = true;
                        }

                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQTYP.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQTYP.SelectedIndex = -1;
                        }

                        break;
                    case "1":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ1.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ1.SelectedIndex = -1;
                        }

                        break;
                    case "2":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ2.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ2.SelectedIndex = -1;
                        }

                        break;
                    case "3":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ3.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ3.SelectedIndex = -1;
                        }

                        break;
                    case "4":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ4.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ4.SelectedIndex = -1;
                        }

                        break;
                    case "5":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ5.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ5.SelectedIndex = -1;
                        }

                        break;
                    case "6":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ6.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ6.SelectedIndex = -1;
                        }

                        break;
                    case "7":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ7.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ7.SelectedIndex = -1;
                        }

                        break;
                    case "8":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ8.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ8.SelectedIndex = -1;
                        }

                        break;
                    case "9":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ9.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ9.SelectedIndex = -1;
                        }

                        break;
                    case "10":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ10.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ10.SelectedIndex = -1;
                        }

                        break;
                    case "11":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ11.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ11.SelectedIndex = -1;
                        }

                        break;
                    case "12":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ12.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ12.SelectedIndex = -1;
                        }

                        break;
                    case "13":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ13.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ13.SelectedIndex = -1;
                        }

                        break;
                    case "14":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ14.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ14.SelectedIndex = -1;
                        }

                        break;
                    case "15":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ15.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ15.SelectedIndex = -1;
                        }

                        break;
                    case "16":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ16.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ16.SelectedIndex = -1;
                        }

                        break;
                    case "17":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ17.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ17.SelectedIndex = -1;
                        }

                        break;
                    case "18":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ18.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ18.SelectedIndex = -1;
                        }

                        break;
                    case "19":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ19.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ19.SelectedIndex = -1;
                        }

                        break;
                    case "20":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ20.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ20.SelectedIndex = -1;
                        }

                        break;
                    case "21":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ21.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ21.SelectedIndex = -1;
                        }

                        break;
                    case "22":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtQ22.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "23":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtQ23.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    case "24":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            txtQ24.Text = sResult.RESULT_TEXT;
                        }

                        break;
                    default:
                        break;
                }
            }

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "VALowVisionSurvey.aspx", "100027", "SetUpSurvey()");
        }
    }// END SetUpSurvey

    /// <summary>
    /// Retrieve instrument title
    /// </summary>
    /// <param name="surveyID"></param>
    /// <returns></returns>
    private string GetInstrumentTitle(int surveyID) {
        string ret = String.Empty;

        try {
            List<string> instrumentTitle = _tbiInstrumentManager.GetInstrumentByID(surveyID).Title;
            if (instrumentTitle.Count > 0) {
                foreach (string titleLine in instrumentTitle) {
                    if (titleLine.Contains("PhD")) {
                        ret = ret + "<small>" + titleLine + "</small>" + "<br \\>";
                    } else {
                        ret = ret + titleLine + "<br \\>";
                    }
                }
            }

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "VALowVisionSurvey.aspx", "100027", "GetInstrumentTitle()");
            // Ignore error
        }

        return ret;
    }

    #endregion
}