﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.TBI;
using VeteransAffairs.Registries.BusinessManager.MDWS;
using System.Text;
using System.Runtime.InteropServices;
using Telerik.Web.UI;

public partial class TBI_InstrumentCarePlanEntry : BaseTBIInstrumentPage
{
    #region Variable Declarations

    PatientManager patientManager = new PatientManager();
    InstrumentType instrumentType = new InstrumentType();
    int _surveyTypeId = 5; // Database Id for CAREPLAN 

    SURVEY _survey;   // the root object in the Survey set
    SURVEY_RESULT _surveyResult; // the object representing the actual data in the Survey

    #endregion

    #region Public Methods

    public TBI_InstrumentCarePlanEntry()
    {
        //Manager Save Notification Event Handler
        patientManager.BOSaveSuccess += manager_BOSaveSuccess;
    }

    #endregion Public Methods

    #region Protected Methods
    /// <summary>
    /// In addition to the normal actions taken in Page_Load,
    /// instantiate the MDWS Web Service (T)ransfer (O)bject.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            SetQuestionLabels();
            CompareValidatorQ11_1.ValueToCompare = DateTime.Now.ToString("MM/dd/yyyy");

            if (_tbiInstrumentManager != null)
            {
                LabelBreadCrumb.Text = "TBI Instruments > Instrument Associations > "
                                      + _tbiInstrumentManager.GetInstrumentByID(_surveyTypeId).Name;
            }

            //TBIServiceTO _tbiServiceTo;
            //_tbiServiceTo = (TBIServiceTO)Session["TBIServiceTO"];
            //_tbiInstrumentManager.TbiServiceTo = _tbiServiceTo;
            _tbiInstrumentManager.TbiServiceTo = base.GetTBIServiceTo();

            if (!IsPostBack)
            {
                string _ssn = string.Empty;

                //Check if the request is to start a new item 
                //a new item page request is signified when the querystring parameter id equals zero or is not present
                if (_tbiInstrumentManager.TbiServiceTo == null)
                {
                    //Redirect to message that session is ended and need to log back in from CPRS.
                    //Response.Redirect("Warning.aspx?err=2");
                }
                else
                {
                    base.AddtoViewStateTBIServiceTo(_tbiInstrumentManager.TbiServiceTo);

                    if (instrumentType.Status != null)
                    {
                        _tbiInstrumentManager.TbiServiceTo.SelectedInstrumentType = instrumentType;
                    }

                    _ssn = _tbiInstrumentManager.TbiServiceTo.PatientSSN;

                    LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                    LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN.Trim().Length == 9 ? String.Format("{0:000-00-0000}", int.Parse(_tbiInstrumentManager.TbiServiceTo.PatientSSN)) : _tbiInstrumentManager.TbiServiceTo.PatientSSN;

                    Label labelUser = (Label)Master.FindControl("LabelCurrentUser");
                    if (labelUser != null)
                    {
                        labelUser.Text = _tbiInstrumentManager.TbiServiceTo.UserName;
                    }
                }

                if (!string.IsNullOrEmpty(_ssn))
                {
                    string patICN = patientManager.GetPatientICNBySSN(_ssn);

                    if (!string.IsNullOrEmpty(patICN))
                    {
                        ViewState["PatientICN"] = patICN;
                        RefreshSurvey(patICN);
                    }
                    else
                    {
                        throw new ArgumentException("Patient with this SSN does not exist.");
                    }

                    string referrer = string.Empty;
                    if (Request.UrlReferrer != null)
                    {
                        referrer = Request.UrlReferrer.PathAndQuery;
                    }
                    ViewState["referrer"] = referrer;

                }
            }
        }
        catch (Exception ex2)
        {
            //Redirect to message that session is ended and need to log back in from CPRS.
            Exception _tbilogerror = HandleTBIException(ex2, "InstrumentCarePlanEntry.aspx", "100030", "Page_Load()");
            //Response.Redirect("Warning.aspx?err=2");
        }
    }

    protected override void OnPreRenderComplete(EventArgs e)
    {
        base.OnPreRenderComplete(e);

        bool isInitialNote = false;
        if (_tbiInstrumentManager.TbiServiceTo != null)
        {
            isInitialNote = _tbiInstrumentManager.IsInitialNoteType(_tbiInstrumentManager.TbiServiceTo.PatientICN, _surveyTypeId);
        }

        if (!IsPostBack)
        {
            if (_survey != null)
            {
                if (!_survey.isNew)
                {
                    SetUpSurvey(_survey);
                }
                else
                {
                    if (!isInitialNote)
                    {
                        CarryForwardData(_tbiInstrumentManager.TbiServiceTo.PatientICN);
                    }
                }
            }
        }

        if (_survey != null)
        {
            //Handle note type
            if (isInitialNote)
            {
                if (RadioButtonListNoteType.SelectedIndex == -1)
                {
                    RadioButtonListNoteType.SelectedIndex = 0;
                }

                hiddenNoteType.Value = "0"; //Used by client side validation
                CustomValidatorNoteType.ErrorMessage = "Please select 'Initial' option to correct Note Type.";
            }
            else
            {
                hiddenNoteType.Value = ""; //Used by client side validation
                CustomValidatorNoteType.ErrorMessage = "Please select an option other than 'Initial' to correct Note Type.";
            }
        }

        //Provide Client IDs of repeater controls to custom validators 
        embedClientIDs(ref RepeaterQ4, ref Q4Label);
        embedClientIDs(ref RepeaterQ6, ref Q6Label);
        embedClientIDs(ref RepeaterQ10, ref Q10Label);
    }

    protected override void LoadViewState(object savedState)
    {
        base.LoadViewState(savedState);
        string surveyXml = (string)ViewState["surveyXml"];
        _survey = SerializerUtil.Deserialize<SURVEY>(surveyXml);
    }

    protected override object SaveViewState()
    {
        ViewState["surveyXml"] = _survey.Serialize();
        return base.SaveViewState();
    }


    protected void btnSaveSurveyDraft_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            Int32 status = 0;

            SaveFailMessage = "Save Successful";

            SaveCompleteSurvey(ref status);

            SetUpSurvey(_survey);
        }
    }

    protected void btnSubmit_Click(object sender, EventArgs e)
    {
        try
        {
            if (Page.IsValid)
            {
                Int32 status = 0;

                SaveFailMessage = "Save Successful";

                SaveCompleteSurvey(ref status);

                if (status > 0)
                {
                    Session["survey"] = _survey;
                    Response.Redirect("InstrumentSummaryNote.aspx", false);
                }
                else
                {
                    SetUpSurvey(_survey); //Follow Save Draft logic 
                }
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentCarePlanEntry.aspx", "100030", "btnSubmit_Click()");
        }
    }

    /// <summary>
    /// Cancel button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCancel_Click(object sender, EventArgs e)
    {
        ViewState["surveyXml"] = null;
        ViewState["PatientICN"] = null;
        ViewState["referrer"] = null;
        Session.Remove("survey");
        Response.Redirect("Instrument.aspx", false);
        //this.ClientScript.RegisterStartupScript(typeof(Page), "closePage", "<script type='text/javascript'> window.open('','_self','');window.close(); </script>");
    }

    /// <summary>
    /// Inactive timer firing.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void InActiveTimer_Tick(object sender, EventArgs e)
    {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";

        //Page.Validate();

        //if (Page.IsValid)
        //{
            SaveCompleteSurvey(ref status);
            //SetUpSurvey(_survey);
            Response.Redirect("InstrumentCarePlanEntry.aspx", false);
        //}
        //else
        //{
        //    SaveFailMessage = "Auto-Save Validation Failed. Please complete required fields in the survey and press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        //}
    }

    #endregion

    #region Private Methods

    private void embedClientIDs(ref Repeater rep, ref Label lbl)
    {
        string questionNumber = rep.ID.LastChars(rep.ID.Length - "RepeaterQ".Length);

        string checkBoxName = "CheckBoxQ" + questionNumber;
        string textBoxName = "TextBoxQ" + questionNumber + "Detail";
        string customValidatorDetailName = "CustomValidatorRepeaterQ" + questionNumber + "_Detail";

        for (int i = 0; i < rep.Items.Count; i++)
        {
            CheckBox checkBoxItem = rep.Items[i].FindControl(checkBoxName) as CheckBox;
            TextBox textBoxItem = rep.Items[i].FindControl(textBoxName) as TextBox;
            CustomValidator cValidator_Detail = rep.Items[i].FindControl(customValidatorDetailName) as CustomValidator;

            if (checkBoxItem != null && textBoxItem != null && cValidator_Detail != null)
            {
                try
                {
                    //Save client names of constituent controls in the item template of the repeater as custom attributes of corresponding custom validator
                    ClientScript.RegisterExpandoAttribute(cValidator_Detail.ClientID, "chkId", checkBoxItem.ClientID);
                    ClientScript.RegisterExpandoAttribute(cValidator_Detail.ClientID, "txtId", textBoxItem.ClientID);

                    //Mark Other item in the repeater 
                    if (checkBoxItem.Text == "Other")
                    {
                        ClientScript.RegisterExpandoAttribute(cValidator_Detail.ClientID, "flagOther", "true");
                    }
                    else
                    {
                        ClientScript.RegisterExpandoAttribute(cValidator_Detail.ClientID, "flagOther", "false");
                    }

                    //Save the name of first checkbox in the repeater 
                    if (i == 0)
                    {
                        ClientScript.RegisterExpandoAttribute(lbl.ClientID, "nameControlToValidate", checkBoxItem.ClientID);
                    }
                }
                catch (Exception ex)
                {
                    Exception _tbilogerror = HandleTBIException(ex, "InstrumentCarePlanEntry.aspx", "100030", "embedClientIds()");
                }
            }
        }
    }

    private void Repeater_ItemDataBound(ref object sender, ref RepeaterItemEventArgs e, string questionNumber, string questionLabelName)
    {
        string checkBoxName = "CheckBoxQ" + questionNumber;
        string textBoxName = "TextBoxQ" + questionNumber + "Detail";
        string customValidatorDetailName = "CustomValidatorRepeaterQ" + questionNumber + "_Detail";

        if (e.Item.ItemType == ListItemType.Item || e.Item.ItemType == ListItemType.AlternatingItem)
        {
            CheckBox checkBoxItem = e.Item.FindControl(checkBoxName) as CheckBox;
            TextBox textBoxItem = e.Item.FindControl(textBoxName) as TextBox;
            CustomValidator cValidator_Detail = e.Item.FindControl(customValidatorDetailName) as CustomValidator;

            if (checkBoxItem != null && textBoxItem != null && cValidator_Detail != null)
            {
                try
                {
                    //Save client names of constituent controls in the item template of the repeater as custom attributes of corresponding custom validator
                    ClientScript.RegisterExpandoAttribute(cValidator_Detail.ClientID, "chkId", checkBoxItem.ClientID);
                    ClientScript.RegisterExpandoAttribute(cValidator_Detail.ClientID, "txtId", textBoxItem.ClientID);

                    //Mark Other item in the repeater 
                    if (checkBoxItem.Text == "Other")
                    {
                        ClientScript.RegisterExpandoAttribute(cValidator_Detail.ClientID, "flagOther", "true");
                    }
                    else
                    {
                        ClientScript.RegisterExpandoAttribute(cValidator_Detail.ClientID, "flagOther", "false");
                    }

                    //Save the name of first checkbox in the repeater 
                    if (e.Item.ItemIndex == 0)
                    {
                        ClientScript.RegisterExpandoAttribute(questionLabelName, "nameControlToValidate", checkBoxItem.ClientID);
                    }
                }
                catch (Exception ex)
                {
                    Exception _tbilogerror = HandleTBIException(ex, "InstrumentCarePlanEntry.aspx", "100030", "Repeater_ItemDataBound()");
                }
            }
        }
    }

    private List<STD_QUESTION> GetAllQuestionsForSurvey()
    {
        return _tbiInstrumentManager.SelectAllQuestions(_surveyTypeId).ToList();
    }

    private void RefreshSurvey(string patientICN)
    {
        try
        {
            var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

            if (surveys.Count > 0)
            {
                _survey = surveys.First();
            }
            else
            {
                _survey = new SURVEY();
            }

            _survey.SetAsChangeTrackingRoot();
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentCarePlanEntry.aspx", "100030", "RefreshSurvey()");
        }
    }

    private void CarryForwardData(string patientICN)
    {
        try
        {
            //Compile list of questions for which responses should be carried over from the last completed survey
            List<string> questionList = new List<string>();
            questionList.Add("1");
            questionList.Add("2");
            questionList.Add("2-1");
            questionList.Add("4");
            questionList.Add("5");


            SURVEY lastSurvey = (_tbiInstrumentManager.SelectLastCompletedSurveyByPatientICNAndSurveyType(patientICN, _surveyTypeId)).First();

            if (lastSurvey != null)
            {
                SetUpSurvey(lastSurvey, questionList);
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentCarePlanEntry.aspx", "100030", "CarryForwardData()");
        }
    }

    private void SaveCompleteSurvey(ref Int32 status)
    {
        try
        {
            string patientICN = String.Empty;

            status = 0;

            //Retrieve patient's ICN from the session variable
            if (ViewState["PatientICN"] != null)
            {
                patientICN = ViewState["PatientICN"].ToString();
                if (String.IsNullOrEmpty(patientICN.Trim()))
                {
                    //Lost state. Redirect to message that need to log back in from CPRS.
                    Response.Redirect("Warning.aspx?err=2", false);
                }
            }
            else
            {
                //Lost state. Redirect to message that need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2", false);
            }

            if (_survey != null)
            {
                if (_survey.isNew)
                {
                    //Ensure that a concurrent session hasn't already inserted a record for this survey
                    var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

                    if (surveys.Count > 0)
                    {
                        //A concurrent session has already inserted a record for this survey and it is not new anymore.
                        Response.Redirect("Warning.aspx?err=9", false);
                    }
                    else
                    {
                        _survey.SURVEY_STATUS = "inprocess";

                        _survey.SetAsInsertOnSubmit();

                        _survey.patient_ICN = patientICN;

                        _survey.STD_SURVEY_TYPE_ID = _surveyTypeId;
                        _survey.SURVEY_DATE = DateTime.Today;

                        //Save survey if it's new before adding any responses
                        SuppressSaveMessage = true;
                        _tbiInstrumentManager.Update(_survey);
                        SuppressSaveMessage = false;

                        RefreshSurvey(patientICN);
                    }
                }

                SaveSurvey(ref _survey);

                _survey.SetAsUpdateOnSubmit();
                status = _tbiInstrumentManager.Update(_survey);

                (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);

                RefreshSurvey(patientICN);
                SuppressSaveMessage = false;
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentCarePlanEntry.aspx", "100030", "SaveCompleteSurvey()");
        }
    }

    /// <summary>
    /// Save survey.
    /// </summary>
    /// <param name="survey"></param>
    private void SaveSurvey(ref SURVEY survey)
    {
        try
        {
            Label labelItem;
            CheckBox checkBoxItem;
            TextBox textBoxItem;

            List<STD_QUESTION> questions = GetAllQuestionsForSurvey();

            //Delete all the survey results from database if there are any.
            SURVEY surveyWithResults = _tbiInstrumentManager.GetSurveyWithAllAnswers(survey.SURVEYS_ID);
            _tbiInstrumentManager.DeleteAllSurveyResults(surveyWithResults);

            //Survey header:
            if (RadioButtonListNoteType.SelectedIndex > -1)
            {
                _surveyResult = new SURVEY_RESULT();
                try
                {
                    _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(RadioButtonListNoteType.SelectedValue);
                }
                catch
                {
                    //Should never happen. Ignore error
                }
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }

            //Question 1
            if (!String.IsNullOrEmpty(TextBoxQ1.Text.Trim()))
            {
                STD_QUESTION question1 = questions.FirstOrDefault(s => s.QUESTION_NUMBER == "1");
                _surveyResult = new SURVEY_RESULT();
                _surveyResult.STD_QUESTION_CHOICE_ID = question1.STD_QUESTION_CHOICEs.FirstOrDefault(a => a.CHOICE_SORT_ORDER == 0).STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = TextBoxQ1.Text.Trim();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }

            //Question 2 
            foreach (ListItem item in CheckBoxListQ2.Items)
            {
                if (item.Selected)
                {
                    _surveyResult = new SURVEY_RESULT();

                    _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(item.Value);

                    if (item.Text.ToUpper() == "OTHER")
                        _surveyResult.RESULT_TEXT = TextBoxQ2Detail.Text.Trim();

                    survey.SURVEY_RESULTs.Add(_surveyResult);
                }
            }

            //Question 2-1 - Additional Comments
            if (!String.IsNullOrEmpty(TextBoxQ2Comment.Text.Trim()))
            {
                STD_QUESTION question2_1 = questions.FirstOrDefault(s => s.QUESTION_NUMBER == "2-1");
                _surveyResult = new SURVEY_RESULT();
                _surveyResult.STD_QUESTION_CHOICE_ID = question2_1.STD_QUESTION_CHOICEs.FirstOrDefault(a => a.CHOICE_SORT_ORDER == 0).STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = TextBoxQ2Comment.Text.Trim();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }

            //Question 3 
            foreach (ListItem item in CheckBoxListQ3.Items)
            {
                if (item.Selected)
                {
                    _surveyResult = new SURVEY_RESULT();

                    _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(item.Value);

                    if (item.Text.ToUpper() == "OTHER")
                        _surveyResult.RESULT_TEXT = TextBoxQ3Detail.Text.Trim();

                    survey.SURVEY_RESULTs.Add(_surveyResult);
                }
            }

            //Question 3-1 - Additional Comments
            if (!String.IsNullOrEmpty(TextBoxQ3Comment.Text.Trim()))
            {
                STD_QUESTION question3_1 = questions.FirstOrDefault(s => s.QUESTION_NUMBER == "3-1");
                _surveyResult = new SURVEY_RESULT();
                _surveyResult.STD_QUESTION_CHOICE_ID = question3_1.STD_QUESTION_CHOICEs.FirstOrDefault(a => a.CHOICE_SORT_ORDER == 0).STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = TextBoxQ3Comment.Text.Trim();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }

            //Question 4
            for (int i = 0; i < RepeaterQ4.Items.Count; i++)
            {
                labelItem = RepeaterQ4.Items[i].FindControl("LabelQ4ChoiceID") as Label;
                checkBoxItem = RepeaterQ4.Items[i].FindControl("CheckBoxQ4") as CheckBox;
                textBoxItem = RepeaterQ4.Items[i].FindControl("TextBoxQ4Detail") as TextBox;

                if (labelItem != null && checkBoxItem != null && textBoxItem != null)
                {
                    if (checkBoxItem.Checked)
                    {
                        _surveyResult = new SURVEY_RESULT();

                        _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(labelItem.Text);

                        if (!String.IsNullOrEmpty(textBoxItem.Text.Trim()))
                            _surveyResult.RESULT_TEXT = textBoxItem.Text.Trim();

                        survey.SURVEY_RESULTs.Add(_surveyResult);
                    }
                }
            }

            //Question 5
            if (!String.IsNullOrEmpty(TextBoxQ5.Text.Trim()))
            {
                STD_QUESTION question5 = questions.FirstOrDefault(s => s.QUESTION_NUMBER == "5");
                _surveyResult = new SURVEY_RESULT();
                _surveyResult.STD_QUESTION_CHOICE_ID = question5.STD_QUESTION_CHOICEs.FirstOrDefault(a => a.CHOICE_SORT_ORDER == 0).STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = TextBoxQ5.Text.Trim();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }

            //Question 6
            for (int i = 0; i < RepeaterQ6.Items.Count; i++)
            {
                labelItem = RepeaterQ6.Items[i].FindControl("LabelQ6ChoiceID") as Label;
                checkBoxItem = RepeaterQ6.Items[i].FindControl("CheckBoxQ6") as CheckBox;
                textBoxItem = RepeaterQ6.Items[i].FindControl("TextBoxQ6Detail") as TextBox;

                if (labelItem != null && checkBoxItem != null && textBoxItem != null)
                {
                    if (checkBoxItem.Checked)
                    {
                        _surveyResult = new SURVEY_RESULT();

                        _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(labelItem.Text);

                        if (!String.IsNullOrEmpty(textBoxItem.Text.Trim()))
                            _surveyResult.RESULT_TEXT = textBoxItem.Text.Trim();

                        survey.SURVEY_RESULTs.Add(_surveyResult);
                    }
                }
            }

            //Question 7
            if (RadioButtonListQ7.SelectedIndex > -1)
            {
                _surveyResult = new SURVEY_RESULT();
                try
                {
                    _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(RadioButtonListQ7.SelectedValue);
                    if ((RadioButtonListQ7.Items[RadioButtonListQ7.SelectedIndex].Text.ToUpper() == "OTHER") && (!String.IsNullOrEmpty(TextBoxQ7Detail.Text.Trim())))
                    {
                        _surveyResult.RESULT_TEXT = TextBoxQ7Detail.Text.Trim();
                    }
                }
                catch
                {
                    //Should never happen. Ignore error
                }
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }

            //Question I7-1
            if (RadioButtonListQI7_1.SelectedIndex > -1)
            {
                _surveyResult = new SURVEY_RESULT();
                try
                {
                    _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(RadioButtonListQI7_1.SelectedValue);
                }
                catch
                {
                    //Should never happen. Ignore error
                }
                survey.SURVEY_RESULTs.Add(_surveyResult);

                //Question I7-2
                if ((RadioButtonListQI7_1.Items[RadioButtonListQI7_1.SelectedIndex].Text.ToUpper() == "YES") && (RadioButtonListQI7_2.SelectedIndex > -1))
                {
                    _surveyResult = new SURVEY_RESULT();
                    try
                    {
                        _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(RadioButtonListQI7_2.SelectedValue);
                    }
                    catch
                    {
                        //Should never happen. Ignore error
                    }
                    survey.SURVEY_RESULTs.Add(_surveyResult);
                }
            }

            //Question 8
            if (!String.IsNullOrEmpty(TextBoxQ8.Text.Trim()))
            {
                STD_QUESTION question8 = questions.FirstOrDefault(s => s.QUESTION_NUMBER == "8");
                _surveyResult = new SURVEY_RESULT();
                _surveyResult.STD_QUESTION_CHOICE_ID = question8.STD_QUESTION_CHOICEs.FirstOrDefault(a => a.CHOICE_SORT_ORDER == 0).STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = TextBoxQ8.Text.Trim();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }

            //Question 9
            if (!String.IsNullOrEmpty(TextBoxQ9.Text.Trim()))
            {
                STD_QUESTION question9 = questions.FirstOrDefault(s => s.QUESTION_NUMBER == "9");
                _surveyResult = new SURVEY_RESULT();
                _surveyResult.STD_QUESTION_CHOICE_ID = question9.STD_QUESTION_CHOICEs.FirstOrDefault(a => a.CHOICE_SORT_ORDER == 0).STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = TextBoxQ9.Text.Trim();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }

            //Question 10
            for (int i = 0; i < RepeaterQ10.Items.Count; i++)
            {
                labelItem = RepeaterQ10.Items[i].FindControl("LabelQ10ChoiceID") as Label;
                checkBoxItem = RepeaterQ10.Items[i].FindControl("CheckBoxQ10") as CheckBox;
                textBoxItem = RepeaterQ10.Items[i].FindControl("TextBoxQ10Detail") as TextBox;

                if (labelItem != null && checkBoxItem != null && textBoxItem != null)
                {
                    if (checkBoxItem.Checked)
                    {
                        _surveyResult = new SURVEY_RESULT();

                        _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(labelItem.Text);

                        if (!String.IsNullOrEmpty(textBoxItem.Text.Trim()))
                            _surveyResult.RESULT_TEXT = textBoxItem.Text.Trim();

                        survey.SURVEY_RESULTs.Add(_surveyResult);
                    }
                }
            }

            //Question 11
            if (!String.IsNullOrEmpty(TextBoxQ11.Text.Trim()))
            {
                STD_QUESTION question11 = questions.FirstOrDefault(s => s.QUESTION_NUMBER == "11");
                _surveyResult = new SURVEY_RESULT();
                _surveyResult.STD_QUESTION_CHOICE_ID = question11.STD_QUESTION_CHOICEs.FirstOrDefault(a => a.CHOICE_SORT_ORDER == 0).STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = TextBoxQ11.Text.Trim();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }

            //Question 12
            if (!String.IsNullOrEmpty(TextBoxQ12.Text.Trim()))
            {
                STD_QUESTION question12 = questions.FirstOrDefault(s => s.QUESTION_NUMBER == "12");
                _surveyResult = new SURVEY_RESULT();
                _surveyResult.STD_QUESTION_CHOICE_ID = question12.STD_QUESTION_CHOICEs.FirstOrDefault(a => a.CHOICE_SORT_ORDER == 0).STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = TextBoxQ12.Text.Trim();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentCarePlanEntry.aspx", "100030", "SetUpSurvey()");
        }
    }// END SaveSurvey

    private void SetUpSurvey(SURVEY survey)
    {
        try
        {
            List<string> questionList = new List<string>();

            IEnumerable<STD_QUESTION> allQuestions = _tbiInstrumentManager.SelectAllQuestions(_surveyTypeId);

            //Include all questions
            foreach (var question in allQuestions)
            {
                questionList.Add(question.QUESTION_NUMBER);
            }

            SetUpSurvey(survey, questionList);
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentCarePlanEntry.aspx", "100030", "SetUpSurvey(1)");
        }
    }

    /// <summary>
    /// Initialize/setup controls
    /// </summary>
    /// <param name="survey"></param>
    private void SetUpSurvey(SURVEY survey, List<string> questionList)
    {
        try
        {
            ListItem cblItem;
            int repeaterIndex;

            foreach (var sResult in survey.SURVEY_RESULTs)
            {
                if (questionList.Contains(sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER))
                {
                    switch (sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER)
                    {
                        case "-1":

                            if (sResult.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.HasValue)
                            {
                                RadioButtonListNoteType.SelectedIndex = sResult.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.Value;
                            }
                            else
                            {
                                RadioButtonListNoteType.SelectedIndex = -1;
                            }

                            break;

                        case "1":

                            TextBoxQ1.Text = sResult.RESULT_TEXT;

                            break;

                        case "2":

                            cblItem = CheckBoxListQ2.Items.FindByValue(sResult.STD_QUESTION_CHOICE_ID.ToString());
                            if (cblItem != null)
                            {
                                cblItem.Selected = true;

                                if (cblItem.Text.ToUpper() == "OTHER")
                                {
                                    TextBoxQ2Detail.Text = sResult.RESULT_TEXT;
                                }
                            }
                            else
                            {
                                //Unable to find a checkbox corresponding to saved survey response
                            }

                            break;

                        case "2-1":

                            TextBoxQ2Comment.Text = sResult.RESULT_TEXT;

                            break;

                        case "3":

                            cblItem = CheckBoxListQ3.Items.FindByValue(sResult.STD_QUESTION_CHOICE_ID.ToString());
                            if (cblItem != null)
                            {
                                cblItem.Selected = true;

                                if (cblItem.Text.ToUpper() == "OTHER")
                                {
                                    TextBoxQ3Detail.Text = sResult.RESULT_TEXT;
                                }
                            }
                            else
                            {
                                //Unable to find a checkbox corresponding to saved survey response
                            }

                            break;

                        case "3-1":

                            TextBoxQ3Comment.Text = sResult.RESULT_TEXT;

                            break;

                        case "4":
                            repeaterIndex = GetRepeaterIndexByChoiceID(ref RepeaterQ4, sResult.STD_QUESTION_CHOICE_ID.ToString());
                            if (repeaterIndex != -1)
                            {
                                CheckBox cbItem = RepeaterQ4.Items[repeaterIndex].FindControl("CheckBoxQ4") as CheckBox;
                                TextBox txtItem = RepeaterQ4.Items[repeaterIndex].FindControl("TextBoxQ4Detail") as TextBox;

                                if (cbItem != null && txtItem != null)
                                {
                                    cbItem.Checked = true;
                                    txtItem.Text = sResult.RESULT_TEXT;
                                }
                                else
                                {
                                    //Should never happen
                                }
                            }
                            else
                            {
                                //Unable to find a repeater item corresponding to saved result
                            }

                            break;

                        case "5":

                            TextBoxQ5.Text = sResult.RESULT_TEXT;

                            break;

                        case "6":

                            repeaterIndex = GetRepeaterIndexByChoiceID(ref RepeaterQ6, sResult.STD_QUESTION_CHOICE_ID.ToString());
                            if (repeaterIndex != -1)
                            {
                                CheckBox cbItem = RepeaterQ6.Items[repeaterIndex].FindControl("CheckBoxQ6") as CheckBox;
                                TextBox txtItem = RepeaterQ6.Items[repeaterIndex].FindControl("TextBoxQ6Detail") as TextBox;

                                if (cbItem != null && txtItem != null)
                                {
                                    cbItem.Checked = true;
                                    txtItem.Text = sResult.RESULT_TEXT;
                                }
                                else
                                {
                                    //Should never happen
                                }
                            }
                            else
                            {
                                //Unable to find a repeater item corresponding to saved result
                            }

                            break;

                        case "7":

                            if (sResult.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.HasValue)
                            {
                                RadioButtonListQ7.SelectedIndex = sResult.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.Value;

                                if (RadioButtonListQ7.SelectedIndex == RadioButtonListQ7.Items.Count - 1)
                                {
                                    TextBoxQ7Detail.Text = sResult.RESULT_TEXT;
                                }
                            }
                            else
                            {
                                RadioButtonListQ7.SelectedIndex = -1;
                            }

                            break;

                        case "I7-1":

                            if (sResult.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.HasValue)
                            {
                                RadioButtonListQI7_1.SelectedIndex = sResult.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.Value;
                            }
                            else
                            {
                                RadioButtonListQI7_1.SelectedIndex = -1;
                            }

                            break;

                        case "I7-2":

                            if (sResult.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.HasValue)
                            {
                                RadioButtonListQI7_2.SelectedIndex = sResult.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.Value;
                            }
                            else
                            {
                                RadioButtonListQI7_2.SelectedIndex = -1;
                            }

                            break;

                        case "8":

                            TextBoxQ8.Text = sResult.RESULT_TEXT;

                            break;

                        case "9":

                            TextBoxQ9.Text = sResult.RESULT_TEXT;

                            break;

                        case "10":

                            repeaterIndex = GetRepeaterIndexByChoiceID(ref RepeaterQ10, sResult.STD_QUESTION_CHOICE_ID.ToString());
                            if (repeaterIndex != -1)
                            {
                                CheckBox cbItem = RepeaterQ10.Items[repeaterIndex].FindControl("CheckBoxQ10") as CheckBox;
                                TextBox txtItem = RepeaterQ10.Items[repeaterIndex].FindControl("TextBoxQ10Detail") as TextBox;

                                if (cbItem != null && txtItem != null)
                                {
                                    cbItem.Checked = true;
                                    txtItem.Text = sResult.RESULT_TEXT;
                                }
                                else
                                {
                                    //Should never happen
                                }
                            }
                            else
                            {
                                //Unable to find a repeater item corresponding to saved result
                            }

                            break;

                        case "11":

                            TextBoxQ11.Text = sResult.RESULT_TEXT;

                            break;

                        case "12":

                            TextBoxQ12.Text = sResult.RESULT_TEXT;

                            break;

                    }
                }
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "InstrumentCarePlanEntry.aspx", "100030", "SetUpSurvey()");
        }
    }// END SaveSurvey(...)

    private void SetQuestionLabels()
    {
        if (_tbiInstrumentManager != null)
        {
            NoteTypeLabel.Text = _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("-1", _surveyTypeId) + ":";
            Q1Label.Text = "1. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("1", _surveyTypeId);
            Q2Label.Text = "2. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("2", _surveyTypeId);
            Q3Label.Text = "3. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("3", _surveyTypeId);
            Q4Label.Text = "4. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("4", _surveyTypeId);
            Q5Label.Text = "5. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("5", _surveyTypeId);
            Q6Label.Text = "6. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("6", _surveyTypeId);
            Q7Label.Text = "7. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("7", _surveyTypeId);
            QI7_1Label.Text = _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("I7-1", _surveyTypeId);
            QI7_2Label.Text = _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("I7-2", _surveyTypeId);
            Q8Label.Text = "8. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("8", _surveyTypeId);
            Q9Label.Text = "9. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("9", _surveyTypeId);
            Q10Label.Text = "10. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("10", _surveyTypeId);
            Q11Label.Text = "11. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("11", _surveyTypeId);
            Q12Label.Text = "12. " + _tbiInstrumentManager.GetQuestionbyNumberAndSurveyType("12", _surveyTypeId);
        }
    }

    private int GetRepeaterIndexByChoiceID(ref Repeater rep, string choiceID)
    {
        int ret = -1;
        string questionID = rep.ID.LastChars(rep.ID.Length - "RepeaterQ".Length);

        string labelName = "LabelQ" + questionID + "ChoiceID";

        for (int i = 0; i < rep.Items.Count; i++)
        {
            Label lblItem = rep.Items[i].FindControl(labelName) as Label;

            if (lblItem != null)
            {
                if (lblItem.Text == choiceID)
                {
                    ret = i;
                    break;
                }
            }
        }

        return ret;
    }

    #endregion

    #region Auto-generated Events
    // CAUTION: Although unused their removal will trigger errors because of their absence from the web-page
    protected void RadioButtonListFatigue_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListSleepDifficulty_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListAnxiety_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListDepression_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListIrritability_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListFrustration_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void CheckBoxGrenade_CheckedChanged(object sender, EventArgs e)
    {

    }
    protected void rblBlastWaveExperience_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblBlastWaveWindExperience_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblBlastOccurrences_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblEstimatedBlastDistance_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListProblemsWithPain_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void cbxListProblemsWithPain_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListOverallInterferenceWithLife_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void cblProblemsWithPain_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListPainInterferenceWithLife_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListOverallSymptomsSinceLastEval_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblExperiencedHeadInjury_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblDurationLossOfConsciousness_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void cblTypeOfBlastExposures_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void tbxYearOfRecentHeadInjury_TextChanged(object sender, EventArgs e)
    {
        //string tbxHeadInjuryYr = sender.ToString();
    }
    protected void rblHearingDifficulty_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblBullet_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblVehicular_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblFall_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblBlastWaveCausingViolentImpact_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblBlastExplosionCausingInjuries_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblLossOfConsciousness_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblDisorientationOrConfusion_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void tbxSymptoms_TextChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListClinicalJudgement_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void RadioButtonListFollowUpPlan_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblExperiencedNewTbi_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblFollowUpPlan_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblDurationOfDisorientationOrConfusion_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblLossOfMemory_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblOverallInterferenceWithLife_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblProblemsWithPain_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
    protected void rblPainInterferenceWithLife_SelectedIndexChanged(object sender, EventArgs e)
    {

    }

    #endregion Auto-generated Events

    #region Object/Form Mapping
    /*********************************************************** 
    ** Unused code that was explored as a potential alternative
    ************************************************************/
    //private void LoadControlsFromObject()
    //{
    //    //SURVEY _survey;

    //    //PATIENT _patient;

    //    foreach (var sResult in survey.SURVEY_RESULTs)
    //    {
    //          switch (sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER)
    //          {
    //              case "A":

    //                  _surveyResult.STD_QUESTION_CHOICE_ID = 4708;
    //                  _surveyResult.RESULT_TEXT = TextBoxChiefComplaint.Text;

    //                  break;

    //              case "B":

    //                  _surveyResult.STD_QUESTION_CHOICE_ID = 4710;
    //                  _surveyResult.RESULT_TEXT = TextBoxHistoryPresentIllness.Text;

    //                  break;

    //              case "1":

    //                  _surveyResult.STD_QUESTION_CHOICE_ID = System.Convert.ToInt32(RadioButtonListChangeInMaritalStatus.SelectedItem.Value);
    //                  _surveyResult.RESULT_TEXT = RadioButtonListChangeInMaritalStatus.SelectedItem.Text;

    //                  break;
    //              //Ques
    //              case "4-A":

    //                  if (rblExperiencedHeadInjury.SelectedItem.Value == "4736")
    //                  {
    //                      _surveyResult.STD_QUESTION_CHOICE_ID = 4739;
    //                      _surveyResult.RESULT_TEXT = "";
    //                  }
    //                  else
    //                  {
    //                      _surveyResult.STD_QUESTION_CHOICE_ID = System.Convert.ToInt32(ddlMonthOfRecentHeadInjury.SelectedItem.Text);
    //                      _surveyResult.RESULT_TEXT = ddlMonthOfRecentHeadInjury.SelectedItem.Text;
    //                  }
    //                  break;
    //          }

    //    }


    //}

    //    private void LoadObjectFromControls()
    //    {
    //        foreach (var sResult in _survey.SURVEY_RESULTs)
    //        {

    //            switch ( sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER )
    //            {
    //                case "A":
    //        
    //                  _surveyResult.STD_QUESTION_CHOICE_ID = 4708;
    //                  _surveyResult.RESULT_TEXT = TextBoxChiefComplaint.Text;

    //                  break;

    //         case "B":
    //        _surveyResult.STD_QUESTION_CHOICE_ID = 4710;
    //        _surveyResult.RESULT_TEXT = TextBoxHistoryPresentIllness.Text;

    //        // Question "1":
    //        _surveyResult.STD_QUESTION_CHOICE_ID = System.Convert.ToInt32(RadioButtonListChangeInMaritalStatus.SelectedItem.Value);
    //        _surveyResult.RESULT_TEXT = RadioButtonListChangeInMaritalStatus.SelectedItem.Text;

    //        //Question "2"


    //        //Question "3"


    //        Case "4-A": // & "4-B"
    //        if (rblExperiencedHeadInjury.SelectedItem.Value == "4736")
    //        {
    //            _surveyResult.STD_QUESTION_CHOICE_ID = 4739;
    //            _surveyResult.RESULT_TEXT = "Not asked (due to responses to other questions)";

    //            _surveyResult.STD_QUESTION_CHOICE_ID = 4741;
    //            _surveyResult.RESULT_TEXT = "Not asked (due to responses to other questions)";
    //        }
    //        else
    //        {
    //            _surveyResult.STD_QUESTION_CHOICE_ID = System.Convert.ToInt32(ddlMonthOfRecentHeadInjury.SelectedItem.Value);
    //            _surveyResult.RESULT_TEXT = ddlMonthOfRecentHeadInjury.SelectedItem.Text;

    //             _surveyResult.STD_QUESTION_CHOICE_ID = 4740; 
    //            _surveyResult.RESULT_TEXT = tbxYearOfRecentHeadInjury.Text;
    //        }

    //        //Ques "5-A"
    //        //_surveyResult


    //               //}

    //           // }

    //    //    JH 10/12/09 - Commenting next section : Client will use SSN within the polytrauma application instead of VPID
    //    //    Patient ID
    //    //    if item is new and patient is selected then add Patient ID to polytrauma entry
    //    //    if (HiddenFieldPatientID.Value != "" && polyTraumEntry.PATIENT_ID == null)
    //    //    {
    //    //        polyTraumEntry = PolyTraumaManager.getNewPatient
    //    //            (Helpers.getIntFromString(HiddenFieldPatientID.Value),polyTraumEntry);

    //    //        polyTraumEntry.PATIENT_ID = Helpers.getIntFromString(HiddenFieldPatientID.Value);
    //    //    }




    //    //    Injury Details
    //    //    foreach (ListItem checkboxListInjury in CheckBoxListInjuryDetails.Items)
    //    //    {
    //    //        for each item in the check box list check if it is an entity in TBI_INJURY_DETAILs
    //    //        TBI_INJURY_DETAIL existingInjury = (from e in polyTraumEntry.TBI_INJURY_DETAILs
    //    //                                           where e.STD_TBIINJURYLIST_ID == Helpers.getIntFromString(checkboxListInjury.Value)
    //    //                                           select e).FirstOrDefault();

    //    //        check if the checkboxlist item was selected,  
    //    //        if (checkboxListInjury.Selected)
    //    //        {
    //    //            if the item is selected then it needs to be in TBI_INJURY_DETAILs
    //    //            if the entity exists, that signifies the item is already in TBI_INJURY_DETAILs therefore nothing needs to be done. 
    //    //            if (existingInjury == null)
    //    //            {
    //    //                TBI_INJURY_DETAIL newInjury = new TBI_INJURY_DETAIL();
    //    //                newInjury.TBI_POLYTRAUMA_ID = polyTraumEntry.TBI_POLYTRAUMA_ID;
    //    //                newInjury.STD_TBIINJURYLIST_ID = Helpers.getIntFromString(checkboxListInjury.Value);

    //    //                polyTraumEntry.TBI_INJURY_DETAILs.Add(newInjury);
    //    //            }
    //    //        }
    //    //        else
    //    //        {
    //    //            if the item is not selected, then it needs to be taken out of TBI_INJURY_DETAILs if it is in the list
    //    //            if (existingInjury != null)
    //    //            {
    //    //                existingInjury.SetAsDeleteOnSubmit();
    //    //                polyTraumEntry.TBI_INJURY_DETAILs.Remove(existingInjury);
    //    //            }
    //    //        }
    //    //    }


    //    //    Followup
    //    //    foreach (ListItem checkboxListFollowup in CheckBoxListFollowUps.Items)
    //    //    {

    //    //        for each item in the check box list check if it is an entity in TBI_FOLLOWUPs
    //    //        TBI_FOLLOWUP existingFollowup = (from e in polyTraumEntry.TBI_FOLLOWUPs
    //    //                                         where e.STD_TBIIFOLLOWUPLIST_ID == Helpers.getIntFromString(checkboxListFollowup.Value)
    //    //                                            select e).FirstOrDefault();

    //    //        check if the checkboxlist item was selected,  
    //    //        if (checkboxListFollowup.Selected)
    //    //        {
    //    //            if the item is selected then it needs to be in TBI_FOLLOWUPs
    //    //            if the entity exists, that signifies the item is already in TBI_FOLLOWUPs therefore nothing needs to be done. 
    //    //            if (existingFollowup == null)
    //    //            {
    //    //                TBI_FOLLOWUP newFollowup = new TBI_FOLLOWUP();
    //    //                newFollowup.TBI_POLYTRAUMA_ID = polyTraumEntry.TBI_POLYTRAUMA_ID;
    //    //                newFollowup.STD_TBIIFOLLOWUPLIST_ID = Helpers.getIntFromString(checkboxListFollowup.Value);

    //    //                polyTraumEntry.TBI_FOLLOWUPs.Add(newFollowup);
    //    //            }
    //    //        }
    //    //        else
    //    //        {
    //    //            if the item is not selected, then it needs to be taken out of TBI_FOLLOWUPs if it is in the list
    //    //            if (existingFollowup != null)
    //    //            {
    //    //                existingFollowup.SetAsDeleteOnSubmit();
    //    //                polyTraumEntry.TBI_FOLLOWUPs.Remove(existingFollowup);
    //    //            }
    //    //        }
    //    //    }


    //}
    //private void SetDatesRangeValidators(bool loadControlsFromObject)
    //{
    //    Load Controls From Object
    //    if (loadControlsFromObject)
    //    {
    //            RangeValidatorTextBoxDateOfDeath.MinimumValue =
    //                !string.IsNullOrEmpty(TextBoxDateOfBirth.Text) ?
    //                      TextBoxDateOfBirth.Text : DateTime.MinValue.ToShortDateString();

    //            RangeValidatorTextBoxDateOfBirth.MaximumValue =
    //                !string.IsNullOrEmpty(TextBoxDateOfDeath.Text) ?
    //                     TextBoxDateOfDeath.Text : DateTime.Now.ToShortDateString();
    //        RangeValidatorTextBoxDateOfBirth.MinimumValue = DateTime.MinValue.ToShortDateString();
    //        RangeValidatorTextBoxDateOfBirth.MaximumValue = DateTime.Now.ToShortDateString();

    //            RangeValidatorTextBoxDateOfDeath.MaximumValue = DateTime.Now.ToShortDateString();
    //        RangeValidatorTextBoxDateOfBirth.MinimumValue = DateTime.MinValue.ToShortDateString();
    //    }
    //    //Load Object From Controls 
    //    else
    //    {
    //            RangeValidatorTextBoxDateOfDeath.MinimumValue =
    //                !string.IsNullOrEmpty(TextBoxDateOfBirth.Text) ?
    //                      TextBoxDateOfBirth.Text : DateTime.MinValue.ToShortDateString();

    //            RangeValidatorTextBoxDateOfBirth.MaximumValue =
    //                !string.IsNullOrEmpty(TextBoxDateOfDeath.Text) ?
    //                     TextBoxDateOfDeath.Text : DateTime.Now.ToShortDateString();
    //        RangeValidatorTextBoxDateOfBirth.MinimumValue = DateTime.MinValue.ToShortDateString();
    //        RangeValidatorTextBoxDateOfBirth.MaximumValue = DateTime.Now.ToShortDateString();
    //    }
    //}

    //private void TransferPatientInformationToNewPolyTraumaEntry(PATIENT patientEntry)
    //{
    //    Patient ID
    //    polyTraumEntry.PATIENT = patientEntry;
    //    HiddenFieldPatientID.Value = patientEntry.PATIENT_ID.ToString();

    //    SSN
    //    polyTraumEntry.SSN = patientEntry.SSN;


    //}

    #endregion Object/Form Mapping

}