﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.TBI;

public partial class TBI_Instruments_MayoPortlandAdaptabilityInventory4 : BaseTBIInstrumentPage {
    #region Variable Declarations

    PatientManager patientManager = new PatientManager();
    InstrumentType instrumentType = new InstrumentType();

    protected int _surveyTypeId = 19; // Database Id for Mayo-Portland

    SURVEY _survey;   // the root object in the Survey set
    SURVEY_RESULT _surveyResult; // the object representing the actual data in the Survey

    List<STD_QUESTION> QuestionList = new List<STD_QUESTION>();

    #endregion

    /// <summary>
    /// Default Constructor
    /// </summary>
    public TBI_Instruments_MayoPortlandAdaptabilityInventory4() {
        //Manager Save Notification Event Handler
        patientManager.BOSaveSuccess += manager_BOSaveSuccess;
    }

    #region Protected Methods

    /// <summary>
    /// Page Load method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e) {
        TBIServiceTO _tbiServiceTo;

        try {
            _tbiServiceTo = (TBIServiceTO)Session["TBIServiceTO"];
            _tbiInstrumentManager.TbiServiceTo = _tbiServiceTo;

            string instTitle = _tbiInstrumentManager.GetInstrumentByID(_surveyTypeId).Name;

            LabelBreadCrumb.Text = "TBI Instruments > Instrument Associations > " + instTitle;
            LabelInstrumentTitle.Text = instTitle;
            Page.Title = "TBI Instruments - " + instTitle;

            if (_tbiInstrumentManager.TbiServiceTo != null) {
                Label labelUser = (Label)Master.FindControl("LabelCurrentUser");
                labelUser.Text = _tbiInstrumentManager.TbiServiceTo.UserName;

                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }
        } catch (Exception ex) {
            //Redirect to message that session is ended and need to log back in from CPRS.
            Exception _tbilogerror = HandleTBIException(ex, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SESSIOENDED");
            Response.Redirect("Warning.aspx?err=2", false);
        }

        if (!IsPostBack) {
            if (instrumentType.Status != null) {
                _tbiInstrumentManager.TbiServiceTo.SelectedInstrumentType = instrumentType;
            }

            string _ssn = "";
            if (_tbiInstrumentManager.TbiServiceTo != null) {
                _ssn = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }

            if (!string.IsNullOrEmpty(_ssn)) {
                string patICN = patientManager.GetPatientICNBySSN(_ssn);

                if (!string.IsNullOrEmpty(patICN)) {
                    ViewState["PatientICN"] = patICN;
                    RefreshSurvey(patICN);
                } else {
                    throw new ArgumentException("Patient with this SSN does not exist.");
                }

                if (_survey != null && !_survey.isNew) {
                    SetUpSurvey(_survey);
                    //calculateTotal();
                }
                string referrer = "";
                if (Request.UrlReferrer != null) {
                    referrer = Request.UrlReferrer.PathAndQuery;
                }
                ViewState["referrer"] = referrer;
            }

            //Check if the request is to start a new item 
            //a new item page request is signified when the querystring parameter id equals zero or is not present
            if (_tbiInstrumentManager.TbiServiceTo == null) {
                //Redirect to message that session is ended and need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            } else {
                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN.Trim().Length == 9 ? String.Format("{0:000-00-0000}", int.Parse(_tbiInstrumentManager.TbiServiceTo.PatientSSN)) : _tbiInstrumentManager.TbiServiceTo.PatientSSN;
                //LabelFacility.Text = _tbiInstrumentManager.TbiServiceTo.InstitutionName;
            }
        }
        QuestionList = GetAllQuestionsForSurvey();

        // Set control values for Diagnosis Question
        ctrlDiagnosis.QuestionText = GetQuestionText("DX");
        ctrlDiagnosis.SurveyTypeCode = HiddenSurveyType.Value;

        LabelDate1.Text = GetQuestionText("D");
        LabelForQRPT.Text = GetQuestionText("QRPT");
        //LabelForQRPT1.Text = GetQuestionText("QRPT1");
        LabelForQ1.Text = GetQuestionText("1");
        LabelForQ2.Text = GetQuestionText("2");
        LabelForQ3.Text = GetQuestionText("3");
        LabelForQ4.Text = GetQuestionText("4");
        LabelForQ5.Text = GetQuestionText("5");
        LabelForQ6.Text = GetQuestionText("6");
        LabelForQ7A.Text = GetQuestionText("7A");
        LabelForQ7B.Text = GetQuestionText("7B");
        LabelForQ8.Text = GetQuestionText("8");
        LabelForQ9.Text = GetQuestionText("9");
        LabelForQ10.Text = GetQuestionText("10");
        LabelForQ11.Text = GetQuestionText("11");
        LabelForQ12.Text = GetQuestionText("12");
        LabelForQ13.Text = GetQuestionText("13");
        LabelForQ14.Text = GetQuestionText("14");
        LabelForQ15.Text = GetQuestionText("15");
        LabelForQ16.Text = GetQuestionText("16");
        LabelForQ17.Text = GetQuestionText("17");
        LabelForQ18.Text = GetQuestionText("18");
        LabelForQ19.Text = GetQuestionText("19");
        LabelForQ20.Text = GetQuestionText("20");
        LabelForQ21.Text = GetQuestionText("21");
        LabelForQ22.Text = GetQuestionText("22");
        LabelForQ23.Text = GetQuestionText("23");
        LabelForQ24.Text = GetQuestionText("24");
        LabelForQ25.Text = GetQuestionText("25");
        LabelForQ26.Text = GetQuestionText("26");
        LabelForQ27.Text = GetQuestionText("27");
        LabelForQ28A.Text = GetQuestionText("28A");
        LabelForQ28B.Text = GetQuestionText("28B");
        LabelForQ28B1.Text = GetQuestionText("28B1");
        LabelForQ29.Text = GetQuestionText("29");
        LabelForQ30PRE.Text = GetQuestionText("30PRE");
        LabelForQ30POST.Text = GetQuestionText("30POST");
        LabelForQ31PRE.Text = GetQuestionText("31PRE");
        LabelForQ31POST.Text = GetQuestionText("31POST");
        LabelForQ32PRE.Text = GetQuestionText("32PRE");
        LabelForQ32POST.Text = GetQuestionText("32POST");
        LabelForQ33PRE.Text = GetQuestionText("33PRE");
        LabelForQ33POST.Text = GetQuestionText("33POST");
        LabelForQ34PRE.Text = GetQuestionText("34PRE");
        LabelForQ34POST.Text = GetQuestionText("34POST");
        LabelForQ35PRE.Text = GetQuestionText("35PRE");
        LabelForQ35POST.Text = GetQuestionText("35POST");
        LabelForQ36Comment.Text = GetQuestionText("36");
        LabelForQ37.Text = GetQuestionText("37");
        LabelForQ38.Text = GetQuestionText("38");
        LabelForQ39.Text = GetQuestionText("39");
        LabelForQ40.Text = GetQuestionText("40");
        LabelForQ41.Text = GetQuestionText("41");
        LabelForQ42.Text = GetQuestionText("42");
        LabelForQ43.Text = GetQuestionText("43");
        LabelForQ44.Text = GetQuestionText("44");
        LabelForQ45.Text = GetQuestionText("45");
        LabelForQ46.Text = GetQuestionText("46");

        dpDate1.MaxDate = DateTime.Now;

        lblErrorQRPT1.Text = "";
        lblError28AB.Text = "";
    }

    protected void Q28aOnSelectedIndexChanged(object sender, EventArgs e)
    {
        rblQ28B.SelectedIndex = -1;
        rblQ28B1.SelectedIndex = -1;
        plxhldrQ28B1.Visible = false;
    }

    protected void Q28bOnSelectedIndexChanged(object sender, EventArgs e)
    {
        rblQ28A.SelectedIndex = -1;
        plxhldrQ28B1.Visible = true;
    }

    protected override void LoadViewState(object savedState) {
        base.LoadViewState(savedState);
        string surveyXml = (string)ViewState["surveyXml"];
        _survey = SerializerUtil.Deserialize<SURVEY>(surveyXml);
    }

    protected override object SaveViewState() {
        ViewState["surveyXml"] = _survey.Serialize();
        return base.SaveViewState();
    }

    /// <summary>
    /// Returns the Question_Text of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected string GetQuestionText(string questionId) {
        foreach (STD_QUESTION ques in QuestionList) {
            if (ques.QUESTION_NUMBER.ToLower() == questionId.ToLower()) {
                return ques.QUESTION_TEXT;
            }
        }

        return "";
    }

    /// <summary>
    /// Returns the Question_Id of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected int GetQuestionId(string questionNumber) {
        foreach (STD_QUESTION ques in QuestionList) {
            if (ques.QUESTION_NUMBER.ToLower() == questionNumber.ToLower()) {
                return ques.ID;
            }
        }

        return 0;
    }

    /// <summary>
    /// Save Draft button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public void btnSaveSurveyDraft_Click(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);
        SetUpSurvey(_survey);
    }

    /// <summary>
    /// Save and Prepare Note button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnSubmit_Click(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSubmitSurvey.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);

        if (status > 0) {
            Session["survey"] = _survey;
            Response.Redirect("InstrumentSummaryNote.aspx", false);
        } else {
            SetUpSurvey(_survey); //Follow Save Draft logic 
        }
    }

    /// <summary>
    /// Inactive timer firing.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void InActiveTimer_Tick(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";

        Page.Validate();

        if (Page.IsValid) {
            SaveCompleteSurvey(ref status);
            //SetUpSurvey(_survey);
            Response.Redirect("MayoPortlandAdaptabilityInventory4.aspx", false);
        } else {
            SaveFailMessage = "Auto-Save Validation Failed. Please complete required fields in the survey and press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        }
    }

    /// <summary>
    /// Cancel button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCancel_Click(object sender, EventArgs e) {
        ViewState["surveyXml"] = null;
        ViewState["PatientICN"] = null;
        ViewState["referrer"] = null;
        Session.Remove("survey");
        Response.Redirect("Instrument.aspx", false);
        //this.ClientScript.RegisterStartupScript(typeof(Page), "closePage", "<script type='text/javascript'> window.open('','_self','');window.close(); </script>");
    }

    /// <summary>
    /// Hide or Show instructions
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void lnkBtnToggleInstructions_OnClick(object sender, EventArgs e) {
        if (PanelInstructions.Visible) {
            PanelInstructions.Visible = false;
            lnkBtnToggleInstructions.Text = "Click for Instructions";
        } else {
            PanelInstructions.Visible = true;
            lnkBtnToggleInstructions.Text = "Hide Instructions";
        }
    }

    /// <summary>
    /// Calculate button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCalculate_Click(object sender, EventArgs e) {
        // perform calculation
        calculateTotal();
    }

    /// <summary>
    /// Calculate total score.
    /// </summary>
    protected void calculateTotal() {
        bool isSurveyCompletelyFilledOut = true;

        double abilitySubTotal = 0;
        double adjustmentSubTotal = 0;
        double participationSubTotal = 0;
        double subtotalOfSubscaleScores = 0;
        double subTotal22to24 = 0;
        double total = 0;
        double abilitiesStandardScore = 0;
        double adjustmentStandardScore = 0;
        double participationStandardScore = 0;
        double totalStandardScore = 0;
        STD_QUESTION_CHOICE choice;

        double q4score = 0;
        double q4newScore = 0;
        double q7aScore = 0;
        double q7bScore = 0;
        double q16score = 0;
        double q16newScore = 0;
        double q27score = 0;
        double q27newScore = 0;

        foreach (STD_QUESTION question in QuestionList) {
            switch (question.QUESTION_NUMBER) {
                // Ability Subscale

                case "1":
                    if (!String.IsNullOrEmpty(rblQ1.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ1.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;

                case "2":
                    if (!String.IsNullOrEmpty(rblQ2.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ2.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "3":
                    if (!String.IsNullOrEmpty(rblQ3.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ3.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "4":
                    if (!String.IsNullOrEmpty(rblQ4.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ4.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            q4score = Convert.ToDouble(choice.CHOICE_NAME);
                            if (q4score == 0) {
                                q4newScore = 0;
                            } else if ((q4score == 1) || (q4score == 2) || (q4score == 3)) {
                                q4newScore = 1;
                            } else if (q4score == 4) {
                                q4newScore = 3;
                            }
                            abilitySubTotal += q4newScore;
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "5":
                    if (!String.IsNullOrEmpty(rblQ5.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ5.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "6":
                    if (!String.IsNullOrEmpty(rblQ6.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ6.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "7A":
                    if (!String.IsNullOrEmpty(rblQ7A.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ7A.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            q7aScore = Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "7B":
                    if (!String.IsNullOrEmpty(rblQ7B.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ7B.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            q7bScore = Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "8":
                    if (!String.IsNullOrEmpty(rblQ8.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ8.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "9":
                    if (!String.IsNullOrEmpty(rblQ9.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ9.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "10":
                    if (!String.IsNullOrEmpty(rblQ10.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ10.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "11":
                    if (!String.IsNullOrEmpty(rblQ11.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ11.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "12":
                    if (!String.IsNullOrEmpty(rblQ12.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ12.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            abilitySubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;



                // Adjustment Subscale

                case "13":
                    if (!String.IsNullOrEmpty(rblQ13.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ13.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "14":
                    if (!String.IsNullOrEmpty(rblQ14.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ14.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "15":
                    if (!String.IsNullOrEmpty(rblQ15.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ15.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "16":
                    if (!String.IsNullOrEmpty(rblQ16.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ16.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            q16score = Convert.ToDouble(choice.CHOICE_NAME);
                            if (q16score == 0) {
                                q16newScore = 0;
                            } else if ((q16score == 1) || (q16score == 2)) {
                                q16newScore = 1;
                            } else if ((q16score == 3) || (q16score == 4)) {
                                q16newScore = 2;
                            }
                            adjustmentSubTotal += q16newScore;
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "17":
                    if (!String.IsNullOrEmpty(rblQ17.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ17.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "18":
                    if (!String.IsNullOrEmpty(rblQ18.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ18.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "19":
                    if (!String.IsNullOrEmpty(rblQ19.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ19.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "20":
                    if (!String.IsNullOrEmpty(rblQ20.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ20.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "21":
                    if (!String.IsNullOrEmpty(rblQ21.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ21.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;


                // Both Adjustment and Participation Subscale

                case "22":
                    if (!String.IsNullOrEmpty(rblQ22.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ22.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                            participationSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                            subTotal22to24 += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "23":
                    if (!String.IsNullOrEmpty(rblQ23.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ23.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                            participationSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                            subTotal22to24 += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "24":
                    if (!String.IsNullOrEmpty(rblQ24.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ24.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            adjustmentSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                            participationSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                            subTotal22to24 += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;


                // Participation-only Subscale

                case "25":
                    if (!String.IsNullOrEmpty(rblQ25.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ25.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            participationSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "26":
                    if (!String.IsNullOrEmpty(rblQ26.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ26.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            participationSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;
                case "27":
                    if (!String.IsNullOrEmpty(rblQ27.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ27.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            q27score = Convert.ToDouble(choice.CHOICE_NAME);
                            if ((q27score == 0) || (q27score == 1)) {
                                q27newScore = 0;
                            } else if ((q27score == 2) || (q27score == 3)) {
                                q27newScore = 1;
                            } else if (q27score == 4) {
                                q27newScore = 3;
                            }
                            participationSubTotal += q27newScore;
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;

                case "28A":
                    if (!String.IsNullOrEmpty(rblQ28A.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ28A.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            participationSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    }
                    //else
                    //{
                    //    isSurveyCompletelyFilledOut = false;
                    //}
                    break;
                case "28B":
                    if (!String.IsNullOrEmpty(rblQ28B.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ28B.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            participationSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    }
                    //else
                    //{
                    //    isSurveyCompletelyFilledOut = false;
                    //}
                    break;


                case "28B1":
                    if (!String.IsNullOrEmpty(rblQ28B1.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ28B1.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            participationSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    }
                    //else
                    //{
                    //    isSurveyCompletelyFilledOut = false;
                    //}
                    break;

                case "29":
                    if (!String.IsNullOrEmpty(rblQ29.SelectedValue)) {
                        choice = question.STD_QUESTION_CHOICEs.FirstOrDefault(C => C.CHOICE_TEXT.ToLower() == rblQ29.SelectedItem.Text.ToLower());
                        if (choice != null) {
                            participationSubTotal += Convert.ToDouble(choice.CHOICE_NAME);
                        }
                    } else {
                        isSurveyCompletelyFilledOut = false;
                    }
                    break;


                default:
                    break;
            }
        }
        // use highest score for 7A or 7B
        abilitySubTotal += Math.Max(q7aScore, q7bScore);

        subtotalOfSubscaleScores = abilitySubTotal + adjustmentSubTotal + participationSubTotal;
        total = subtotalOfSubscaleScores - subTotal22to24;

        abilitiesStandardScore = getAbilitiesStandardScore(abilitySubTotal);
        adjustmentStandardScore = getAdjustmentStandardScore(adjustmentSubTotal);
        participationStandardScore = getParticipationStandardScore(participationSubTotal);
        totalStandardScore = getTotalStandardScore(total);


        lblAbilitySubTotal.Text = abilitySubTotal.ToString();
        lblAdjustmentSubTotal.Text = adjustmentSubTotal.ToString();
        lblParticipationSubTotal.Text = participationSubTotal.ToString();
        lblSubtotalOfSubscaleScores.Text = subtotalOfSubscaleScores.ToString();
        lblSumOfScoresQ22_24.Text = subTotal22to24.ToString();
        lblTotalScore.Text = total.ToString();
        lblAbilitiesStandardScore.Text = abilitiesStandardScore.ToString();
        lblAdjustmentStandardScore.Text = adjustmentStandardScore.ToString();
        lblParticipationStandardScore.Text = participationStandardScore.ToString();
        lblTotalStandardScore.Text = totalStandardScore.ToString();

        if (isSurveyCompletelyFilledOut) {
            lblSurveyError.Text = "";
        } else {
            // ERROR ... 
            lblSurveyError.Text = "ERROR: Survey is not completed."; // TODO - Alert error.
        }
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Converts Ability Raw Score into Standard Score, based on Reference Tables (MPAI-4 Manual)
    /// </summary>
    /// <param name="abilitySubTotal"></param>
    /// <returns></returns>
    private double getAbilitiesStandardScore(double abilitySubTotal) {
        int intAbilitySubTotal = 0;
        double standardAbilitySubtotal = 0;
        try {
            intAbilitySubTotal = Convert.ToInt32(abilitySubTotal);
        } catch (Exception ex) {
            return 0;
        }
        switch (intAbilitySubTotal) {
            case 0:
                standardAbilitySubtotal = -4;
                break;
            case 1:
                standardAbilitySubtotal = 12;
                break;
            case 2:
                standardAbilitySubtotal = 21;
                break;
            case 3:
                standardAbilitySubtotal = 25;
                break;
            case 4:
                standardAbilitySubtotal = 29;
                break;
            case 5:
                standardAbilitySubtotal = 31;
                break;
            case 6:
                standardAbilitySubtotal = 34;
                break;
            case 7:
                standardAbilitySubtotal = 35;
                break;
            case 8:
                standardAbilitySubtotal = 37;
                break;
            case 9:
                standardAbilitySubtotal = 39;
                break;
            case 10:
                standardAbilitySubtotal = 40;
                break;
            case 11:
                standardAbilitySubtotal = 41;
                break;
            case 12:
                standardAbilitySubtotal = 42;
                break;
            case 13:
                standardAbilitySubtotal = 44;
                break;
            case 14:
                standardAbilitySubtotal = 45;
                break;
            case 15:
                standardAbilitySubtotal = 46;
                break;
            case 16:
                standardAbilitySubtotal = 47;
                break;
            case 17:
                standardAbilitySubtotal = 48;
                break;
            case 18:
                standardAbilitySubtotal = 49;
                break;
            case 19:
                standardAbilitySubtotal = 50;
                break;
            case 20:
                standardAbilitySubtotal = 51;
                break;
            case 21:
                standardAbilitySubtotal = 52;
                break;
            case 22:
                standardAbilitySubtotal = 53;
                break;
            case 23:
                standardAbilitySubtotal = 54;
                break;
            case 24:
                standardAbilitySubtotal = 55;
                break;
            case 25:
                standardAbilitySubtotal = 56;
                break;
            case 26:
                standardAbilitySubtotal = 57;
                break;
            case 27:
                standardAbilitySubtotal = 58;
                break;
            case 28:
                standardAbilitySubtotal = 59;
                break;
            case 29:
                standardAbilitySubtotal = 60;
                break;
            case 30:
                standardAbilitySubtotal = 61;
                break;
            case 31:
                standardAbilitySubtotal = 62;
                break;
            case 32:
                standardAbilitySubtotal = 63;
                break;
            case 33:
                standardAbilitySubtotal = 65;
                break;
            case 34:
                standardAbilitySubtotal = 66;
                break;
            case 35:
                standardAbilitySubtotal = 68;
                break;
            case 36:
                standardAbilitySubtotal = 69;
                break;
            case 37:
                standardAbilitySubtotal = 71;
                break;
            case 38:
                standardAbilitySubtotal = 73;
                break;
            case 39:
                standardAbilitySubtotal = 74;
                break;
            case 40:
                standardAbilitySubtotal = 77;
                break;
            case 41:
                standardAbilitySubtotal = 79;
                break;
            case 42:
                standardAbilitySubtotal = 81;
                break;
            case 43:
                standardAbilitySubtotal = 84;
                break;
            case 44:
                standardAbilitySubtotal = 88;
                break;
            case 45:
                standardAbilitySubtotal = 92;
                break;
            case 46:
                standardAbilitySubtotal = 99;
                break;
            case 47:
                standardAbilitySubtotal = 109;
                break;

            default:
                standardAbilitySubtotal = 0;
                break;
        }

        return standardAbilitySubtotal;
    }


    /// <summary>
    /// Converts Adjustment Raw Score into Standard Score, based on Reference Tables (MPAI-4 Manual)
    /// </summary>
    /// <param name="adjustmentSubTotal"></param>
    /// <returns></returns>
    private double getAdjustmentStandardScore(double adjustmentSubTotal) {
        int intAdjustmentSubTotal = 0;
        double standardAdjustmentSubtotal = 0;
        try {
            intAdjustmentSubTotal = Convert.ToInt32(adjustmentSubTotal);
        } catch (Exception ex) {
            return 0;
        }
        switch (intAdjustmentSubTotal) {
            case 0:
                standardAdjustmentSubtotal = -6;
                break;
            case 1:
                standardAdjustmentSubtotal = 10;
                break;
            case 2:
                standardAdjustmentSubtotal = 19;
                break;
            case 3:
                standardAdjustmentSubtotal = 24;
                break;
            case 4:
                standardAdjustmentSubtotal = 28;
                break;
            case 5:
                standardAdjustmentSubtotal = 30;
                break;
            case 6:
                standardAdjustmentSubtotal = 33;
                break;
            case 7:
                standardAdjustmentSubtotal = 35;
                break;
            case 8:
                standardAdjustmentSubtotal = 36;
                break;
            case 9:
                standardAdjustmentSubtotal = 38;
                break;
            case 10:
                standardAdjustmentSubtotal = 39;
                break;
            case 11:
                standardAdjustmentSubtotal = 41;
                break;
            case 12:
                standardAdjustmentSubtotal = 42;
                break;
            case 13:
                standardAdjustmentSubtotal = 43;
                break;
            case 14:
                standardAdjustmentSubtotal = 44;
                break;
            case 15:
                standardAdjustmentSubtotal = 45;
                break;
            case 16:
                standardAdjustmentSubtotal = 46;
                break;
            case 17:
                standardAdjustmentSubtotal = 47;
                break;
            case 18:
                standardAdjustmentSubtotal = 48;
                break;
            case 19:
                standardAdjustmentSubtotal = 49;
                break;
            case 20:
                standardAdjustmentSubtotal = 50;
                break;
            case 21:
                standardAdjustmentSubtotal = 51;
                break;
            case 22:
                standardAdjustmentSubtotal = 52;
                break;
            case 23:
                standardAdjustmentSubtotal = 53;
                break;
            case 24:
                standardAdjustmentSubtotal = 54;
                break;
            case 25:
                standardAdjustmentSubtotal = 55;
                break;
            case 26:
                standardAdjustmentSubtotal = 55;
                break;
            case 27:
                standardAdjustmentSubtotal = 56;
                break;
            case 28:
                standardAdjustmentSubtotal = 57;
                break;
            case 29:
                standardAdjustmentSubtotal = 58;
                break;
            case 30:
                standardAdjustmentSubtotal = 59;
                break;
            case 31:
                standardAdjustmentSubtotal = 60;
                break;
            case 32:
                standardAdjustmentSubtotal = 62;
                break;
            case 33:
                standardAdjustmentSubtotal = 63;
                break;
            case 34:
                standardAdjustmentSubtotal = 64;
                break;
            case 35:
                standardAdjustmentSubtotal = 65;
                break;
            case 36:
                standardAdjustmentSubtotal = 67;
                break;
            case 37:
                standardAdjustmentSubtotal = 68;
                break;
            case 38:
                standardAdjustmentSubtotal = 70;
                break;
            case 39:
                standardAdjustmentSubtotal = 71;
                break;
            case 40:
                standardAdjustmentSubtotal = 73;
                break;
            case 41:
                standardAdjustmentSubtotal = 75;
                break;
            case 42:
                standardAdjustmentSubtotal = 78;
                break;
            case 43:
                standardAdjustmentSubtotal = 80;
                break;
            case 44:
                standardAdjustmentSubtotal = 84;
                break;
            case 45:
                standardAdjustmentSubtotal = 88;
                break;
            case 46:
                standardAdjustmentSubtotal = 94;
                break;

            default:
                standardAdjustmentSubtotal = 0;
                break;
        }

        return standardAdjustmentSubtotal;
    }


    /// <summary>
    /// Converts Participation Raw Score into Standard Score, based on Reference Tables (MPAI-4 Manual)
    /// </summary>
    /// <param name="participationSubTotal"></param>
    /// <returns></returns>
    private double getParticipationStandardScore(double participationSubTotal) {
        int intParticipationSubTotal = 0;
        double standardParticipationSubtotal = 0;
        try {
            intParticipationSubTotal = Convert.ToInt32(participationSubTotal);
        } catch (Exception ex) {
            return 0;
        }
        switch (intParticipationSubTotal) {
            case 0:
                standardParticipationSubtotal = 7;
                break;
            case 1:
                standardParticipationSubtotal = 19;
                break;
            case 2:
                standardParticipationSubtotal = 25;
                break;
            case 3:
                standardParticipationSubtotal = 28;
                break;
            case 4:
                standardParticipationSubtotal = 31;
                break;
            case 5:
                standardParticipationSubtotal = 33;
                break;
            case 6:
                standardParticipationSubtotal = 34;
                break;
            case 7:
                standardParticipationSubtotal = 36;
                break;
            case 8:
                standardParticipationSubtotal = 37;
                break;
            case 9:
                standardParticipationSubtotal = 38;
                break;
            case 10:
                standardParticipationSubtotal = 39;
                break;
            case 11:
                standardParticipationSubtotal = 40;
                break;
            case 12:
                standardParticipationSubtotal = 41;
                break;
            case 13:
                standardParticipationSubtotal = 42;
                break;
            case 14:
                standardParticipationSubtotal = 43;
                break;
            case 15:
                standardParticipationSubtotal = 44;
                break;
            case 16:
                standardParticipationSubtotal = 45;
                break;
            case 17:
                standardParticipationSubtotal = 46;
                break;
            case 18:
                standardParticipationSubtotal = 47;
                break;
            case 19:
                standardParticipationSubtotal = 48;
                break;
            case 20:
                standardParticipationSubtotal = 49;
                break;
            case 21:
                standardParticipationSubtotal = 50;
                break;
            case 22:
                standardParticipationSubtotal = 52;
                break;
            case 23:
                standardParticipationSubtotal = 53;
                break;
            case 24:
                standardParticipationSubtotal = 55;
                break;
            case 25:
                standardParticipationSubtotal = 57;
                break;
            case 26:
                standardParticipationSubtotal = 59;
                break;
            case 27:
                standardParticipationSubtotal = 62;
                break;
            case 28:
                standardParticipationSubtotal = 65;
                break;
            case 29:
                standardParticipationSubtotal = 69;
                break;
            case 30:
                standardParticipationSubtotal = 74;
                break;

            default:
                standardParticipationSubtotal = 0;
                break;
        }

        return standardParticipationSubtotal;
    }


    /// <summary>
    /// Converts Total Raw Score into Standard Score, based on Reference Tables (MPAI-4 Manual)
    /// </summary>
    /// <param name="totalOfSubscaleScores"></param>
    /// <returns></returns>
    private double getTotalStandardScore(double totalOfSubscaleScores) {
        int intTotal = 0;
        double standardTotal = 0;
        try {
            intTotal = Convert.ToInt32(totalOfSubscaleScores);
        } catch (Exception ex) {
            return 0;
        }
        switch (intTotal) {
            case 0:
                standardTotal = -38;
                break;
            case 1:
                standardTotal = -16;
                break;
            case 2:
                standardTotal = -4;
                break;
            case 3:
                standardTotal = 2;
                break;
            case 4:
                standardTotal = 7;
                break;
            case 5:
                standardTotal = 11;
                break;
            case 6:
                standardTotal = 13;
                break;
            case 7:
                standardTotal = 16;
                break;
            case 8:
                standardTotal = 18;
                break;
            case 9:
                standardTotal = 20;
                break;
            case 10:
                standardTotal = 21;
                break;
            case 11:
                standardTotal = 23;
                break;
            case 12:
                standardTotal = 24;
                break;
            case 13:
                standardTotal = 25;
                break;
            case 14:
                standardTotal = 26;
                break;
            case 15:
                standardTotal = 27;
                break;
            case 16:
                standardTotal = 28;
                break;
            case 17:
                standardTotal = 29;
                break;
            case 18:
                standardTotal = 30;
                break;
            case 19:
                standardTotal = 31;
                break;
            case 20:
                standardTotal = 32;
                break;
            case 21:
                standardTotal = 33;
                break;
            case 22:
                standardTotal = 33;
                break;
            case 23:
                standardTotal = 34;
                break;
            case 24:
                standardTotal = 35;
                break;
            case 25:
                standardTotal = 35;
                break;
            case 26:
                standardTotal = 36;
                break;
            case 27:
                standardTotal = 37;
                break;
            case 28:
                standardTotal = 37;
                break;
            case 29:
                standardTotal = 38;
                break;
            case 30:
                standardTotal = 39;
                break;
            case 31:
                standardTotal = 39;
                break;
            case 32:
                standardTotal = 40;
                break;
            case 33:
                standardTotal = 40;
                break;
            case 34:
                standardTotal = 41;
                break;
            case 35:
                standardTotal = 41;
                break;
            case 36:
                standardTotal = 42;
                break;
            case 37:
                standardTotal = 42;
                break;
            case 38:
                standardTotal = 43;
                break;
            case 39:
                standardTotal = 44;
                break;
            case 40:
                standardTotal = 44;
                break;
            case 41:
                standardTotal = 45;
                break;
            case 42:
                standardTotal = 45;
                break;
            case 43:
                standardTotal = 46;
                break;
            case 44:
                standardTotal = 46;
                break;
            case 45:
                standardTotal = 47;
                break;
            case 46:
                standardTotal = 47;
                break;
            case 47:
                standardTotal = 47;
                break;
            case 48:
                standardTotal = 48;
                break;
            case 49:
                standardTotal = 48;
                break;
            case 50:
                standardTotal = 49;
                break;
            case 51:
                standardTotal = 49;
                break;
            case 52:
                standardTotal = 50;
                break;
            case 53:
                standardTotal = 50;
                break;
            case 54:
                standardTotal = 51;
                break;
            case 55:
                standardTotal = 51;
                break;
            case 56:
                standardTotal = 52;
                break;
            case 57:
                standardTotal = 52;
                break;
            case 58:
                standardTotal = 53;
                break;
            case 59:
                standardTotal = 53;
                break;
            case 60:
                standardTotal = 54;
                break;
            case 61:
                standardTotal = 54;
                break;
            case 62:
                standardTotal = 55;
                break;
            case 63:
                standardTotal = 55;
                break;
            case 64:
                standardTotal = 56;
                break;
            case 65:
                standardTotal = 56;
                break;
            case 66:
                standardTotal = 57;
                break;
            case 67:
                standardTotal = 57;
                break;
            case 68:
                standardTotal = 58;
                break;
            case 69:
                standardTotal = 58;
                break;
            case 70:
                standardTotal = 59;
                break;
            case 71:
                standardTotal = 59;
                break;
            case 72:
                standardTotal = 60;
                break;
            case 73:
                standardTotal = 60;
                break;
            case 74:
                standardTotal = 61;
                break;
            case 75:
                standardTotal = 62;
                break;
            case 76:
                standardTotal = 62;
                break;
            case 77:
                standardTotal = 63;
                break;
            case 78:
                standardTotal = 63;
                break;
            case 79:
                standardTotal = 64;
                break;
            case 80:
                standardTotal = 65;
                break;
            case 81:
                standardTotal = 65;
                break;
            case 82:
                standardTotal = 66;
                break;
            case 83:
                standardTotal = 67;
                break;
            case 84:
                standardTotal = 67;
                break;
            case 85:
                standardTotal = 68;
                break;
            case 86:
                standardTotal = 69;
                break;
            case 87:
                standardTotal = 69;
                break;
            case 88:
                standardTotal = 70;
                break;
            case 89:
                standardTotal = 71;
                break;
            case 90:
                standardTotal = 72;
                break;
            case 91:
                standardTotal = 73;
                break;
            case 92:
                standardTotal = 74;
                break;
            case 93:
                standardTotal = 75;
                break;
            case 94:
                standardTotal = 76;
                break;
            case 95:
                standardTotal = 77;
                break;
            case 96:
                standardTotal = 78;
                break;
            case 97:
                standardTotal = 79;
                break;
            case 98:
                standardTotal = 80;
                break;
            case 99:
                standardTotal = 81;
                break;
            case 100:
                standardTotal = 82;
                break;
            case 101:
                standardTotal = 84;
                break;
            case 102:
                standardTotal = 85;
                break;
            case 103:
                standardTotal = 87;
                break;
            case 104:
                standardTotal = 88;
                break;
            case 105:
                standardTotal = 90;
                break;
            case 106:
                standardTotal = 92;
                break;
            case 107:
                standardTotal = 94;
                break;
            case 108:
                standardTotal = 97;
                break;
            case 109:
                standardTotal = 99;
                break;
            case 110:
                standardTotal = 102;
                break;
            case 111:
                standardTotal = 106;
                break;

            default:
                standardTotal = 0;
                break;
        }

        return standardTotal;
    }


    /// <summary>
    /// Get all questions for this survey
    /// </summary>
    /// <returns></returns>
    private List<STD_QUESTION> GetAllQuestionsForSurvey() {
        return _tbiInstrumentManager.SelectAllQuestions(_surveyTypeId).ToList();
    }

    /// <summary>
    /// Refresh Survey
    /// </summary>
    /// <param name="patientICN"></param>
    private void RefreshSurvey(string patientICN) {
        var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

        if (surveys.Count > 0) {
            _survey = surveys.First();
        } else {
            _survey = new SURVEY();
        }

        _survey.SetAsChangeTrackingRoot();
    }

    /// <summary>
    /// Save Complete Survey
    /// </summary>
    /// <param name="status"></param>
    private void SaveCompleteSurvey(ref Int32 status) {
        Page.Validate();
        //if (Page.IsValid) {
        if ((Page.IsValid) && IsQRPT1Valid() && Is28ABValid()) {
            string patientICN = String.Empty;

            status = 0;

            //Retrieve patient's ICN from the session variable
            if (ViewState["PatientICN"] != null) {
                patientICN = ViewState["PatientICN"].ToString();
                if (String.IsNullOrEmpty(patientICN.Trim())) {
                    //Lost state. Redirect to message that need to log back in from CPRS.
                    Response.Redirect("Warning.aspx?err=2", false);
                }
            } else {
                //Lost state. Redirect to message that need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }

            if (_survey != null) {
                if (_survey.isNew) {
                    //Ensure that a concurrent session hasn't already inserted a record for this survey
                    var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

                    if (surveys.Count > 0) {
                        //A concurrent session has already inserted a record for this survey and it is not new anymore.
                        Response.Redirect("Warning.aspx?err=9", false);
                    } else {
                        _survey.SURVEY_STATUS = "inprocess";

                        _survey.SetAsInsertOnSubmit();

                        _survey.patient_ICN = patientICN;

                        _survey.STD_SURVEY_TYPE_ID = _surveyTypeId;

                        // TODO: change to datepicker
                        _survey.SURVEY_DATE = DateTime.Today;

                        //Save survey if it's new before adding any responses
                        SuppressSaveMessage = true;
                        _tbiInstrumentManager.Update(_survey);
                        SuppressSaveMessage = false;

                        RefreshSurvey(patientICN);
                    }
                }

                SaveSurvey(ref _survey);

                _survey.SetAsUpdateOnSubmit();
                status = _tbiInstrumentManager.Update(_survey);

                (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);

                RefreshSurvey(patientICN);
                SuppressSaveMessage = false;
            }
        }
    }

    private bool IsQRPT1Valid() {
        if (rblQRPT.SelectedItem.Text == "Significant Other") {
            if (txtQRPT1.Text.Trim().Equals("")) {
                lblErrorQRPT1.Text = "Please specify 'Significant other'";
                return false;
            }
        }
        return true;
    }

    private bool Is28ABValid() {
        if ((rblQ28A.SelectedIndex == -1) && (rblQ28B.SelectedIndex == -1)) {
            lblError28AB.Text = "Rate either item 28A or 28B to reflect the primary desired social role. Do not rate both. Rate 28A if the primary social role is paid employment. If another social role is primary, rate only 28B.";
            return false;
        }

        if ((rblQ28A.SelectedIndex != -1) && (rblQ28B.SelectedIndex != -1)) {
            rblQ28A.ClearSelection();
            rblQ28B.ClearSelection();
            lblError28AB.Text = "Rate either item 28A or 28B to reflect the primary desired social role. Do not rate both. Rate 28A if the primary social role is paid employment. If another social role is primary, rate only 28B.";
            return false;
        }
        if ((rblQ28A.SelectedIndex == -1) && (rblQ28B.SelectedIndex != -1)) {
            if (rblQ28B1.SelectedIndex == -1) {
                lblError28AB.Text = "Please select primary desired social role (28B1)";
                return false;
            }
        }
        return true;
    }

    /// <summary>
    /// Save survey.
    /// </summary>
    /// <param name="survey"></param>
    private void SaveSurvey(ref SURVEY survey) {

        calculateTotal(); // call to re-calculate total score.

        // Ensure lblTotal does not contain - ERROR
        if (!lblSurveyError.Text.Contains("ERROR")) {
            List<STD_QUESTION> questions = GetAllQuestionsForSurvey();

            //Delete all the survey results from database if there are any.
            SURVEY surveyWithResults = _tbiInstrumentManager.GetSurveyWithAllAnswers(survey.SURVEYS_ID);
            _tbiInstrumentManager.DeleteAllSurveyResults(surveyWithResults);

            // Question DX
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question DX choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(ctrlDiagnosis.GetSelectedId());
                _surveyResult.RESULT_TEXT = ctrlDiagnosis.GetSelectedResultText();
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question DX");
            }

            // Question D
            _surveyResult = new SURVEY_RESULT();

            if (dpDate1.SelectedDate != null) {
                STD_QUESTION questionD = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "D");
                _surveyResult = new SURVEY_RESULT();

                try {
                    // Add Question DTE choice
                    _surveyResult.STD_QUESTION_CHOICE_ID = questionD.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                    _surveyResult.RESULT_TEXT = ((DateTime)dpDate1.SelectedDate).ToShortDateString();
                    survey.SURVEY_RESULTs.Add(_surveyResult);
                } catch (Exception surveyEx) {
                    Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question D");
                }
            }

            // Question QRPT
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question QRPT choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQRPT.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQRPT.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question QRPT");
            }

            if (rblQRPT.SelectedIndex == 3) // only save this if QRPT = "Significant Other"
            {
                // Question QRPT1
                STD_QUESTION questionQRPT1 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "QRPT1");
                _surveyResult = new SURVEY_RESULT();
                try
                {
                    _surveyResult.STD_QUESTION_CHOICE_ID = questionQRPT1.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                    _surveyResult.RESULT_TEXT = txtQRPT1.Text;
                    survey.SURVEY_RESULTs.Add(_surveyResult);
                }
                catch (Exception surveyEx)
                {
                    Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question QRPT1");
                }
            }

            // Question 1
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 1 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ1.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ1.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 1");
            }

            // Question 2
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 2 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ2.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ2.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 2");
            }

            // Question 3
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 3 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ3.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ3.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 3");
            }

            // Question 4
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 4 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ4.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ4.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 4");
            }

            // Question 5
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 5 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ5.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ5.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 5");
            }

            // Question 6
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 6 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ6.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ6.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 6");
            }

            // Question 7A
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 7A choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ7A.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ7A.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 7A");
            }

            // Question 7B
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 7B choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ7B.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ7B.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 7B");
            }

            // Question 8
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 8 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ8.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ8.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 8");
            }

            // Question 9
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 9 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ9.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ9.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 9");
            }

            // Question 10
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 10 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ10.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ10.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 10");
            }

            // Question 11
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 11 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ11.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ11.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 11");
            }

            // Question 12
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 12 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ12.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ12.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 12");
            }

            // Question 13
            _surveyResult = new SURVEY_RESULT();

            try {
                // Add Question 13 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ13.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ13.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 13");
            }

            // Question 14
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 14 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ14.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ14.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 14");
            }

            // Question 15
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 15 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ15.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ15.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 15");
            }

            // Question 16
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 16 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ16.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ16.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 16");
            }

            // Question 17
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 17 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ17.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ17.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 17");
            }

            // Question 18
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 18 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ18.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ18.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 18");
            }

            // Question 19
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 19 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ19.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ19.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 19");
            }

            // Question 20
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 20 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ20.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ20.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 20");
            }

            // Question 21
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 21 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ21.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ21.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 21");
            }

            // Question 22
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 22 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ22.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ22.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 22");
            }

            // Question 23
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 23 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ23.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ23.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 23");
            }

            // Question 24
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 24 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ24.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ24.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 24");
            }

            // Question 25
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 25 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ25.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ25.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 25");
            }

            // Question 26
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 26 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ26.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ26.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 26");
            }

            // Question 27
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 27 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ27.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ27.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 27");
            }

            // Question 28A
            if (rblQ28A.SelectedValue != "") {
                _surveyResult = new SURVEY_RESULT();
                try {
                    // Add Question 28A choice
                    _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ28A.SelectedValue);
                    _surveyResult.RESULT_TEXT = rblQ28A.SelectedItem.Text;
                    survey.SURVEY_RESULTs.Add(_surveyResult);
                } catch (Exception surveyEx) {
                    Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 28A");
                }
            }

            // Question 28B
            if (rblQ28B.SelectedValue != "") {
                _surveyResult = new SURVEY_RESULT();
                try {
                    // Add Question 28B choice
                    _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ28B.SelectedValue);
                    _surveyResult.RESULT_TEXT = rblQ28B.SelectedItem.Text;
                    survey.SURVEY_RESULTs.Add(_surveyResult);
                } catch (Exception surveyEx) {
                    Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 28B");
                }
            }

            // Question 28B1
            if (rblQ28B1.SelectedValue != "") {
                _surveyResult = new SURVEY_RESULT();
                try {
                    // Add Question 28B1 choice
                    _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ28B1.SelectedValue);
                    _surveyResult.RESULT_TEXT = rblQ28B1.SelectedItem.Text;
                    survey.SURVEY_RESULTs.Add(_surveyResult);
                } catch (Exception surveyEx) {
                    Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 28B1");
                }
            }

            // Question 29
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 29 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ29.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ29.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 29");
            }

            // Question 30PRE
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 30PRE choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ30PRE.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ30PRE.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 30PRE");
            }

            // Question 30POST
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 30POST choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ30POST.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ30POST.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 30POST");
            }

            // Question 31PRE
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 31PRE choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ31PRE.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ31PRE.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 31PRE");
            }

            // Question 31POST
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 31POST choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ31POST.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ31POST.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 31POST");
            }

            // Question 32PRE
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 32PRE choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ32PRE.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ32PRE.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 32PRE");
            }

            // Question 32POST
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 32POST choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ32POST.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ32POST.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 32POST");
            }

            // Question 33PRE
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 33PRE choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ33PRE.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ33PRE.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 33PRE");
            }

            // Question 33POST
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 33POST choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ33POST.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ33POST.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 33POST");
            }

            // Question 34PRE
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 34PRE choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ34PRE.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ34PRE.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 34PRE");
            }

            // Question 34POST
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 34POST choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ34POST.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ34POST.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 34POST");
            }

            // Question 35PRE
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 35PRE choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ35PRE.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ35PRE.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 35PRE");
            }

            // Question 35POST
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 35POST choice
                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rblQ35POST.SelectedValue);
                _surveyResult.RESULT_TEXT = rblQ35POST.SelectedItem.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 35POST");
            }


            // Question 36 - Comments
            STD_QUESTION question36 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "36");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 36 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question36.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = txtCommentsQ36.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 36");
            }

            // Question 37
            STD_QUESTION question37 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "37");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 37 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question37.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblAbilitySubTotal.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 37");
            }

            // Question 38
            STD_QUESTION question38 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "38");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 38 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question38.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblAdjustmentSubTotal.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 38");
            }

            // Question 39
            STD_QUESTION question39 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "39");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 39 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question39.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblParticipationSubTotal.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 39");
            }

            // Question 40
            STD_QUESTION question40 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "40");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 40 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question40.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblSubtotalOfSubscaleScores.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 40");
            }

            // Question 41
            STD_QUESTION question41 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "41");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 41 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question41.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblSumOfScoresQ22_24.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 41");
            }

            // Question 42
            STD_QUESTION question42 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "42");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 42 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question42.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblTotalScore.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 42");
            }

            // Question 43
            STD_QUESTION question43 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "43");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 43 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question43.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblAbilitiesStandardScore.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 43");
            }

            // Question 44
            STD_QUESTION question44 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "44");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 44 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question44.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblAdjustmentStandardScore.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 44");
            }

            // Question 45
            STD_QUESTION question45 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "45");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 45 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question45.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblParticipationStandardScore.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 45");
            }

            // Question 46
            STD_QUESTION question46 = questions.FirstOrDefault(Q => Q.QUESTION_NUMBER == "46");
            _surveyResult = new SURVEY_RESULT();
            try {
                // Add Question 46 choice
                _surveyResult.STD_QUESTION_CHOICE_ID = question46.STD_QUESTION_CHOICEs.FirstOrDefault().STD_QUESTION_CHOICE_ID;
                _surveyResult.RESULT_TEXT = lblTotalStandardScore.Text;
                survey.SURVEY_RESULTs.Add(_surveyResult);
            } catch (Exception surveyEx) {
                Exception _tbilogerror = HandleTBIException(surveyEx, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SaveSurvey() - Question 46");
            }

        } else {
            // do not save ... 
        }
    }// END SaveSurvey

    /// <summary>
    /// Initialize/setup controls
    /// </summary>
    /// <param name="survey"></param>
    private void SetUpSurvey(SURVEY survey) {
        try {
            foreach (var sResult in survey.SURVEY_RESULTs) {
                switch (sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER) {
                    case "DX":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            ctrlDiagnosis.SelectedChoiceId = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();

                            if (sResult.STD_QUESTION_CHOICE.CHOICE_TEXT.ToLower().Contains("other")) {
                                ctrlDiagnosis.SetOtherDescriptionText(sResult.RESULT_TEXT);
                            }
                        } else {
                            ctrlDiagnosis.SelectedChoiceId = null;
                        }
                        break;
                    case "D":
                        if (!String.IsNullOrEmpty(sResult.RESULT_TEXT)) {
                            DateTime temp;
                            if (DateTime.TryParse(sResult.RESULT_TEXT, out temp)) {
                                dpDate1.SelectedDate = temp;
                            }
                        }
                        break;
                    case "QRPT":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQRPT.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQRPT.SelectedIndex = -1;
                        }
                        break;
                    case "QRPT1":
                        txtQRPT1.Text = sResult.RESULT_TEXT;
                        break;

                    case "1":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ1.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ1.SelectedIndex = -1;
                        }
                        break;

                    case "2":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ2.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ2.SelectedIndex = -1;
                        }
                        break;
                    case "3":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ3.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ3.SelectedIndex = -1;
                        }
                        break;
                    case "4":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ4.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ4.SelectedIndex = -1;
                        }
                        break;
                    case "5":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ5.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ5.SelectedIndex = -1;
                        }
                        break;
                    case "6":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ6.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ6.SelectedIndex = -1;
                        }
                        break;
                    case "7A":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ7A.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ7A.SelectedIndex = -1;
                        }
                        break;
                    case "7B":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ7B.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ7B.SelectedIndex = -1;
                        }
                        break;
                    case "8":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ8.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ8.SelectedIndex = -1;
                        }
                        break;
                    case "9":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ9.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ9.SelectedIndex = -1;
                        }
                        break;
                    case "10":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ10.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ10.SelectedIndex = -1;
                        }
                        break;
                    case "11":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ11.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ11.SelectedIndex = -1;
                        }
                        break;
                    case "12":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ12.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ12.SelectedIndex = -1;
                        }
                        break;
                    case "13":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ13.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ13.SelectedIndex = -1;
                        }
                        break;
                    case "14":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ14.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ14.SelectedIndex = -1;
                        }
                        break;
                    case "15":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ15.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ15.SelectedIndex = -1;
                        }
                        break;
                    case "16":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ16.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ16.SelectedIndex = -1;
                        }
                        break;
                    case "17":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ17.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ17.SelectedIndex = -1;
                        }
                        break;
                    case "18":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ18.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ18.SelectedIndex = -1;
                        }
                        break;
                    case "19":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ19.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ19.SelectedIndex = -1;
                        }
                        break;
                    case "20":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ20.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ20.SelectedIndex = -1;
                        }
                        break;
                    case "21":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ21.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ21.SelectedIndex = -1;
                        }
                        break;
                    case "22":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ22.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ22.SelectedIndex = -1;
                        }
                        break;
                    case "23":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ23.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ23.SelectedIndex = -1;
                        }
                        break;
                    case "24":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ24.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ24.SelectedIndex = -1;
                        }
                        break;
                    case "25":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ25.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ25.SelectedIndex = -1;
                        }
                        break;
                    case "26":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ26.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ26.SelectedIndex = -1;
                        }
                        break;
                    case "27":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ27.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ27.SelectedIndex = -1;
                        }
                        break;

                    case "28A":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1)  //(sResult.STD_QUESTION_CHOICE.CHOICE_SORT_ORDER.HasValue)
                        {
                            rblQ28A.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ28A.SelectedIndex = -1;
                        }
                        break;
                    case "28B":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ28B.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ28B.SelectedIndex = -1;
                        }
                        break;

                    case "28B1":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ28B1.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ28B1.SelectedIndex = -1;
                        }
                        break;

                    case "29":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ29.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ29.SelectedIndex = -1;
                        }
                        break;
                    case "30PRE":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ30PRE.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ30PRE.SelectedIndex = -1;
                        }
                        break;
                    case "30POST":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ30POST.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ30POST.SelectedIndex = -1;
                        }
                        break;
                    case "31PRE":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ31PRE.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ31PRE.SelectedIndex = -1;
                        }
                        break;
                    case "31POST":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ31POST.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ31POST.SelectedIndex = -1;
                        }
                        break;
                    case "32PRE":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ32PRE.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ32PRE.SelectedIndex = -1;
                        }
                        break;
                    case "32POST":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ32POST.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ32POST.SelectedIndex = -1;
                        }
                        break;
                    case "33PRE":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ33PRE.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ33PRE.SelectedIndex = -1;
                        }
                        break;
                    case "33POST":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ33POST.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ33POST.SelectedIndex = -1;
                        }
                        break;
                    case "34PRE":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ34PRE.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ34PRE.SelectedIndex = -1;
                        }
                        break;
                    case "34POST":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ34POST.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ34POST.SelectedIndex = -1;
                        }
                        break;
                    case "35PRE":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ35PRE.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ35PRE.SelectedIndex = -1;
                        }
                        break;
                    case "35POST":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            rblQ35POST.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
                        } else {
                            rblQ35POST.SelectedIndex = -1;
                        }
                        break;

                    case "36":
                        txtCommentsQ36.Text = sResult.RESULT_TEXT;
                        break;


                    case "37":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblAbilitySubTotal.Text = sResult.RESULT_TEXT;
                        }
                        break;
                    case "38":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblAdjustmentSubTotal.Text = sResult.RESULT_TEXT;
                        }
                        break;
                    case "39":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblParticipationSubTotal.Text = sResult.RESULT_TEXT;
                        }
                        break;
                    case "40":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblSubtotalOfSubscaleScores.Text = sResult.RESULT_TEXT;
                        }
                        break;
                    case "41":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblSumOfScoresQ22_24.Text = sResult.RESULT_TEXT;
                        }
                        break;
                    case "42":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblTotalScore.Text = sResult.RESULT_TEXT;
                        }
                        break;
                    case "43":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblAbilitiesStandardScore.Text = sResult.RESULT_TEXT;
                        }
                        break;
                    case "44":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblAdjustmentStandardScore.Text = sResult.RESULT_TEXT;
                        }
                        break;
                    case "45":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblParticipationStandardScore.Text = sResult.RESULT_TEXT;
                        }
                        break;
                    case "46":
                        if (sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1) {
                            lblTotalStandardScore.Text = sResult.RESULT_TEXT;
                        }
                        break;

                    default:
                        break;
                }
            }
        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "SetUpSurvey()");
        }
    }// END SetUpSurvey

    /// <summary>
    /// Retrieve instrument title
    /// </summary>
    /// <param name="surveyID"></param>
    /// <returns></returns>
    private string GetInstrumentTitle(int surveyID) {
        string ret = String.Empty;

        try {
            List<string> instrumentTitle = _tbiInstrumentManager.GetInstrumentByID(surveyID).Title;
            if (instrumentTitle.Count > 0) {
                foreach (string titleLine in instrumentTitle) {
                    if (titleLine.Contains("PhD")) {
                        ret = ret + "<small>" + titleLine + "</small>" + "<br \\>";
                    } else {
                        ret = ret + titleLine + "<br \\>";
                    }
                }
            }

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "MayoPortlandAdaptabilityInventory4.aspx", "100027", "GetInstrumentTitle()");
            // Ignore error
        }

        return ret;
    }

    #endregion
}
