﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.Business.Utilities;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.TBI;
using System.Text;

public partial class PolyTraumaEntry : BasePage
{

    #region Variable Declarations

    TBI_POLYTRAUMA polyTraumEntry;
    PolyTraumaEntriesManager PolyTraumaManager = new PolyTraumaEntriesManager();
    //TBIExceptionHandler _tbiexception = new TBIExceptionHandler();

    #endregion

    #region Public Overrides

    public PolyTraumaEntry()
    {
        //Manager Save Notification Event Handler
        PolyTraumaManager.BOSaveSuccess += new BaseBO.BOSaveSuccessEvent(manager_BOSaveSuccess);
    }
    public override string TopMenuName()
    {
        return "My Tasks";
    }
    public override string MenuItemName()
    {
        return "TBI POLYTRAUMA";
    }
    public override string AppObjectCode()
    {
        return "TBI POLYTRAUMA";
    }

    #endregion

    #region Protected Methods

    protected void Page_Load(object sender, EventArgs e)
    {
        
        if (!IsPostBack)
        {
            //if page is loading for first time get ID from querystring and set to hidden field to use on postbacks
            int? entryID = Convert.ToInt32(Request.QueryString["id"]);
            HiddenFieldEntryID.Value = entryID.GetValueOrDefault().ToString();

            //Check if the request is to start a new item 
            //a new item page request is signified when the querystring parameter id equals zero or is not present
            if (entryID == null || entryID == 0)
            {
                //set entryID to zero so that a default linq entity will be instantiated
                entryID = 0;

                if (this.AllowInsert)
                {
                    //if a new item then set patient search to true and hide all others
                    SetFormToNewPatientSearchMode();
                }
                else
                {
                    SetFormToNoInsertAllowedMode();
                }

            }
            else
            {
                //if not a new item hide patient search, show patient read only and hide patient edit
                //SetFormToEditPolyTraumaHidePatientDetails(entryID.GetValueOrDefault());

                ////Modify breadcrumb to display editing ID
                //LabelBreadCrumb.Text = LabelBreadCrumb.Text + " > Edit Tracking Item ID : " + entryID;
                SetBreadCrumbLabelText(entryID); 

            }               
        }

        //Create PolyTrauma Entity object and Set properties to state of form
        getEntry(Convert.ToInt32(HiddenFieldEntryID.Value));

        CompareValidatorDeathPastDaste.ValueToCompare = DateTime.Now.ToShortDateString(); 

        if (IsPostBack)
        {
            LoadObjectFromControls();
        }

    }

    protected void getEntry(int id)
    {
        try
        {
            polyTraumEntry = PolyTraumaManager.SelectByID(id);

            //initial display panels
            PanelNoAccess.Visible = false;
            PanelIdNotFound.Visible = false;
            PanelObjectContent.Visible = false;

            if (polyTraumEntry == null)
            {
                PanelObjectContent.Visible = false;

                if (PolyTraumaManager.PolyTraumaExists(id))
                {
                    PanelNoAccess.Visible = true;
                }
                else
                {
                    PanelIdNotFound.Visible = true;
                }
            }
            else
            {
                if (id != 0)
                {
                    EntityInstitution = polyTraumEntry.POLYTRAUMA_CENTER_ID;
                }
                PanelObjectContent.Visible = true;
            }
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "GetEntry()");
        } 
    }

    protected void ButtonClose_Click(object sender, EventArgs e)
    {
        Response.Redirect("PolyTraumaEntries.aspx");
    }

    protected void ButtonSearchPatient_Click(object sender, EventArgs e)
    {
        try
        {
            if (!(TextBoxSSNSearch.Text.Trim() == "" && TextBoxPatientNameSearch.Text.Trim() == ""))
            {

                GridViewPatients.Visible = true;
                GridViewPatients.DataBind();
            }
            else
            {
                GridViewPatients.Visible = false;
            }

            PanelPatientExistsNotice.Visible = false;
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "ButtonSearchPatient_Click()");
        } 
    }

    protected void LinkButtonEditPatientInformation_Click(object sender, EventArgs e)
    {
        SetFormToEditPolyTraumaWithPatientDetails();
    }

    protected void GridViewPatients_SelectedIndexChanged(object sender, EventArgs e)
    {
        try
        {
            PATIENT patientEntry;
            PatientManager patientManager = new PatientManager();
            patientEntry = patientManager.SelectByID((int)GridViewPatients.SelectedDataKey.Value);

            //check if patient exists in polytrauma database by comparing SSN
            if (PolyTraumaManager.SSNExistsInPolyTrauma(patientEntry.SSN))
            {
                //if the ssn exists in polytrauma alert user and ask if it is a new admission
                PanelPatientExistsNotice.Visible = true;

                HiddenFieldPatientExistsSelectedID.Value = patientEntry.PATIENT_ID.ToString();
            }
            else
            {
                PanelPatientExistsNotice.Visible = false;

                TransferPatientInformationToNewPolyTraumaEntry(patientEntry);

                SetFormToEditPolyTraumaWithPatientDetails();
            }
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "GridViewPatients_SelectedIndexChanged()");
        } 
    }

    #region Postback Events

    protected void ButtonSave_Click(object sender, EventArgs e)
    {
        int returnSaveValue = 0;
        try
        {
            //RangeValidatorTextBoxDateOfBirth.Validate();
            //RangeValidatorTextBoxDateOfDeath.Validate();
            Page.Validate();
            if (Page.IsValid)
            {
                if (polyTraumEntry.SSNChanged == true
                    && HiddenFieldUserAcceptedSSN.Value != "true"
                    && PolyTraumaManager.SSNExistsInPolyTrauma(polyTraumEntry.SSN))
                {
                    //if the ssn exists in polytrauma alert user and ask if it is a new admission
                    PanelSSNExists.Visible = true;
                    //ButtonContinueWithSSNChange.Focus();
                    HiddenField focusField = (HiddenField)Master.FindControl("focusField");
                    focusField.Value = ButtonContinueWithSSNChange.ClientID;

                }
                else
                {
                    PanelSSNExists.Visible = false;

                    int addNewTbiPolyTraumEntryId = polyTraumEntry.TBI_POLYTRAUMA_ID;
                    this.SuppressSaveMessage = false;
                    returnSaveValue = PolyTraumaManager.Update(polyTraumEntry);

                    if (returnSaveValue > 0)
                    {
                        HiddenFieldEntryID.Value = returnSaveValue.ToString();
                    }

                    //Set Broken Rule Error Display
                    if (polyTraumEntry.BrokenRules.Count == 0)
                    {
                        ErrorList1.Visible = false;
                    }
                    else
                    {
                        ErrorList1.Visible = true;
                    }

                    ErrorList1.BrokenRules = polyTraumEntry.BrokenRules;
                    DisplaySaveMessage(returnSaveValue, addNewTbiPolyTraumEntryId);
                }
            }
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "ButtonSave_Click()");
        }
    }

    protected void ButtonSkipPatient_Click(object sender, EventArgs e)
    {
        SetFormToEditPolyTraumaWithPatientDetails();
    }

    protected void ButtonPatientExistsContinue_Click(object sender, EventArgs e)
    {
        PATIENT patientEntry;
        PatientManager patientManager = new PatientManager();
        try
        {
            patientEntry = patientManager.SelectByID(Helpers.getIntFromString(HiddenFieldPatientExistsSelectedID.Value));


            PanelPatientExistsNotice.Visible = false;
            HiddenFieldPatientExistsSelectedID.Value = "";

            TransferPatientInformationToNewPolyTraumaEntry(patientEntry);
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "ButtonPatientExistsContinue_Click()");
        }

        SetFormToEditPolyTraumaWithPatientDetails();

        HiddenFieldUserAcceptedSSN.Value = "true";

    }
    protected void ButtonContinueWithSSNChange_Click(object sender, EventArgs e)
    {
        HiddenFieldUserAcceptedSSN.Value = "true";

        ButtonSave_Click(sender, e);

    }
    #endregion

    #endregion

    #region Private Methods

    private void SetDatesRangeValidators(bool loadControlsFromObject)
    {
        try
        {
            //Load Controls From Object
            if (loadControlsFromObject)
            {
                RangeValidatorTextBoxDateOfBirth.MinimumValue = DateTime.MinValue.ToShortDateString();
                RangeValidatorTextBoxDateOfBirth.MaximumValue = DateTime.Now.ToShortDateString();
            }
            ////Load Object From Controls 
            else
            {
                RangeValidatorTextBoxDateOfBirth.MinimumValue = DateTime.MinValue.ToShortDateString();
                RangeValidatorTextBoxDateOfBirth.MaximumValue = DateTime.Now.ToShortDateString();
            }
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "SetDatesRangeValidators()");
        }
    }

    private void PopulateDropDowns()
    {
        try
        {

            if (DropDownListGender.Items.Count == 1)
            {
                DropDownListGender.DataSourceID = "ObjectDataSourceGender";
                DropDownListGender.DataBind();
            }

            if (DropDownListMaritalStatus.Items.Count == 1)
            {
                DropDownListMaritalStatus.DataSourceID = "ObjectDataSourceMaritalStatus";
                DropDownListMaritalStatus.DataBind();
            }

            if (DropDownListRace.Items.Count == 1)
            {
                DropDownListRace.DataSourceID = "ObjectDataSourceRace";
                DropDownListRace.DataBind();
            }

            if (DropDownListEthnicity.Items.Count == 1)
            {
                DropDownListEthnicity.DataSourceID = "ObjectDataSourceEthnicity";
                DropDownListEthnicity.DataBind();
            }

            if (DropDownListHomeVAMC.Items.Count == 1)
            {
                DropDownListHomeVAMC.DataSourceID = "ObjectDataSourceVAMC";
                DropDownListHomeVAMC.DataBind();
            }

            if (DropDownListMilitartServiceBranch.Items.Count == 1)
            {
                DropDownListMilitartServiceBranch.DataSourceID = "ObjectDataSourceMilitaryServiceBranch";
                DropDownListMilitartServiceBranch.DataBind();
            }

            if (DropDownListMilitaryServiceStatus.Items.Count == 1)
            {
                DropDownListMilitaryServiceStatus.DataSourceID = "ObjectDataSourceMilitaryServiceStatus";
                DropDownListMilitaryServiceStatus.DataBind();
            }

            if (DropDownListDischargeStatus.Items.Count == 1)
            {
                DropDownListDischargeStatus.DataSourceID = "ObjectDataSourceDischargeStatus";
                DropDownListDischargeStatus.DataBind();
            }

            if (DropDownListTBIFacilitySource.Items.Count == 1)
            {
                DropDownListTBIFacilitySource.DataSourceID = "ObjectDataSourceTBIFacilitySource";
                DropDownListTBIFacilitySource.DataBind();
            }

            if (DropDownListCombatLocation.Items.Count == 1)
            {
                DropDownListCombatLocation.DataSourceID = "ObjectDataSourceCombatLocation";
                DropDownListCombatLocation.DataBind();
            }

            if (CheckBoxListInjuryDetails.Items.Count == 0)
            {
                CheckBoxListInjuryDetails.DataSourceID = "ObjectDataSourceInjuryDetailList";
                CheckBoxListInjuryDetails.DataBind();
            }

            if (DropDownListPolyTraumaVAMC.Items.Count == 1)
            {
                DropDownListPolyTraumaVAMC.DataSourceID = "ObjectDataSourcePolyTraumaVAMC";
                DropDownListPolyTraumaVAMC.DataBind();
            }

            if (DropDownListRehabDischargedTo.Items.Count == 1)
            {
                DropDownListRehabDischargedTo.DataSourceID = "ObjectDataSourceRehabDischargeTo";
                DropDownListRehabDischargedTo.DataBind();
            }

            if (CheckBoxListFollowUps.Items.Count == 0)
            {
                CheckBoxListFollowUps.DataSourceID = "ObjectDataSourceFollowUpList";
                CheckBoxListFollowUps.DataBind();
            }

            if (DropDownListEmergingEtiology.Items.Count == 0)
            {
                DropDownListEmergingEtiology.DataSourceID = "ObjectDataSourceEmergingEtiology";
                DropDownListEmergingEtiology.DataBind();

            }
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "PopulateDropDowns()");
        }
    }

    private void SetBreadCrumbLabelText(int? entryId)
    {
        try
        {
            SetFormToEditPolyTraumaHidePatientDetails(entryId.GetValueOrDefault());

            //Modify breadcrumb to display editing ID
            LabelBreadCrumb.Text = LabelBreadCrumb.Text + " > Edit Tracking Item ID : " + entryId;
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "SetBreadCrumbLabelText");
        } 
    }

    void Page_PreRender(object sender, EventArgs e)
    {
        if (polyTraumEntry != null)
        {
            //This is the last event in the page life cycle where changes can be made to the page object
            //so enforce business rules on the object (which could possibly change the propertie of the object
            //then load the form controls from the object

            //Enforce business rules on the referral object
            //need to call EnforceBusinessRules method in the ReferralManager because some rules require a database call
            //pass in the local instance of referral to the method.  The method will return the object with modifications due to business rules.
            if (IsPostBack)
            {
                int ret;
                ret = PolyTraumaManager.EnforceBusinessRules(ref polyTraumEntry);
            }

            //Load Controls from the Object
            LoadControlsFromObject();
        }

        if (!this.ReadOnly )
        {
            const string scriptName = "__polyTraumaClientSideScript";
            if (!Page.ClientScript.IsClientScriptIncludeRegistered(scriptName))
            {
                Page.ClientScript.RegisterClientScriptBlock(Page.GetType(), scriptName, GetPageJavaScript(),true);
            }
        }

        if (this.AllowUpdate)
        {
            ButtonSave.Visible = true;
        }
        else
        {
            ButtonSave.Visible = false;
        }
    }

    private string GetPageJavaScript()
    {

        StringBuilder myStringBuilder = new StringBuilder();
        try
        {
            myStringBuilder.Append("function SetTextBoxOtherInjuryDescriptionEnabledStatus()");
            myStringBuilder.Append("{");
            myStringBuilder.Append("var TextBoxOtherInjuryDescription = document.getElementById('" + TextBoxOtherInjuryDescription.ClientID + "');");
            myStringBuilder.Append("if ($get('" + DropDownListOtherInjury.ClientID + "').value  != 'Y')");
            myStringBuilder.Append("{");
            myStringBuilder.Append("TextBoxOtherInjuryDescription.value = '';");
            myStringBuilder.Append("TextBoxOtherInjuryDescription.disabled = true;");
            myStringBuilder.Append("}");
            myStringBuilder.Append("else");
            myStringBuilder.Append("{");
            myStringBuilder.Append("TextBoxOtherInjuryDescription.disabled = false;");
            myStringBuilder.Append("}");
            myStringBuilder.Append("}");
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "GetPageJavaScript()");
        } 
        return myStringBuilder.ToString();
     
    }

    private void SetFormToNewPatientSearchMode()
    {
        PanelSelectPatient.Visible = true;
        PanelManualPatientEntry.Visible = true;
        LabelOrMessage.Visible = true;
        PanelPatient.Visible = false;
        PanelPatientReadOnly.Visible = false;
        PanelTracking.Visible = false;
        PanelPatientExistsNotice.Visible = false;
        PanelSSNExists.Visible = false;

        SetDOBDODCompareValidators();
    }

    private void SetFormToNoInsertAllowedMode()
    {
        PanelNoInsertAllowed.Visible = true;

        PanelSelectPatient.Visible = false;
        PanelManualPatientEntry.Visible = false;
        LabelOrMessage.Visible = false;
        PanelPatient.Visible = false;
        PanelPatientReadOnly.Visible = false;
        PanelTracking.Visible = false;
        PanelPatientExistsNotice.Visible = false;
        PanelSSNExists.Visible = false;

        SetDOBDODCompareValidators();
    }

    private void SetFormToEditPolyTraumaHidePatientDetails(int entryID)
    {
        PanelSelectPatient.Visible = false;
        PanelManualPatientEntry.Visible = false;
        LabelOrMessage.Visible = false;
        PanelPatient.Visible = false;
        PanelPatientReadOnly.Visible = true;
        PanelTracking.Visible = true;
        PanelPatientExistsNotice.Visible = false;
        PanelSSNExists.Visible = false;

        SetDOBDODCompareValidators();

    }

    private void SetFormToEditPolyTraumaWithPatientDetails()
    {
        this.PanelPatient.Visible = true;
        this.PanelPatientReadOnly.Visible = false;
        this.PanelTracking.Visible = true;
        this.PanelSelectPatient.Visible = false;
        PanelManualPatientEntry.Visible = false;
        LabelOrMessage.Visible = false;

        PanelPatientExistsNotice.Visible = false;
        PanelSSNExists.Visible = false;

        SetDOBDODCompareValidators();
    }

    private void SetDOBDODCompareValidators()
    {
        try
        {
            if (this.PanelPatient.Visible)
            {
                //to do
                CompareValidatorDateAlteredConsciousnessDOB.ControlToCompare = "TextBoxDateOfBirth";
                CompareValidatorDateAlteredConsciousnessDOD.ControlToCompare = "TextBoxDateOfDeath";

                CompareValidatorPatientEmergedDateDOB.ControlToCompare = "TextBoxDateOfBirth";
                CompareValidatorPatientEmergedDateDOD.ControlToCompare = "TextBoxDateOfDeath";

                CompareValidatorRehabAdmitDateDOB.ControlToCompare = "TextBoxDateOfBirth";
                CompareValidatorRehabAdmitDateDOD.ControlToCompare = "TextBoxDateOfDeath";

                CompareValidatorRehabDischargeDateDOB.ControlToCompare = "TextBoxDateOfBirth";
                CompareValidatorRehabDischargeDateDOD.ControlToCompare = "TextBoxDateOfDeath";

            }
            else
            {
                CompareValidatorDateAlteredConsciousnessDOB.ControlToCompare = "HiddenFieldDateOfBirth";
                CompareValidatorDateAlteredConsciousnessDOD.ControlToCompare = "HiddenFieldDateOfDeath";

                CompareValidatorPatientEmergedDateDOB.ControlToCompare = "HiddenFieldDateOfBirth";
                CompareValidatorPatientEmergedDateDOD.ControlToCompare = "HiddenFieldDateOfDeath";

                CompareValidatorRehabAdmitDateDOB.ControlToCompare = "HiddenFieldDateOfBirth";
                CompareValidatorRehabAdmitDateDOD.ControlToCompare = "HiddenFieldDateOfDeath";

                CompareValidatorRehabDischargeDateDOB.ControlToCompare = "HiddenFieldDateOfBirth";
                CompareValidatorRehabDischargeDateDOD.ControlToCompare = "HiddenFieldDateOfDeath";


            }
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "SetDOBDODCompareValidators()");
        } 
    }

    private void DisplaySaveMessage(int returnSaveValue, int addNewTbiPolyTraumEntryId)
    {
        try
        {
            (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);

            HiddenField hiddenFieldShowSaveMessage =
                ((HiddenField)this.Master.FindControl("HiddenFieldShowSaveMessage"));
            hiddenFieldShowSaveMessage.Value = "true";

            if (returnSaveValue > 0)
            {
                IsSaveSuccess = true;

                if (addNewTbiPolyTraumEntryId == 0)
                {
                    //Response.Redirect(Request.Url.AbsolutePath + "?id=" + returnSaveValue);
                    //int? entryID = Convert.ToInt32(Request.QueryString["id"]);
                    //HiddenFieldEntryID.Value = entryID.GetValueOrDefault().ToString();
                    //LabelBreadCrumb.Text = LabelBreadCrumb.Text + " > Edit Tracking Item ID : " + entryID;
                    SetBreadCrumbLabelText(returnSaveValue);

                }
            }
            else
            {
                IsSaveSuccess = false;
            }
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "DisplaySaveMessage()");
        }
    }

    #endregion

    #region Object/Form Mapping

    private void LoadControlsFromObject()
    {
        try
        {
            //populate dropdowns
            //TO DO : modify drop down population to add archived item if persisted in entity 
            //          and remove "-- Select a Value --" when an item is selected and field is not nullable 
            PopulateDropDowns();

            //SSN
            TextBoxSSN.Text = polyTraumEntry.SSN;
            LabelSSNReadOnly.Text = polyTraumEntry.SSN;

            //First Name
            TextBoxFirstName.Text = polyTraumEntry.FIRST_NAME;

            //Middle Name
            TextBoxMiddleName.Text = polyTraumEntry.MIDDLE_NAME;

            //Last Name
            TextBoxLastName.Text = polyTraumEntry.LAST_NAME;

            //Label Full Name
            LabelFullName.Text = polyTraumEntry.LAST_NAME + ", " + polyTraumEntry.FIRST_NAME + (polyTraumEntry.MIDDLE_NAME == "" ? "" : " " + polyTraumEntry.MIDDLE_NAME);

            //Date of Birth
            TextBoxDateOfBirth.Text = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.BIRTH_DATE);
            HiddenFieldDateOfBirth.Text = TextBoxDateOfBirth.Text;
            //DateOfBirth.DateValue = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.BIRTH_DATE);

            //Gender
            DropDownListGender.SelectedValue = polyTraumEntry.STD_GENDER_ID.ToString();

            //Marital Status
            DropDownListMaritalStatus.SelectedValue = polyTraumEntry.STD_MARITALSTATUS_ID.ToString();

            //Race
            DropDownListRace.SelectedValue = polyTraumEntry.STD_RACE_ID.ToString();

            //Ethnicity
            DropDownListEthnicity.SelectedValue = polyTraumEntry.STD_ETHNICITY_ID.ToString();

            //Date of Death
            TextBoxDateOfDeath.Text = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.DEATH_DATE);
            HiddenFieldDateOfDeath.Text = TextBoxDateOfDeath.Text;
            //DateOfDeath.DateValue = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.DEATH_DATE);

            //Address
            TextBoxAddressLine1.Text = polyTraumEntry.ADDRESS_LINE1;

            //Address Line 2
            TextBoxAddressLine2.Text = polyTraumEntry.ADDRESS_LINE2;

            //Adrress Line3
            TextBoxAddressLine3.Text = polyTraumEntry.ADDRESS_LINE3;

            //City
            TextBoxCity.Text = polyTraumEntry.CITY;

            //State
            TextBoxState.Text = polyTraumEntry.STATE;

            //County
            TextBoxCounty.Text = polyTraumEntry.COUNTY;

            //Country
            TextBoxCountry.Text = polyTraumEntry.COUNTRY;

            //Postal Code
            TextBoxPostalCode.Text = polyTraumEntry.POSTAL_CODE;

            //Home Phone
            TextBoxHomePhone.Text = polyTraumEntry.HOME_PHONE;

            //Email Address
            TextBoxEmailAddress.Text = polyTraumEntry.EMAIL_ADDRESS;

            //Home VAMC
            DropDownListHomeVAMC.SelectedValue = polyTraumEntry.PATIENT_HOME_VAMC_ID.ToString();
            LabelHomeVAMCReadOnly.Text = PolyTraumaManager.getHomeVAMCName(polyTraumEntry.TBI_POLYTRAUMA_ID);

            //Military Service Branch
            DropDownListMilitartServiceBranch.SelectedValue = polyTraumEntry.STD_SERVICEBRANCH_ID.ToString();

            //Military Duty Status
            DropDownListMilitaryServiceStatus.SelectedValue = polyTraumEntry.STD_MILITARYDUTYSTS_ID.ToString();

            //Discharge Status
            DropDownListDischargeStatus.SelectedValue = polyTraumEntry.STD_TBIDSCHRGSTS_ID.ToString();

            //MTF / Referral Source
            DropDownListTBIFacilitySource.SelectedValue = polyTraumEntry.STD_TBIFACILITYSRC_ID.ToString();

            //Injury Date
            TextBoxInjuryDate.Text = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.TBI_INJURY_DATE);
            //InjuryDate.DateValue = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.TBI_INJURY_DATE);
            RangeValidatorTextBoxInjuryDate.MinimumValue = DateTime.MinValue.ToShortDateString();
            RangeValidatorTextBoxInjuryDate.MaximumValue = DateTime.Now.ToShortDateString();

            //Combat Location
            DropDownListCombatLocation.SelectedValue = polyTraumEntry.STD_COMBATLOCATION_ID.ToString();

            //Blast Explosion
            if (!string.IsNullOrEmpty(polyTraumEntry.BLAST_EXPLOSION_INJRY_IND))
            {
                DropDownListBlastExplosion.SelectedValue = polyTraumEntry.BLAST_EXPLOSION_INJRY_IND;
            }

            //Vehicle Injury
            if (!string.IsNullOrEmpty(polyTraumEntry.VEHICLE_INJRY_IND))
            {
                DropDownListVechicleInjury.SelectedValue = polyTraumEntry.VEHICLE_INJRY_IND;
            }

            //Bullet Injury
            if (!string.IsNullOrEmpty(polyTraumEntry.BULLET_INJRY_IND))
            {
                DropDownListBulletInjury.SelectedValue = polyTraumEntry.BULLET_INJRY_IND;
            }

            //Other Injury
            if (!string.IsNullOrEmpty(polyTraumEntry.OTHER_INJRY_IND))
            {
                DropDownListOtherInjury.SelectedValue = polyTraumEntry.OTHER_INJRY_IND;
            }

            //Other Injury Description
            if (DropDownListOtherInjury.SelectedValue == "Y")
            {
                TextBoxOtherInjuryDescription.Text = polyTraumEntry.OTHER_INJRY_DSCR;

                if (!this.ReadOnly)
                {
                    TextBoxOtherInjuryDescription.Enabled = true;
                }
            }
            else
            {
                //set other description to non editable
                TextBoxOtherInjuryDescription.Text = "";
                TextBoxOtherInjuryDescription.Enabled = false;
            }

            //Injury Details
            foreach (TBI_INJURY_DETAIL injury in polyTraumEntry.TBI_INJURY_DETAILs)
            {
                ListItem foundListItem = CheckBoxListInjuryDetails.Items.FindByValue(injury.STD_TBIINJURYLIST_ID.ToString());

                if (foundListItem != null)
                {
                    foundListItem.Selected = true;
                }
            }

            //Provider Location
            //DropDownListProviderLocation.SelectedValue = polyTraumEntry.STD_TBIPROVIDERLOC_ID.ToString();
            TextBoxFollowupProviderLocation.Text = polyTraumEntry.PROVIDER_LOCATION;

            //Provider ID Number
            //TextBoxProviderIDNumber.Text = polyTraumEntry.PROVIDER_ID_NUMBER;

            //Follow Up Polytrauma Case Manager
            TextBoxProviderName.Text = polyTraumEntry.PROVIDER_NAME;

            //Provider Email
            TextBoxProviderEmail.Text = polyTraumEntry.PROVIDER_EMAIL;

            //Provider Phone
            TextBoxProviderPhone.Text = polyTraumEntry.PROVIDER_PHONE;

            //PolyTrauma VAMC
            DropDownListPolyTraumaVAMC.SelectedValue = polyTraumEntry.POLYTRAUMA_CENTER_ID.ToString();

            //Is Patient Emerging Consciousness
            CheckBoxEmergingConsciousness.Checked = (polyTraumEntry.EMERGE_CONSCIOUSNESS_FLAG == 1 ? true : false);

            //Emerging Etiology
            DropDownListEmergingEtiology.SelectedValue = polyTraumEntry.STD_TBIEMERGINGETIOLOGY_ID.ToString();

            //Date Altered Consciousness
            TextBoxDateAlteredConsciousness.Text = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.ALTERED_CONSCIOUSNESS_DATE);
            //DateAlteredConsciousness.DateValue = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.ALTERED_CONSCIOUSNESS_DATE);
            RangeValidatorDateAlteredConsciousness.MinimumValue = DateTime.MinValue.ToShortDateString();
            RangeValidatorDateAlteredConsciousness.MaximumValue = DateTime.Now.ToShortDateString();

            //Date Patient Emerged
            //TextBoxPatientEmergedDate.Text = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.EMERGE_CONSCIOUSNESS_DATE);
            PatientEmergedDate.DateValue = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.EMERGE_CONSCIOUSNESS_DATE);

            //Admission Date
            //TextBoxRehabAdmitDate.Text = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.REHAB_ADMIT_DATE);
            RehabAdmitDate.DateValue = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.REHAB_ADMIT_DATE);

            //TextBoxDaysOffUnit
            TextBoxDaysOffUnit.Text = polyTraumEntry.DAYS_OFF_UNIT_TOTAL.ToString();

            //Admission FIM Code
            TextBoxAdmissionFIMScore.Text = polyTraumEntry.ADMISSION_FIM_CODE;

            //FRG Code
            DropDownListFRGCode.SelectedValue = polyTraumEntry.STD_TBI_FRG_ID.ToString();

            //Rehab Discharge Date
            //TextBoxRehabDischargeDate.Text = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.REHAB_DISCHARGE_DATE);
            RehabDischargeDate.DateValue = Helpers.getShortDateStringFromNullableDate(polyTraumEntry.REHAB_DISCHARGE_DATE);

            //Discharge FIM Code
            TextBoxDischargeFIMScore.Text = polyTraumEntry.DISCHARGE_FIM_CODE;

            //Discharged To
            DropDownListRehabDischargedTo.SelectedValue = polyTraumEntry.STD_TBIDISCHARGETO_ID.ToString();

            //Followup
            foreach (TBI_FOLLOWUP followup in polyTraumEntry.TBI_FOLLOWUPs)
            {
                ListItem foundListItem = CheckBoxListFollowUps.Items.FindByValue(followup.STD_TBIIFOLLOWUPLIST_ID.ToString());
                foundListItem.Selected = true;
            }

            //Followup Site Description
            TextBoxFollowupSiteDescription.Text = polyTraumEntry.FOLLOWUP_SITE_DSCR;

            //LOS
            LabelLOSValue.Text = polyTraumEntry.TotalLengthOfStay.ToString();

            //Age at Admission
            if (polyTraumEntry.AgeAtAdmission > 0)
            {
                LabelAgeAtAdmission.Text = polyTraumEntry.AgeAtAdmission.ToString();
            }
            else
            {
                LabelAgeAtAdmission.Text = "";
            }

            //Set Range Validators for Date Of Birth and Date Of Death
            SetDatesRangeValidators(true);
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "LoadControlsFromObject()");
        }
    }
    private void LoadObjectFromControls()
    {
        try
        {
            //SSN
            polyTraumEntry.SSN = TextBoxSSN.Text;

            //First Name
            polyTraumEntry.FIRST_NAME = TextBoxFirstName.Text;

            //Middle Name
            polyTraumEntry.MIDDLE_NAME = TextBoxMiddleName.Text;

            //Last Name
            polyTraumEntry.LAST_NAME = TextBoxLastName.Text;

            //Date of Birth
            polyTraumEntry.BIRTH_DATE = Helpers.getNullableDateFromString(TextBoxDateOfBirth.Text);
            //polyTraumEntry.BIRTH_DATE = Helpers.getNullableDateFromString(DateOfBirth.DateValue);

            //Gender
            polyTraumEntry.STD_GENDER_ID = Helpers.getNullableIntFromString(DropDownListGender.SelectedValue);

            //Marital Status
            polyTraumEntry.STD_MARITALSTATUS_ID = Helpers.getNullableIntFromString(DropDownListMaritalStatus.SelectedValue);

            //Race
            polyTraumEntry.STD_RACE_ID = Helpers.getNullableIntFromString(DropDownListRace.SelectedValue);

            //Ethnicity
            polyTraumEntry.STD_ETHNICITY_ID = Helpers.getNullableIntFromString(DropDownListEthnicity.SelectedValue);

            //Date of Death
            polyTraumEntry.DEATH_DATE = Helpers.getNullableDateFromString(TextBoxDateOfDeath.Text);
            //polyTraumEntry.DEATH_DATE = Helpers.getNullableDateFromString(DateOfDeath.DateValue);

            //Address
            polyTraumEntry.ADDRESS_LINE1 = TextBoxAddressLine1.Text;

            //Address Line 2
            polyTraumEntry.ADDRESS_LINE2 = TextBoxAddressLine2.Text;

            //Adrress Line3
            polyTraumEntry.ADDRESS_LINE3 = TextBoxAddressLine3.Text;

            //City
            polyTraumEntry.CITY = TextBoxCity.Text;

            //State
            polyTraumEntry.STATE = TextBoxState.Text;

            //County
            polyTraumEntry.COUNTY = TextBoxCounty.Text;

            //Country
            polyTraumEntry.COUNTRY = TextBoxCountry.Text;

            //Postal Code
            polyTraumEntry.POSTAL_CODE = TextBoxPostalCode.Text;

            //Home Phone
            polyTraumEntry.HOME_PHONE = TextBoxHomePhone.Text;

            //Email Address
            polyTraumEntry.EMAIL_ADDRESS = TextBoxEmailAddress.Text;

            //Home VAMC
            polyTraumEntry.PATIENT_HOME_VAMC_ID = Helpers.getIntFromString(DropDownListHomeVAMC.SelectedValue);

            //Military Service Branch
            polyTraumEntry.STD_SERVICEBRANCH_ID = Helpers.getNullableIntFromString(DropDownListMilitartServiceBranch.SelectedValue);

            //Military Duty Status
            polyTraumEntry.STD_MILITARYDUTYSTS_ID = Helpers.getNullableIntFromString(DropDownListMilitaryServiceStatus.SelectedValue);

            //Discharge Status
            polyTraumEntry.STD_TBIDSCHRGSTS_ID = Helpers.getNullableIntFromString(DropDownListDischargeStatus.SelectedValue);

            //MTF / Referral Source
            polyTraumEntry.STD_TBIFACILITYSRC_ID = Helpers.getNullableIntFromString(DropDownListTBIFacilitySource.SelectedValue);

            //Injury Date
            polyTraumEntry.TBI_INJURY_DATE = Helpers.getNullableDateFromString(TextBoxInjuryDate.Text);
            //polyTraumEntry.TBI_INJURY_DATE = Helpers.getNullableDateFromString(InjuryDate.DateValue);

            //Combat Location
            polyTraumEntry.STD_COMBATLOCATION_ID = Helpers.getNullableIntFromString(DropDownListCombatLocation.SelectedValue);

            //Blast Explosion
            polyTraumEntry.BLAST_EXPLOSION_INJRY_IND = Helpers.getNullableStringFromString(DropDownListBlastExplosion.SelectedValue);

            //Vehicle Injury
            polyTraumEntry.VEHICLE_INJRY_IND = Helpers.getNullableStringFromString(DropDownListVechicleInjury.SelectedValue);

            //Bullet Injury
            polyTraumEntry.BULLET_INJRY_IND = Helpers.getNullableStringFromString(DropDownListBulletInjury.SelectedValue);

            //Other Injury
            polyTraumEntry.OTHER_INJRY_IND = Helpers.getNullableStringFromString(DropDownListOtherInjury.SelectedValue);

            //Other Injury Description
            polyTraumEntry.OTHER_INJRY_DSCR = TextBoxOtherInjuryDescription.Text;

            //Injury Details
            foreach (ListItem checkboxListInjury in CheckBoxListInjuryDetails.Items)
            {
                //for each item in the check box list check if it is an entity in TBI_INJURY_DETAILs
                TBI_INJURY_DETAIL existingInjury = (from e in polyTraumEntry.TBI_INJURY_DETAILs
                                                    where e.STD_TBIINJURYLIST_ID == Helpers.getIntFromString(checkboxListInjury.Value)
                                                    select e).FirstOrDefault();

                //check if the checkboxlist item was selected,  
                if (checkboxListInjury.Selected)
                {
                    //if the item is selected then it needs to be in TBI_INJURY_DETAILs
                    //if the entity exists, that signifies the item is already in TBI_INJURY_DETAILs therefore nothing needs to be done. 
                    if (existingInjury == null)
                    {
                        TBI_INJURY_DETAIL newInjury = new TBI_INJURY_DETAIL();
                        newInjury.TBI_POLYTRAUMA_ID = polyTraumEntry.TBI_POLYTRAUMA_ID;
                        newInjury.STD_TBIINJURYLIST_ID = Helpers.getIntFromString(checkboxListInjury.Value);

                        polyTraumEntry.TBI_INJURY_DETAILs.Add(newInjury);
                    }
                }
                else
                {
                    //if the item is not selected, then it needs to be taken out of TBI_INJURY_DETAILs if it is in the list
                    if (existingInjury != null)
                    {
                        existingInjury.SetAsDeleteOnSubmit();
                        //polyTraumEntry.TBI_INJURY_DETAILs.Remove(existingInjury);
                    }
                }
            }

            //Provider Location
            //polyTraumEntry.STD_TBIPROVIDERLOC_ID = Helpers.getNullableIntFromString(DropDownListProviderLocation.SelectedValue);
            polyTraumEntry.PROVIDER_LOCATION = TextBoxFollowupProviderLocation.Text;

            //Follow Up Polytrauma Case Manager
            polyTraumEntry.PROVIDER_NAME = TextBoxProviderName.Text;

            //Provider Email
            polyTraumEntry.PROVIDER_EMAIL = TextBoxProviderEmail.Text;

            //Provider Phone
            polyTraumEntry.PROVIDER_PHONE = TextBoxProviderPhone.Text;

            //PolyTrauma VAMC
            polyTraumEntry.POLYTRAUMA_CENTER_ID = Helpers.getIntFromString(DropDownListPolyTraumaVAMC.SelectedValue);

            //Is Patient Emerging Consciousness
            polyTraumEntry.EMERGE_CONSCIOUSNESS_FLAG = (CheckBoxEmergingConsciousness.Checked == true ? 1 : 0);

            //Emerging Etiology
            polyTraumEntry.STD_TBIEMERGINGETIOLOGY_ID = Helpers.getNullableIntFromString(DropDownListEmergingEtiology.SelectedValue);

            //Date Altered Consciousness
            polyTraumEntry.ALTERED_CONSCIOUSNESS_DATE = Helpers.getNullableDateFromString(TextBoxDateAlteredConsciousness.Text);
            //polyTraumEntry.ALTERED_CONSCIOUSNESS_DATE = Helpers.getNullableDateFromString(DateAlteredConsciousness.DateValue);

            //Date Patient Emerged
            //polyTraumEntry.EMERGE_CONSCIOUSNESS_DATE = Helpers.getNullableDateFromString(TextBoxPatientEmergedDate.Text);
            polyTraumEntry.EMERGE_CONSCIOUSNESS_DATE = Helpers.getNullableDateFromString(PatientEmergedDate.DateValue);

            //Admission Date
            polyTraumEntry.REHAB_ADMIT_DATE = Helpers.getNullableDateFromString(RehabAdmitDate.DateValue);

            //TextBoxDaysOffUnit
            polyTraumEntry.DAYS_OFF_UNIT_TOTAL = Helpers.getNullableIntFromString(TextBoxDaysOffUnit.Text);

            //Admission FIM Code
            polyTraumEntry.ADMISSION_FIM_CODE = TextBoxAdmissionFIMScore.Text;

            //FRG Code
            polyTraumEntry.STD_TBI_FRG_ID = Helpers.getNullableIntFromString(DropDownListFRGCode.SelectedValue);

            //Rehab Discharge Date
            //polyTraumEntry.REHAB_DISCHARGE_DATE = Helpers.getNullableDateFromString(TextBoxRehabDischargeDate.Text);
            polyTraumEntry.REHAB_DISCHARGE_DATE = Helpers.getNullableDateFromString(RehabDischargeDate.DateValue);

            //Discharge FIM Code
            polyTraumEntry.DISCHARGE_FIM_CODE = TextBoxDischargeFIMScore.Text;

            //Discharged To
            polyTraumEntry.STD_TBIDISCHARGETO_ID = Helpers.getNullableIntFromString(DropDownListRehabDischargedTo.SelectedValue);

            //Followup
            foreach (ListItem checkboxListFollowup in CheckBoxListFollowUps.Items)
            {

                //for each item in the check box list check if it is an entity in TBI_FOLLOWUPs
                TBI_FOLLOWUP existingFollowup = (from e in polyTraumEntry.TBI_FOLLOWUPs
                                                 where e.STD_TBIIFOLLOWUPLIST_ID == Helpers.getIntFromString(checkboxListFollowup.Value)
                                                 select e).FirstOrDefault();

                //check if the checkboxlist item was selected,  
                if (checkboxListFollowup.Selected)
                {
                    //if the item is selected then it needs to be in TBI_FOLLOWUPs
                    //if the entity exists, that signifies the item is already in TBI_FOLLOWUPs therefore nothing needs to be done. 
                    if (existingFollowup == null)
                    {
                        TBI_FOLLOWUP newFollowup = new TBI_FOLLOWUP();
                        newFollowup.TBI_POLYTRAUMA_ID = polyTraumEntry.TBI_POLYTRAUMA_ID;
                        newFollowup.STD_TBIIFOLLOWUPLIST_ID = Helpers.getIntFromString(checkboxListFollowup.Value);

                        polyTraumEntry.TBI_FOLLOWUPs.Add(newFollowup);
                    }
                }
                else
                {
                    //if the item is not selected, then it needs to be taken out of TBI_FOLLOWUPs if it is in the list
                    if (existingFollowup != null)
                    {
                        existingFollowup.SetAsDeleteOnSubmit();
                        //polyTraumEntry.TBI_FOLLOWUPs.Remove(existingFollowup);
                    }
                }
            }

            //Followup Site Description
            polyTraumEntry.FOLLOWUP_SITE_DSCR = TextBoxFollowupSiteDescription.Text;

            //Set Range Validators for Date Of Birth and Date Of Death
            SetDatesRangeValidators(false);
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "LoadObjectsFromControl()");
        }
    }

    private void TransferPatientInformationToNewPolyTraumaEntry(PATIENT patientEntry)
    {
        try
        {
            //Patient ID
            polyTraumEntry.PATIENT = patientEntry;
            HiddenFieldPatientID.Value = patientEntry.PATIENT_ID.ToString();

            //SSN
            polyTraumEntry.SSN = patientEntry.SSN;

            //First Name
            polyTraumEntry.FIRST_NAME = patientEntry.FIRST_NAME;

            //Middle Name
            polyTraumEntry.MIDDLE_NAME = patientEntry.MIDDLE_NAME;

            //Last Name
            polyTraumEntry.LAST_NAME = patientEntry.LAST_NAME;

            //Date of Birth
            polyTraumEntry.BIRTH_DATE = patientEntry.BIRTH_DATE;

            //Gender
            //polyTraumEntry.STD_GENDER = PolyTraumaPatientManager.getGenderEntityFromCode(patientEntry.GENDER_CODE);
            polyTraumEntry.STD_GENDER_ID = patientEntry.STD_GENDER_ID;

            //Marital Status
            //polyTraumEntry.STD_MARITALSTATUS = PolyTraumaPatientManager.getMaritalStatusEntityFromCode(patientEntry.MARITAL_STATUS_CODE);
            polyTraumEntry.STD_MARITALSTATUS_ID = patientEntry.STD_MARITALSTATUS_ID;

            //Race
            //polyTraumEntry.STD_RACE = PolyTraumaPatientManager.getRaceEntityFromCode(patientEntry.RACE_CODE);
            polyTraumEntry.STD_RACE_ID = patientEntry.STD_RACE_ID;

            //Ethnicity
            //polyTraumEntry.STD_ETHNICITY = PolyTraumaPatientManager.getEthnicityEntityFromCode(patientEntry.ETHNICITY_CODE);
            polyTraumEntry.STD_ETHNICITY_ID = patientEntry.STD_ETHNICITY_ID;

            //Date of Death
            polyTraumEntry.DEATH_DATE = patientEntry.DEATH_DATE;

            //Address
            polyTraumEntry.ADDRESS_LINE1 = patientEntry.ADDRESS_LINE1;

            //Address Line 2
            polyTraumEntry.ADDRESS_LINE2 = patientEntry.ADDRESS_LINE2;

            //Adrress Line3
            polyTraumEntry.ADDRESS_LINE3 = patientEntry.ADDRESS_LINE3;

            //City
            polyTraumEntry.CITY = patientEntry.CITY;

            //State
            polyTraumEntry.STATE = patientEntry.STATE;

            //County
            polyTraumEntry.COUNTY = patientEntry.COUNTY;

            //Country
            polyTraumEntry.COUNTRY = patientEntry.COUNTRY;

            //Postal Code
            polyTraumEntry.POSTAL_CODE = patientEntry.POSTAL_CODE;

            //Home Phone
            polyTraumEntry.HOME_PHONE = patientEntry.HOME_PHONE;

            //Email Address
            polyTraumEntry.EMAIL_ADDRESS = patientEntry.EMAIL_ADDRESS;

            //Home VAMC
            polyTraumEntry.PATIENT_HOME_VAMC_ID = Helpers.getIntFromString(patientEntry.STD_INSTITUTION_ID.ToString());

            //Military Service Branch
            //polyTraumEntry.STD_SERVICEBRANCH = PolyTraumaPatientManager.getServiceBranchEntityFromCode(patientEntry.SERVICE_BRANCH_CODE);
            polyTraumEntry.STD_SERVICEBRANCH_ID = patientEntry.STD_SERVICEBRANCH_ID;
        }
        catch (Exception _ex)
        {
            _tbiexception.HandleTBIException(_ex, "PolyTraumaEntry.aspx", "100004", "TransferPatientInformationToNewPolyTraumaEntry()");
        }
    }

    #endregion

}
